C
C****** program package for V n-parton processes, n=3,4,5 *************
C
C  Authors: Dieter Zeppenfeld, W.Long, T.Han, Dorothea Zeppenfeld
C  Original version: May 5, 1989
C  Latest revisions: June 5, 2001
C
C  Modified to use real*8 and complex*16 variables.
C  Constants were also converted to double precision.
C  Adam Duff, 1990 October 2
C
C   bug in HELVEC for KT=0 corrected Nov.4, 1993  (D.Z.)
C
C  This version includes massive leptons in the photon/Z current
C  and is suited for photoproduction at very small Q**2.
c
c  inclusion of color flow information via common blocks
c  /cvqqgg/, /cvqqqq/, /cv2q3g/, /cv4q1g/ for 4 and 5 parton amplitudes
c  Dieter Zeppenfeld,  February 15, 2000
c
c  Color flow information is also provided via the additional entries
c  RVcQQGG, LVcQQGG etc which give the output in the format RV(ic,ifl,i3) etc.
c  where ifl labels quark flavors and i3 = 0,...,6 labels possible color flows
C  See subroutines RVQQGG etc for details
c  Dieter Zeppenfeld,  February 22, 2001. 
C
C  The package contains the main subroutines
C
C   RVQQG            with entries  RWQQG,  LVQQG,  LWQQG
c                                  RWcQQG, LVcQQG, LWcQQG
C   RVQQGG           with entries  RWQQGG, LVQQGG, LWQQGG
c                                  RWcQQGG,LVcQQGG,LWcQQGG
C   RV2Q3G           with entries  RW2Q3G, LV2Q3G, LW2Q3G
c                                  RVc2Q3G,LVc2Q3G,LWc2Q3g
C   RVQQQQ           with entries  RWQQQQ, LVQQQQ, LWQQQQ
c                                  RWcQQQQQ,LVcQQQQ,LWcQQQQ
C   RV4Q1G           with entries  RW4Q1G, LV4Q1G, LW4Q1G
c                                  RWc4Q1G,LVc4Q1G,LWc4Q1G
C
c   RWcQQG, ect. carry color flow info. 
c
C  plus a number of subroutines called by these five.
C  The code is based on
C
C     Hagiwara, Zeppenfeld: Nucl.Phys. B313 (1989) 560
C
C  hereafter called HZ. The main subroutines calculate the
C  color summed amplitude square for fixed fermion and boson helicities.
C  RV... and RW... are for real (onshell) photon/Z and real W
C  respectively.
C  For offshell Z/photon and W coupled to a leptonic current (e.g.
C  in Z or W decay) the entries LV... and LW... have to be called.
C
C
C  Helicity/polarization and color summed amplitudes squared are
C  provided instead by calling the subroutines
C
C   HSRW2Q           with entries  HSLW2Q
C   HSRV2Q           with entries  HSLV2Q
C
C  for processes involving 2 quarks and NG=1,2,3 gluons, and
C
C   SRW4Q            with entries  SLW4Q, SRW4QG, SLW4QG
C   SRV4Q            with entries  SLV4Q, SRV4QG, SLV4QG
C
C  for processes involving 4 quarks and NG=0,1 gluons.
C----------------------------------------------------------------
C  Helicity/polarization summed squared amplitudes are provided by
c  calling the subroutines
c  
c     HcSRWQ        with entries HcSRW2Q
c     HCSRVQ        with entries HCSRV2Q
c  for processes involving 2 quarks and NG = 1,2,3 gluons, and
c     
C   ScRW4Q            with entries  ScLW4Q, ScRW4QG, ScLW4QG
C   ScRV4Q            with entries  ScLV4Q, ScRV4QG, ScLV4QG
C
C  for processes involving 4 quarks and NG=0,1 gluons.
C  INPUT and OUTPUT for the subroutines is standardized:
C
C
C  INPUT :
C  --------
C
C  PBAR(0:3,NF)      The physical momenta of the NF fermions,
C                    i.e. PBAR(0,*),the energy, is assumed positive.
C
C  FSIGN(NF)         The sign factors S_i of table 5 or 6 in HZ
C
C  FPOL(NF)          Physical helicity of the fermion. An integer +1 or
C                    -1 is assumed
C
C
C  QBAR(0:4,NG)      Physical momentum of the gluon. The fourth
C                    component is the gluon mass (0). NG is the number
C                    of gluons in a given amplitude.
C
C  GSIGN(NG)         Sign factor g_i of table 5 or 6 in HZ
C
C  GPOL(NG)          Rectangular polarisation index for the gluon.
C                    GPOL(*) = 1 or 2 is assumed.
C
C
C  BOS               Identifier of the electroweak boson V.
C                    The convention is
C                           1: Photon
C                           2: Z0 Boson
C                           3: W+ Boson
C                           4: W- Boson
C                           10: Virtual Z and photon
C                           30: Virtual W+
C                           40: Virtual W-
C
C                    This information is used to decide which fermion
C                    line(s) to radiate off the boson V, and to obtain
C                    appropriate Vqq couplings.
C
C  KBAR(0:4)         Physical momentum of the vector boson V (KBAR(mu))
C  LBAR(0:3,2)       or physical momenta of the two massless leptons
C                    coupling to the electroweak boson V for BOS >= 10.
C                    The fourth copmponent of KBAR stores the V mass
C                    ( KBAR(4)=sqrt(kbar**2) ).
C
C  KSIGN             Sign factor for boson momentum
C  LSIGN(2)          Sign factor for leptons coupled to V.
C
C  VPOL              Index for polarisation index of outgoing vector
C                    boson V. Integers 1,2,3 for the rectangular
C                    polarisation basis are expected.
C  LPOL              Physical helicity of the massless leptons coupled
C                    to V.
C
C  LFLAVR(2)         Flavor of leptons coupled to decaying electroweak
C                    boson. The flavor indices are two digit numbers.
C                    first digit is  1: neutrino of electron (muon,tau)
C                                    2: electron (muon,tau)
C                                    3: up quark (charm,top)
C                                    4: down quark (strange,bottom)
C                    second digit is the generation index
C                                    1: electron family
C                                    2: muon family
C                                    3: tau family
C
C
C  OUTPUT:
C  -------
C
C  RVUU(2),RVDD(2),      Color sum of the amplitude squared for fixed
C  RVUUDD(2), etc.       polarisations of all external particles.
C  RWUD,RWUDCC etc.      The second letter indicates whether the BOSon
C                        is a real or virtual Z/photon (=V) or a W.
C                        Letters 3 through 6 give the flavors of the
C                        quarks assumed for a particular
C                        matrixelement**2.
C                        For the photon/Z case 2 values are given
C                        for each flavor combination:
C                        1: the basic amplitude**2 as given in HZ
C                        2: the amplitude**2 of the corresponding
C                        charge conjugation transformed reaction, i.e.
C                        for quarks replaced by antiquarks of the SAME
C                        physical helicity and momentum.
C
C                        For more information, in particular in the
C                        case of 4-quark amplitudes, see the individual
C                        subroutines.
C
C
C  Selected local variables:
C  -------------------------
C
C  JVLL(0:3)      "Decay current" of the virtual electroweak boson V
C                    as in eq.2.20 of HZ, except for the coupling
C                    constant and boson propagator factor, which is
C                    stored in
C  CORFAC(NF,3:4,2)  The time component of JVLL is eliminated
C                    by using current conservation, which is valid for
C                    MASSLESS QUARKS. CORFAC contains the appropriate
C                    correction factor including coupling costants to
C                    u- and d-type quarks (second index = 3 and 4,
C                    respectively) and for photon/Z exchange the
C                    coupling constants for the C related process are
C                    stored in the array with third index = 2.
C
C
C  BRAKET(2,NF,NG+1) Contains the ket |q,i> for odd and the bra <i,q|
C                    for all even fermion indices (NF) and all external
C                    gluons (first NG of third index) and the vector
C                    boson V (third index = NG+1 ).
C
C
C************ RVQQG ****************************************************
C
      SUBROUTINE RVQQG(PBAR,FSIGN,FPOL,
     1                QBAR,GSIGN,GPOL,
     2            BOS,KBAR,KSIGN,VPOL,
     3                RVUU,RVDD)
      IMPLICIT NONE
      INTEGER NF, NG, NF4, NGPL1, NF48
      PARAMETER (NF=2, NG=3, NF4=NF*4,NF48=NF*48,NGPL1=NG+1)
C          
C  RVQQG calculates the color-summed amplitude squared of the Vqqg
C  amplitude (eqns. 2.5 of HZ ) for fixed fermion helicities
C  and fixed V and gluon polarizations. The cartesian polarization basis 
C  is chosen for V and gluons. All fermions are assumed to be massless,
C  except in the case BOS=10 and LFLAVR=21,22,23, i.e. for NC processes with
C  incoming charged leptons. In this case electron mass effects are included
C  in order to allow a reliable simulation of ep scattering at extremely
C  low Q**2 ( << m_e**2 ).
C
C  5 different channels are considered:
C     CHANEL = 1:   real photon or Z
C     CHANEL = 2:   real W+ or W-
C     CHANEL = 3:   virtual photon and Z coupled to e+e-
C     CHANEL = 4:   virtual photon and Z coupled to nu nu-bar
C     CHANEL = 5:   virtual W+- coupled to nu e
C
C
      real*8 PBAR(0:3,NF), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2)
      real*8 P   (0:3,NF), Q   (0:4,NG), K   (0:4), L   (0:3,2)
C
      INTEGER FSIGN(NF), FPOL(NF), SIGMA(NF),
     1        GSIGN(NG), GPOL(NG),
     2        BOS, KSIGN, VPOL, LSIGN(2), LPOL(2), LFLAVR(2)
C
      real*8 CSAM2, RVUU(2), RVDD(2), RWUD, F2HAT, F2HAT2,
     1     EPSV(0:3), EPSG(0:3,NG), JVXYZ(0:3,3), OMFACF, DUMMY(0:4),
     2     RV(2,2,0:1),RW(0:1)   
      complex*16  CHIF(2,NF), JVLL(0:3), CORFAC(NF,3:4,2),
     1         BRAKET(2,NF,NGPL1), JV10(0:3,NF,3:4,2,-1:1), 
     2         AMPLIT, AM10(-1:1,3:4,2)
      REAL*8 AM10SQ(3:4,2)
C
C  Further local variables
      INTEGER IFL, MU, IC, IL, I3, CHANEL
      complex*16  Z1,Z2,Z3,Z4,ZJ0,ZJ1,ZJ2,ZJ3,Z3M4
C
      logical lcol
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
C
      DATA JVLL /4*(0.0d0, 0.0d0)/, CORFAC /NF4*(0.0d0, 0.0d0)/
      DATA JVXYZ /0D0,1D0,0D0,0D0,  0D0,0D0,1D0,0D0,  0D0,0D0,0D0,1D0/  
      DATA JV10 /NF48*(0D0,0D0)/
      RVUU(1) = 0.0d0
      RVUU(2) = 0.0d0
      RVDD(1) = 0.0d0
      RVDD(2) = 0.0d0
      IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
        CHANEL = 1
      ELSE 
        CHANEL = -10
      ENDIF
      lcol = .false.
      GOTO 10
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    RV(ic,ifl,i3) ic= 1,2 charge conjugation
c                  ifl = 1,2 flavour
c                  i3 = 0 color sum 1,2 color structures
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry RVcQQG(PBAR,FSIGN,FPOL,
     1             QBAR,GSIGN,GPOL,
     2             BOS,KBAR,KSIGN,VPOL,
     3             RV)
      Do ic = 1,2
         do ifl = 1,2
               rv(ic,ifl,0) = 0d0  
               rv(ic,ifl,1) = 0d0  
         enddo
      enddo
      IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
        CHANEL = 1
      ELSE 
        CHANEL = -10
      ENDIF
      lcol = .true.
      goto 10
C
c
      ENTRY RWcQQG(PBAR,FSIGN,FPOL,
     1            QBAR,GSIGN,GPOL,
     2        BOS,KBAR,KSIGN,VPOL,
     3            RW)
      RW(0) = 0.0d0
      RW(1) = 0.0d0
      IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
        CHANEL = 2
      ELSE 
        CHANEL = -20
      ENDIF 
      lcol = .true.
      goto 10
c
      ENTRY RWQQG(PBAR,FSIGN,FPOL,
     1            QBAR,GSIGN,GPOL,
     2        BOS,KBAR,KSIGN,VPOL,
     3            RWUD)
      RWUD = 0.0d0
      IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
        CHANEL = 2
      ELSE 
        CHANEL = -20
      ENDIF
      lcol = .false.
C
      
 10   CONTINUE
C
C
C  Get polarisation vector, momentum K and Vqq coupling for onshell
C     boson  V
      SIGMA(1) = FSIGN(1)*FPOL(1)
      SIGMA(2) = FSIGN(2)*FPOL(2)
      IF (SIGMA(1).NE.SIGMA(2)) THEN
        WRITE(*,*) ' Helicity nonconserving amplitude called in VQQG '
        RETURN
      ENDIF
      CALL POLVEC(KBAR,VPOL,EPSV)
      K(0) = KSIGN*KBAR(0)
      DO 20 MU = 1,3
        K(MU) = KSIGN*KBAR(MU)
 20     JVLL(MU) = EPSV(MU)
      K(4) = KBAR(4)**2
      DO 30 IFL = 3,4
        CORFAC(1,IFL,1) = CLR(IFL,BOS,SIGMA(1))
 30     CORFAC(1,IFL,2) = CLR(IFL,BOS,-SIGMA(1))
C
      GOTO 40
C
C  Now the entry for offshell electroweak bosons
      ENTRY LVQQG(PBAR,FSIGN,FPOL,
     1           QBAR,GSIGN,GPOL,
     2           BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3           RVUU,RVDD)
      RVUU(1) = 0.0d0
      RVUU(2) = 0.0d0
      RVDD(1) = 0.0d0
      RVDD(2) = 0.0d0
C      IF (BOS.EQ.10 .AND. LFLAVR(1)/10.EQ.2 .AND. LFLAVR(2)/10.EQ.2 ) THEN
      IF (BOS.EQ.10 .AND. LFLAVR(1)/10.EQ.20 .AND. LFLAVR(2)/10.EQ.20 ) THEN
        CHANEL = 3
      ELSEIF (BOS.EQ.10) THEN 
        CHANEL = 4
      ELSE 
        CHANEL = -30
      ENDIF
      lcol = .false. 
      GOTO 50
c
      ENTRY LVcQQG(PBAR,FSIGN,FPOL,
     1           QBAR,GSIGN,GPOL,
     2           BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3           RV)

      Do ic = 1,2
         do ifl = 1,2
            rv(ic,ifl,0) = 0d0  
            rv(ic,ifl,1) = 0d0  
         enddo
      enddo

C      IF (BOS.EQ.10 .AND. LFLAVR(1)/10.EQ.2 .AND. LFLAVR(2)/10.EQ.2 ) THEN
      IF (BOS.EQ.10 .AND. LFLAVR(1)/10.EQ.20 .AND. LFLAVR(2)/10.EQ.20 ) THEN
        CHANEL = 3
      ELSEIF (BOS.EQ.10) THEN 
        CHANEL = 4
      ELSE 
        CHANEL = -30
      ENDIF
      lcol = .true. 
      GOTO 50
C
      ENTRY LWQQG(PBAR,FSIGN,FPOL,
     1           QBAR,GSIGN,GPOL,
     2           BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3           RWUD)
      RWUD = 0.0d0
      IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
        CHANEL = 5
      ELSE 
        CHANEL = -50
      ENDIF
      lcol = .false.
      goto 50

      ENTRY LWcQQG(PBAR,FSIGN,FPOL,
     1           QBAR,GSIGN,GPOL,
     2           BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3           RW)
      RW(0) = 0.0d0
      RW(1) = 0.0d0
      IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
        CHANEL = 5
      ELSE 
        CHANEL = -50
      ENDIF
      lcol = .true.
      goto 50

 50   CONTINUE
C
      SIGMA(1) = FSIGN(1)*FPOL(1)
      SIGMA(2) = FSIGN(2)*FPOL(2)
      IF (SIGMA(1).NE.SIGMA(2)) THEN
        WRITE(*,*) ' Helicity nonconserving amplitude called in VQQG '
        RETURN
      ENDIF
C
C  Get the decay current JVLL
      IF ( CHANEL.EQ.3 ) THEN
        CALL DECAYN(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMA,JV10,K)
      ELSE
        CALL DECAYC(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMA,BOS,
     *            JVLL,CORFAC,K)
      ENDIF
C
C  Now virtual and real bosons merge again
 40   CONTINUE
C
C  Get the unbared quark and gluon momenta
      DO 60 MU = 0,3
        P(MU,1) = FSIGN(1)*PBAR(MU,1)
        P(MU,2) = FSIGN(2)*PBAR(MU,2)
 60     Q(MU,1) = GSIGN(1)*QBAR(MU,1)
      Q(4,1) = 0D0
C
C  Get the gluon polarisation vector and the 2 quark spinors CHIF
      CALL POLVEC(QBAR(0,1),GPOL(1),EPSG(0,1))
      CALL CHIOM(NF,PBAR,SIGMA,CHIF,OMFACF)
C
C  Get the bra <2,q| and the ket |q,1>
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),Q(0,1),EPSG(0,1),
     *           BRAKET(1,1,1),DUMMY)
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),Q(0,1),EPSG(0,1),
     *           BRAKET(1,2,1),DUMMY)
C
C  Form the current SIGMA(1)*( <2,q|mu|1>+<2|mu|q,1> ).......
      Z1 = BRAKET(1,2,1)*CHIF(1,1) + CHIF(1,2)*BRAKET(1,1,1)
      Z2 = BRAKET(2,2,1)*CHIF(2,1) + CHIF(2,2)*BRAKET(2,1,1)
      Z3 = BRAKET(1,2,1)*CHIF(2,1) + CHIF(1,2)*BRAKET(2,1,1)
      Z4 = BRAKET(2,2,1)*CHIF(1,1) + CHIF(2,2)*BRAKET(1,1,1)
      ZJ1  = Z3+Z4
      Z3M4 = Z3-Z4
      ZJ2  = dcmplx( dimag(Z3M4),-dble(Z3M4))
      ZJ3  = Z1-Z2
C
C  ......and contract it with the decay current
      IF (CHANEL.NE.3) THEN
        AMPLIT =  -ZJ1*JVLL(1)-ZJ2*JVLL(2)-ZJ3*JVLL(3)
C
C  Get overall factor F2, forget about sign factor 
C  SIGMA(1)*FSIGN(1)*FSIGN(2)
        F2HAT = CLR(3,5,1)*OMFACF
C
C  ... and now the final answer. The factor 4. is the color factor
C  (N**2-1)/2 of eqn.3.3 in HZ. Multiply with appropriate correction
C     factors here.
        CSAM2 = 4.0d0 * ( dble(AMPLIT)**2+dimag(AMPLIT)**2 )*F2HAT**2
C
        IF (BOS.EQ.1 .OR. BOS.EQ.2 .OR. BOS.EQ.10) THEN
          DO 70 IC = 1,2
             if (lcol) then               
                RV(IC,2,0) = CSAM2 *
     1         ( dble(CORFAC(1,3,IC))**2+dimag(CORFAC(1,3,IC))**2 )
                RV(IC,1,0) = CSAM2 *
     1         ( dble(CORFAC(1,4,IC))**2+dimag(CORFAC(1,4,IC))**2 )
                do ifl = 1,2 
                   rv(ic,ifl,1)=rv(ic,ifl,0)
                enddo
             else   
                RVUU(IC) = CSAM2 *
     1         ( dble(CORFAC(1,3,IC))**2+dimag(CORFAC(1,3,IC))**2 )
                RVDD(IC) = CSAM2 *
     1         ( dble(CORFAC(1,4,IC))**2+dimag(CORFAC(1,4,IC))**2 )
             endif
 70       CONTINUE
        ELSE
           if(lcol) then
              rw(0)  = CSAM2 *
     1             ( dble(CORFAC(1,3,1))**2+dimag(CORFAC(1,3,1))**2 )
              rw(1) = rw(0)
           else
              RWUD  = CSAM2 *
     1             ( dble(CORFAC(1,3,1))**2+dimag(CORFAC(1,3,1))**2 )
           ENDIF
        endif
      ELSE
C
C  Get overall factor F2**2 * color factor 4
        F2HAT2 = (CLR(3,5,1) * OMFACF)**2 *4D0
C  Now contract with the lepton current JV10
        DO 80 IC = 1,2 
        DO 80 IFL = 3,4
          AM10SQ(IFL,IC) = 0D0
          DO 81 IL = -1,1,2
            AM10(IL,IFL,IC) = -ZJ1*JV10(1,1,IFL,IC,IL)-
     2                         ZJ2*JV10(2,1,IFL,IC,IL)-
     3                         ZJ3*JV10(3,1,IFL,IC,IL)
            AM10SQ(IFL,IC) = AM10SQ(IFL,IC) +
     1          (DBLE(AM10(IL,IFL,IC))**2+DIMAG(AM10(IL,IFL,IC))**2)
 81      CONTINUE
          AM10SQ(IFL,IC) = AM10SQ(IFL,IC) * F2HAT2
 80    CONTINUE

C
C  And the final answer is
       if (lcol) then 
           DO 90 IC = 1,2
              RV(IC,2,1) = AM10SQ(3,IC)
              RV(IC,1,1) = AM10SQ(4,IC)
 90        CONTINUE
       else
           DO 91 IC = 1,2
              RVUU(IC) = AM10SQ(3,IC)
              RVDD(IC) = AM10SQ(4,IC)
 91        CONTINUE
       endif
      ENDIF
C
      RETURN
      END
C
C************ RVQQGG ***************************************************
C    checked June 1, 88: Lorentz and QCD gauge invariance
C
      SUBROUTINE RVQQGG(PBAR,FSIGN,FPOL,
     1                 QBAR,GSIGN,GPOL,
     2             BOS,KBAR,KSIGN,VPOL,
     3             RVUU,RVDD)
      implicit none

      INTEGER NF, NG, NF4, NGPL1
      real*8 CPLUS, CMINUS
      PARAMETER (NF=2, NG=2, NF4=NF*4,NGPL1=NG+1)
      PARAMETER (CPLUS=2.3333333333333d0, CMINUS=3.0d0)
C
C  RVQQGG calculates the color-summed amplitude squared of the Vqqgg
C  amplitude (eqns. 4.11 to 4.15 of HZ ) for fixed fermion helicities
C  and fixed V and gluon polarization. The cartesian polarization
C  basis is chosen for V and gluons. All fermions are assumed to be
C  massless.
C
C
      real*8 PBAR(0:3,NF), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2)
      real*8 P   (0:3,NF), Q   (0:4,NG), K   (0:4), L   (0:3,2)
C
      INTEGER FSIGN(NF), FPOL(NF), SIGMAF(NF),
     1        GSIGN(NG), GPOL(NG),
     2        BOS, KSIGN, VPOL, LSIGN(2), LPOL(2), LFLAVR(2)
C
      real*8 CSAM2, RVUU(2), RVDD(2), rwud, RV(2,2,0:2), RW(0:2),
     1     EPSG(0:3,NG), EPSV(0:3), OMFACF, DUMMY(0:4)
      complex*16  CHIF(2,NF), JVLL(0:3), CORFAC(NF,3:4,2),
     1         BRAKET(2,NF,NGPL1)
C
C  Further local variables
      INTEGER MU, IFL1, SIGMA, ifl, ic, i1,i2,i3
      real*8  F4HAT, Q1EPS2,Q2EPS1,EPS12, G12S0,G12S1,G12S2,G12S3, PROP
      complex*16  Z1,Z2,Z3,Z4, ZJ0,ZJ1,ZJ2,ZJ3,Z3M4,
     1          AG12, B12,B21, MSP,MSM, S1R,S1C
      EXTERNAL S1R,S1C
C
C  Information on coupling constants from KOPPEL
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
c
c  provide information on the color structure in large N approxination
      real*8 r12, r21, col11_22, col12_12
      common /cvqqgg/ col11_22, col12_12
      logical lcol, ldebug
      parameter (ldebug=.false.)
c  This information is also transferred via the new entry points RVcQQGG
c  etc. (implying lcol= .true.). For qqggV amplitudes the resulting amplitude
c  square information is then stored in
c
c  output:  RV(ic,ifl,i3)     ic = 1,2: direct and charge conjugate process
c                             ifl = 1: ddggV,  ifl=2: uuggV
c                             i3=0: color summed result, e.g. 
c  rv(ic,1,0) = rvdd(ic) etc.
c  rv(ic,2,1) = rvuu(ic)*col11_22    i3=1: color connection q1 - g1 = g2 - q2
c  rv(ic,1,2) = rvdd(ic)*col12_12    i3=2: color connection q1 - g2 = g1 - q2
c
C
      DATA JVLL /4*(0.0d0, 0.0d0)/, CORFAC /NF4*(0.0d0, 0.0d0)/
      RVUU(1) = 0.0d0
      RVUU(2) = 0.0d0
      RVDD(1) = 0.0d0
      RVDD(2) = 0.0d0
      lcol = .false.
      GOTO 10
c
      ENTRY RVcQQGG(PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,KBAR,KSIGN,VPOL,
     3              RV)
      do ic = 1,2
         do ifl = 1,2
            do i3 = 0,2
               rv(ic,ifl,i3) = 0d0
            enddo
         enddo
      enddo
      lcol = .true.
      goto 10
C
      ENTRY RWQQGG(PBAR,FSIGN,FPOL,
     1             QBAR,GSIGN,GPOL,
     2         BOS,KBAR,KSIGN,VPOL,
     3             RWUD)
      RWUD = 0.0d0
      lcol = .false.
      goto 10
c
      ENTRY RWcQQGG(PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,KBAR,KSIGN,VPOL,
     3              RW)
      RW(0) = 0.0d0
      RW(1) = 0.0d0
      RW(2) = 0.0d0
      lcol = .true.
C
 10   CONTINUE
C
C  Get polarisation vector, momentum K and Vqq coupling for onshell
C  boson  V
      SIGMAF(1) = FSIGN(1)*FPOL(1)
      SIGMAF(2) = FSIGN(2)*FPOL(2)
      IF (SIGMAF(1).NE.SIGMAF(2)) THEN
        WRITE(*,*) ' Helicity nonconserving amplitude called in VQQGG '
        RETURN
      ENDIF
C
      SIGMA = SIGMAF(1)
      CALL POLVEC(KBAR,VPOL,EPSV)
      K(0) = KSIGN*KBAR(0)
      JVLL(0) = 0.0d0
      DO 20 MU = 1,3
        K(MU) = KSIGN*KBAR(MU)
 20     JVLL(MU) = EPSV(MU)
      K(4) = KBAR(4)**2
C
      DO 30 IFL = 3,4
        CORFAC(1,IFL,1) = CLR(IFL,BOS,SIGMA)
 30     CORFAC(1,IFL,2) = CLR(IFL,BOS,-SIGMA)
C
        GOTO 40
C
C  Now the entry for offshell electroweak bosons
      ENTRY LVQQGG (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,LBAR,LSIGN,LPOL,LFLAVR,
     1              RVUU,RVDD)
      RVUU(1) = 0.0d0
      RVUU(2) = 0.0d0
      RVDD(1) = 0.0d0
      RVDD(2) = 0.0d0
      lcol = .false.
      GOTO 50
c
      ENTRY LVcQQGG (PBAR,FSIGN,FPOL,
     1               QBAR,GSIGN,GPOL,
     2               BOS,LBAR,LSIGN,LPOL,LFLAVR,
     1               RV)
      do ic = 1,2
         do ifl = 1,2
            do i3 = 0,2
               rv(ic,ifl,i3) = 0d0
            enddo
         enddo
      enddo
      lcol = .true.
      goto 50
C
      ENTRY LWQQGG (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3              RWUD)
      RWUD = 0.0d0
      lcol = .false.
      goto 50
      ENTRY LWcQQGG (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3              RW)
      RW(0) = 0.0d0
      RW(1) = 0.0d0
      RW(2) = 0.0d0
      lcol = .true.
 50   CONTINUE
C
      SIGMAF(1) = FSIGN(1)*FPOL(1)
      SIGMAF(2) = FSIGN(2)*FPOL(2)
      IF (SIGMAF(1).NE.SIGMAF(2)) THEN
        WRITE(*,*) ' Helicity nonconserving amplitude called in VQQGG '
        RETURN
      ENDIF
      SIGMA = SIGMAF(1)
C
C  Get the decay current JVLL
      CALL DECAYC(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMAF,BOS,
     *            JVLL,CORFAC,K)
      if (ldebug) then
         print*,' LVQQGG info: corfac = ',corfac
         print*,' JVLL = ',JVLL
      endif
C
C  Now virtual and real bosons merge again
 40   CONTINUE
C
C  Get the unbared quark and gluon momenta
      DO 60 MU = 0,3
        P(MU,1) = FSIGN(1)*PBAR(MU,1)
        P(MU,2) = FSIGN(2)*PBAR(MU,2)
        Q(MU,1) = GSIGN(1)*QBAR(MU,1)
 60     Q(MU,2) = GSIGN(2)*QBAR(MU,2)
        Q(4,1)  = 0.0d0
        Q(4,2)  = 0.0d0
C
C  Get the 2 quark spinors CHIF and the gluon polarization vectors EPSG
      CALL CHIOM(NF,PBAR,SIGMAF,CHIF,OMFACF)
      CALL POLVEC(QBAR(0,1),GPOL(1),EPSG(0,1))
      CALL POLVEC(QBAR(0,2),GPOL(2),EPSG(0,2))
C
C  Get the bras <2,k|, <2,q_i| and the kets |k,1>, |q_i,1>
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA,Q(0,1),EPSG(0,1),
     *           BRAKET(1,1,1),DUMMY)
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA,Q(0,2),EPSG(0,2),
     *           BRAKET(1,1,2),DUMMY)
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA,Q(0,1),EPSG(0,1),
     *           BRAKET(1,2,1),DUMMY)
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA,Q(0,2),EPSG(0,2),
     *           BRAKET(1,2,2),DUMMY)
      CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA,K,JVLL,
     *           BRAKET(1,1,3),DUMMY)
      CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA,K,JVLL,
     *           BRAKET(1,2,3),DUMMY)
C
C  Calculate the 4-vector (Gamma_12)^mu  and contract it with
C  <2|mu|k,1>+<2,k|mu|1>:
c
c	Include anomalous dimension-6 coupling
c
      Q1EPS2 = -2D0*(Q(1,1)*EPSG(1,2)+Q(2,1)*EPSG(2,2)+Q(3,1)*EPSG(3,2))
      Q2EPS1 = -2D0*(Q(1,2)*EPSG(1,1)+Q(2,2)*EPSG(2,1)+Q(3,2)*EPSG(3,1))
      EPS12 = -EPSG(1,1)*EPSG(1,2)-EPSG(2,1)*EPSG(2,2)
     *        -EPSG(3,1)*EPSG(3,2)
      G12S0 = ( q(0,1) - q(0,2) ) * eps12
      G12S1 = ( q(1,1) - q(1,2) ) * eps12
     +      - epsg(1,1) * q1eps2
     +      + epsg(1,2) * q2eps1
      G12S2 = ( q(2,1) - q(2,2) ) * eps12
     +      - epsg(2,1) * q1eps2
     +      + epsg(2,2) * q2eps1
      G12S3 = ( q(3,1) - q(3,2) ) * eps12
     +      - epsg(3,1) * q1eps2
     +      + epsg(3,2) * q2eps1
c
      Z1 = BRAKET(1,2,3)*CHIF(1,1) + CHIF(1,2)*BRAKET(1,1,3)
      Z2 = BRAKET(2,2,3)*CHIF(2,1) + CHIF(2,2)*BRAKET(2,1,3)
      Z3 = BRAKET(1,2,3)*CHIF(2,1) + CHIF(1,2)*BRAKET(2,1,3)
      Z4 = BRAKET(2,2,3)*CHIF(1,1) + CHIF(2,2)*BRAKET(1,1,3)
      ZJ0  = Z1+Z2
      ZJ1  = Z3+Z4
      Z3M4 = Z3-Z4
      ZJ2  = dcmplx( dimag(Z3M4),-dble(Z3M4))
      ZJ3  = Z1-Z2
C
      PROP = 0.5d0 /( Q(0,1)*Q(0,2)-Q(1,1)*Q(1,2)-
     1             Q(2,1)*Q(2,2)-Q(3,1)*Q(3,2) )
      IF (SIGMA .EQ. -1) THEN
        AG12 = (ZJ0*G12S0 + ZJ1*G12S1 + ZJ2*G12S2 + ZJ3*G12S3) * PROP
      ELSE
        AG12 = (ZJ0*G12S0 - ZJ1*G12S1 - ZJ2*G12S2 - ZJ3*G12S3) * PROP
      ENDIF
C
C  Get B(1,2) and B(2,1) of eqn.4.14 ....
      B12 = S1R(BRAKET(1,2,1),EPSG(0,2),.FALSE.,SIGMA,BRAKET(1,1,3))
     1    + S1C(BRAKET(1,2,1),JVLL,.FALSE.,SIGMA,BRAKET(1,1,2))
     2    + S1R(BRAKET(1,2,3),EPSG(0,1),.FALSE.,SIGMA,BRAKET(1,1,2))
     3    - AG12
      B21 = S1R(BRAKET(1,2,2),EPSG(0,1),.FALSE.,SIGMA,BRAKET(1,1,3))
     1    + S1C(BRAKET(1,2,2),JVLL,.FALSE.,SIGMA,BRAKET(1,1,1))
     2    + S1R(BRAKET(1,2,3),EPSG(0,2),.FALSE.,SIGMA,BRAKET(1,1,1))
     3    + AG12
C
C  .... and form the coefficients of O^(+) and O^(-) in eq. 4.15
      MSP = B12 + B21
      MSM = B12 - B21
c
c  Extract color flow information
      r12 = dble(b12)**2+dimag(b12)**2
      r21 = dble(b21)**2+dimag(b21)**2
      col11_22 = r12/(r12+r21)
      col12_12 = r21/(r12+r21)
C
C  Get overall factor F3, forget about sign factor FSIGN(1)*FSIGN(2)
      F4HAT = CLR(3,5,1)**2 * OMFACF
C
C  ... and now the final answer. The color factors are for N = 3 in
C  table 2 of HZ
      CSAM2 =  CPLUS  * ( dble(MSP)**2+dimag(MSP)**2 )
     1        +CMINUS * ( dble(MSM)**2+dimag(MSM)**2 )
      CSAM2 = CSAM2 * F4HAT**2
      if (ldebug) then
         print*,' LVQQGG info: F4HAT = ',F4HAT
         print*,' CSAM2 = ',CSAM2,' lcol = ',lcol,' bos = ',bos
      endif
C
      IF (BOS.EQ.1 .OR. BOS.EQ.2 .OR. BOS.EQ.10) THEN
         DO IC = 1,2
            if (lcol) then
               RV(ic,1,0) = CSAM2 *            ! color sum for d-quark
     1           ( dble(CORFAC(1,4,IC))**2+dimag(CORFAC(1,4,IC))**2 )
               RV(ic,2,0) = CSAM2 *            ! color sum for u-quark
     1           ( dble(CORFAC(1,3,IC))**2+dimag(CORFAC(1,3,IC))**2 )
               do ifl=1,2
                  rv(ic,ifl,1) = rv(ic,ifl,0)*col11_22
                  rv(ic,ifl,2) = rv(ic,ifl,0)*col12_12
               enddo
            else
               RVUU(IC) = CSAM2 *
     1           ( dble(CORFAC(1,3,IC))**2+dimag(CORFAC(1,3,IC))**2 )
               RVDD(IC) = CSAM2 *
     1           ( dble(CORFAC(1,4,IC))**2+dimag(CORFAC(1,4,IC))**2 )
            endif
         ENDDO
         if (ldebug) then
            print*,' LVQQGG info: RV = ',RV
         endif

      ELSE
         if (lcol) then
            RW(0)  = CSAM2 *               ! color summed output
     1        ( dble(CORFAC(1,3,1))**2+dimag(CORFAC(1,3,1))**2 )
            rw(1) = rw(0)*col11_22
            rw(2) = rw(0)*col12_12
         else
            RWUD  = CSAM2 *
     1        ( dble(CORFAC(1,3,1))**2+dimag(CORFAC(1,3,1))**2 )
         endif
      ENDIF
C
      RETURN
      END
C
C************ RV2Q3G,AV2Q3G ********************************************
C
      SUBROUTINE RV2Q3G(PBAR,FSIGN,FPOL,
     1                  QBAR,GSIGN,GPOL,
     2              BOS,KBAR,KSIGN,VPOL,
     3                  RVUU,RVDD)
      IMPLICIT NONE
      INTEGER NF, NG, NF4, NGPL1, NF48
      PARAMETER (NF=2, NG=3, NF4=NF*4,NF48=NF*48,NGPL1=NG+1)
C          
C  RV2Q3G calculates the color-summed amplitude squared of the Vqqggg
C  amplitude (eqns. 5.12 to 5.20 of HZ ) for fixed fermion helicities
C  and fixed V and gluon polarizations. The cartesian polarization 
C  basis is chosen for V and gluons. All fermions are assumed to be 
C  massless, except in the case BOS=10 and LFLAVR=21,22,23, i.e. for 
C  NC processes with incoming charged leptons. In this case electron 
C  mass effects are included in order to allow for a reliable 
C  simulation of ep scattering at extremely low Q**2 ( << m_e**2 ).
C
C  5 different channels are considered:
C     CHANEL = 1:   real photon or Z
C     CHANEL = 2:   real W+ or W-
C     CHANEL = 3:   virtual photon and Z coupled to e+e-
C     CHANEL = 4:   virtual photon and Z coupled to nu nu-bar
C     CHANEL = 5:   virtual W+- coupled to nu e
C
C
      real*8 PBAR(0:3,NF), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2)
      real*8 P   (0:3,NF), Q   (0:4,NG), K   (0:4), L   (0:3,2)
C
      INTEGER FSIGN(NF), FPOL(NF), SIGMAF(NF),
     1        GSIGN(NG), GPOL(NG),
     2        BOS, KSIGN, VPOL, LSIGN(2), LPOL(2), LFLAVR(2)
C
      real*8 CSAM2, RVUU(2), RVDD(2), rwud, rv(2,2,0:6), rw(0:6),
     1     EPSG(0:3,NG), EPSV(0:3), JVXYZ(0:3,3), OMFACF, DUMMY(0:4),
     2     ZKOEFF(6,6),CKOEFF(6)
      complex*16  CHIF(2,NF), JVLL(0:3), CORFAC(NF,3:4,2),
     1         BRAKET(2,NF,NGPL1), JV10(0:3,NF,3:4,2,-1:1)
C
C  Vectors for the gluon vertices
      real*8   G(0:3,3,3), D123S0, D123S1, D123S2, D123S3,
     1                   D213S0, D213S1, D213S2, D213S3,
     2                   D312S0, D312S1, D312S2, D312S3,
     3                   V123S0, V123S1, V123S2, V123S3,
     4                   V213S0, V213S1, V213S2, V213S3
C
C  Further local variables
      INTEGER MU, IFL1, SIGMA, NCALL, NN, KK, IFL, IC, IL, 
     1        I3,CHANEL
      REAL*8  F6HAT, F6HAT2, PQK(0:4,NF,NGPL1), PROP,
     1       Q1EPS2, Q2EPS1, Q1EPS3, Q3EPS1, Q2EPS3, Q3EPS2,
     2       EPS12, EPS13, EPS23, Q12, Q13, Q23,
     3       TQ1G23, TQ2G13, TQ3G12, E1G23, E2G13, E3G12,
     4       summ2,summ1
C
      complex*16  DEBK(2,NF,NGPL1,NGPL1),
     1          Z1, Z2, Z3, Z4, Z3M4, AH12(3,3), AH23(3,3),
     2          E21S0, E21S1, E21S2, E21S3,
     3          ED123, ED213, ED312, EV123, EV213, EV231,
     4          M(6), AMSQ(6), FOURG(3,3,3),
     5          M10(6,3), AMOC10(6,3), AM10(6,-1:1,3:4,2),
     6          COM10(-1:1,3:4,2,6)
      REAL*8 AM10SQ(3:4,2), COM10SUM(3:4,2), COM10SQ(3:4,2,6)
      logical lcol
C
C  DEBK stores "Double Emission Bra Kets".
C      NF index:   fermion index as in BRAKET
C      3rd index:  index of boson emitted closest to fermion
C      4th index:  index of second boson emitted
C
C  Information passed to AV2Q3G by common block:
c      COMMON /BV2Q3G/ CHIF, BRAKET, DEBK, P, SIGMA, Q, EPSG,
c     1                FOURG, G, AH12, AH23
      COMMON /BV2Q3G/ CHIF, BRAKET, DEBK, AH12, AH23, FOURG,  
     1                P, Q, G, EPSG, SIGMA 
C
C  Information on coupling constants from KOPPEL
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
C
      DATA JVLL /4*(0.0d0, 0.0d0)/, CORFAC /NF4*(0.0d0, 0.0d0)/
      DATA JVXYZ /0D0,1D0,0D0,0D0,  0D0,0D0,1D0,0D0,  0D0,0D0,0D0,1D0/  
      DATA JV10 /NF48*(0D0,0D0)/
      DATA ZKOEFF /  1.0d0, 1.0d0, 1.0d0, 1.0d0, 1.0d0, 1.0d0,
     1               1.0d0, -1.0d0, -1.0d0, 1.0d0, 1.0d0, -1.0d0,
     2               1.0d0, -0.5d0, -0.5d0, -0.5d0, -0.5d0, 1.0d0,
     3               1.0d0, 0.5d0, 0.5d0, -0.5d0, -0.5d0, -1.0d0,
     4               0.0d0, -0.86602540378d0, 0.86602540378d0, 
     4                    0.86602540378d0, -0.86602540378d0, 0.0d0,
     5               0.0d0, -0.86602540378d0, 0.86602540378d0,
     5                    -0.86602540378d0, 0.86602540378d0, 0.0d0/
      DATA CKOEFF /  1.11111111111111111d0,
     1               1.33333333333333333d0,
     2               3.0d0,
     3               1.66666666666666667d0,
     4               1.66666666666666667d0,
     5               3.0d0 /
      RVUU(1) = 0.0d0
      RVUU(2) = 0.0d0
      RVDD(1) = 0.0d0
      RVDD(2) = 0.0d0
      IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
        CHANEL = 1
      ELSE 
        CHANEL = -10
      ENDIF
      lcol = .false.
      GOTO 10
c
      entry RVc2Q3G( PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,KBAR,KSIGN,VPOL,
     3              RV)
      do ic = 1,2
         do ifl = 1,2
            do i3 = 0,6
               rv(ic,ifl,i3) = 0d0
            enddo
         enddo
      enddo
      IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
         CHANEL = 1
      ELSE 
         CHANEL = -10
      ENDIF
      lcol = .true.
      goto 10

C
      ENTRY RW2Q3G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,KBAR,KSIGN,VPOL,
     3              RWUD)
      RWUD = 0.0d0
      IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
        CHANEL = 2
      ELSE 
        CHANEL = -20
      ENDIF
      lcol = .false. 
      goto 10
c           
      ENTRY RWc2Q3G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,KBAR,KSIGN,VPOL,
     3              RW)
      do i3 = 0,6
         RW(i3) = 0.0d0
      enddo
c
      IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
        CHANEL = 2
      ELSE 
        CHANEL = -20
      ENDIF
      lcol = .true. 
      
      
 10   CONTINUE
C
C  Get polarisation vector, momentum K and Vqq coupling for onshell
C  boson  V
      SIGMAF(1) = FSIGN(1)*FPOL(1)
      SIGMAF(2) = FSIGN(2)*FPOL(2)
      IF (SIGMAF(1).NE.SIGMAF(2)) THEN
        WRITE(*,*) ' Helicity nonconserving amplitude called in V2Q3G '
        RETURN
      ENDIF
      SIGMA = SIGMAF(1)
      CALL POLVEC(KBAR,VPOL,EPSV)
      K(0) = KSIGN*KBAR(0)
      JVLL(0) = 0.0d0
      DO 20 MU = 1,3
        K(MU) = KSIGN*KBAR(MU)
 20     JVLL(MU) = EPSV(MU)
      K(4) = KBAR(4)**2
      DO 30 IFL = 3,4
        CORFAC(1,IFL,1) = CLR(IFL,BOS,SIGMA)
 30     CORFAC(1,IFL,2) = CLR(IFL,BOS,-SIGMA)
C
      GOTO 40
C
C  Now the entry for offshell electroweak bosons
      ENTRY LV2Q3G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     1              RVUU,RVDD)
      RVUU(1) = 0.0d0
      RVUU(2) = 0.0d0
      RVDD(1) = 0.0d0
      RVDD(2) = 0.0d0
c      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.2.AND.LFLAVR(2)/10.EQ.2 ) THEN
      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.20.AND.LFLAVR(2)/10.EQ.20 ) THEN
        CHANEL = 3
      ELSEIF (BOS.EQ.10) THEN 
        CHANEL = 4
      ELSE 
        CHANEL = -30
      ENDIF
      lcol = .false.
      GOTO 50
c
      ENTRY LVc2Q3G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     1              RV)
      do ic =1,2
         do ifl = 1,2
            do i3 = 0,6
               rv(ic,ifl,i3) = 0d0
            enddo
         enddo
      enddo

c      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.2.AND.LFLAVR(2)/10.EQ.2 ) THEN
      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.20.AND.LFLAVR(2)/10.EQ.20 ) THEN
        CHANEL = 3
      ELSEIF (BOS.EQ.10) THEN 
        CHANEL = 4
      ELSE 
        CHANEL = -30
      ENDIF
      lcol = .true.
      GOTO 50
C
      ENTRY LW2Q3G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3              RWUD)
      RWUD = 0.0d0
      IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
        CHANEL = 5
      ELSE 
        CHANEL = -50
      ENDIF
      lcol = .false.
      goto 50
c
      ENTRY LWc2Q3G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2              BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3              RW)
      do i3 = 0,6
         RW(i3) = 0.0d0
      enddo
c
      IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
        CHANEL = 5
      ELSE 
        CHANEL = -50
      ENDIF
      lcol = .true.
 50   CONTINUE
C
      SIGMAF(1) = FSIGN(1)*FPOL(1)
      SIGMAF(2) = FSIGN(2)*FPOL(2)
      IF (SIGMAF(1).NE.SIGMAF(2)) THEN
        WRITE(*,*) ' Helicity nonconserving amplitude called in VQQGG '
        RETURN
      ENDIF
      SIGMA = SIGMAF(1)
C
C  Get the decay current JVLL
      IF ( CHANEL.EQ.3 ) THEN
        CALL DECAYN(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMAF,JV10,K)
      ELSE
        CALL DECAYC(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMAF,BOS,
     *            JVLL,CORFAC,K)
      ENDIF
C
C  Now virtual and real bosons merge again
 40   CONTINUE
C
C  Get the unbared quark and gluon momenta
      DO 60 MU = 0,3
        P(MU,1) = FSIGN(1)*PBAR(MU,1)
        P(MU,2) = FSIGN(2)*PBAR(MU,2)
        Q(MU,1) = GSIGN(1)*QBAR(MU,1)
        Q(MU,2) = GSIGN(2)*QBAR(MU,2)
 60     Q(MU,3) = GSIGN(3)*QBAR(MU,3)
        Q(4,1)  = 0.0d0
        Q(4,2)  = 0.0d0
        Q(4,3)  = 0.0d0
C
C  Get the 2 quark spinors CHIF and the gluon polarization vectors EPSG
      CALL CHIOM(NF,PBAR,SIGMAF,CHIF,OMFACF)
      CALL POLVEC(QBAR(0,1),GPOL(1),EPSG(0,1))
      CALL POLVEC(QBAR(0,2),GPOL(2),EPSG(0,2))
      CALL POLVEC(QBAR(0,3),GPOL(3),EPSG(0,3))
C
C  Get the bras <2,k|, <2,q_i| and the kets |k,1>, |q_i,1>
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA,Q(0,1),EPSG(0,1),
     *           BRAKET(1,1,1),PQK(0,1,1))
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA,Q(0,2),EPSG(0,2),
     *           BRAKET(1,1,2),PQK(0,1,2))
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA,Q(0,3),EPSG(0,3),
     *           BRAKET(1,1,3),PQK(0,1,3))
C
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA,Q(0,1),EPSG(0,1),
     *           BRAKET(1,2,1),PQK(0,2,1))
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA,Q(0,2),EPSG(0,2),
     *           BRAKET(1,2,2),PQK(0,2,2))
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA,Q(0,3),EPSG(0,3),
     *           BRAKET(1,2,3),PQK(0,2,3))
C
C  Calculate the 4-vectors (Gamma_ij)^mu
      Q1EPS2 = -2D0*(Q(1,1)*EPSG(1,2)+Q(2,1)*EPSG(2,2)+Q(3,1)*EPSG(3,2))
      Q2EPS1 = -2D0*(Q(1,2)*EPSG(1,1)+Q(2,2)*EPSG(2,1)+Q(3,2)*EPSG(3,1))
      EPS12 = -EPSG(1,1)*EPSG(1,2)-
     *         EPSG(2,1)*EPSG(2,2)-EPSG(3,1)*EPSG(3,2)
      G(0,1,2) =                     (Q(0,1)-Q(0,2))*EPS12
      G(1,1,2) = -EPSG(1,1)*Q1EPS2 + (Q(1,1)-Q(1,2))*EPS12 +
     *            EPSG(1,2)*Q2EPS1
      G(2,1,2) = -EPSG(2,1)*Q1EPS2 + (Q(2,1)-Q(2,2))*EPS12 +
     *            EPSG(2,2)*Q2EPS1
      G(3,1,2) = -EPSG(3,1)*Q1EPS2 + (Q(3,1)-Q(3,2))*EPS12 +
     *            EPSG(3,2)*Q2EPS1
      Q12 = Q(0,1)*Q(0,2)-Q(1,1)*Q(1,2)-Q(2,1)*Q(2,2)-Q(3,1)*Q(3,2)
C
      PROP = 0.5d0 / Q12
      G(0,1,2) = G(0,1,2)*PROP
      G(1,1,2) = G(1,1,2)*PROP
      G(2,1,2) = G(2,1,2)*PROP
      G(3,1,2) = G(3,1,2)*PROP
C
      Q1EPS3 = -2D0*(Q(1,1)*EPSG(1,3)+Q(2,1)*EPSG(2,3)+Q(3,1)*EPSG(3,3))
      Q3EPS1 = -2D0*(Q(1,3)*EPSG(1,1)+Q(2,3)*EPSG(2,1)+Q(3,3)*EPSG(3,1))
      EPS13 = - EPSG(1,1)*EPSG(1,3) -
     *          EPSG(2,1)*EPSG(2,3) - EPSG(3,1)*EPSG(3,3)
      G(0,1,3) =                     (Q(0,1)-Q(0,3))*EPS13
      G(1,1,3) = -EPSG(1,1)*Q1EPS3 + (Q(1,1)-Q(1,3))*EPS13 +
     *            EPSG(1,3)*Q3EPS1
      G(2,1,3) = -EPSG(2,1)*Q1EPS3 + (Q(2,1)-Q(2,3))*EPS13 +
     *            EPSG(2,3)*Q3EPS1
      G(3,1,3) = -EPSG(3,1)*Q1EPS3 + (Q(3,1)-Q(3,3))*EPS13 +
     *            EPSG(3,3)*Q3EPS1
      Q13 = Q(0,1)*Q(0,3)-Q(1,1)*Q(1,3)-Q(2,1)*Q(2,3)-Q(3,1)*Q(3,3)
C
      PROP = 0.5d0 / Q13
      G(0,1,3) = G(0,1,3)*PROP
      G(1,1,3) = G(1,1,3)*PROP
      G(2,1,3) = G(2,1,3)*PROP
      G(3,1,3) = G(3,1,3)*PROP
C
      Q2EPS3 = -2D0*(Q(1,2)*EPSG(1,3)+Q(2,2)*EPSG(2,3)+Q(3,2)*EPSG(3,3))
      Q3EPS2 = -2D0*(Q(1,3)*EPSG(1,2)+Q(2,3)*EPSG(2,2)+Q(3,3)*EPSG(3,2))
      EPS23 = - EPSG(1,2)*EPSG(1,3) -
     *          EPSG(2,2)*EPSG(2,3) - EPSG(3,2)*EPSG(3,3)
      G(0,2,3) =                     (Q(0,2)-Q(0,3))*EPS23
      G(1,2,3) = -EPSG(1,2)*Q2EPS3 + (Q(1,2)-Q(1,3))*EPS23 +
     *            EPSG(1,3)*Q3EPS2
      G(2,2,3) = -EPSG(2,2)*Q2EPS3 + (Q(2,2)-Q(2,3))*EPS23 +
     *            EPSG(2,3)*Q3EPS2
      G(3,2,3) = -EPSG(3,2)*Q2EPS3 + (Q(3,2)-Q(3,3))*EPS23 +
     *            EPSG(3,3)*Q3EPS2
      Q23 = Q(0,2)*Q(0,3)-Q(1,2)*Q(1,3)-Q(2,2)*Q(2,3)-Q(3,2)*Q(3,3)
C
      PROP = 0.5d0 / Q23
      G(0,2,3) = G(0,2,3)*PROP
      G(1,2,3) = G(1,2,3)*PROP
      G(2,2,3) = G(2,2,3)*PROP
      G(3,2,3) = G(3,2,3)*PROP
C
C
C  Calculate the four-vectors D_ijk for the double 3-gluon vertex
C
      PROP = 0.5d0 / ( Q12+Q13+Q23 )
      TQ1G23 = 2D0 *( Q(0,1)*G(0,2,3) - Q(1,1)*G(1,2,3) -
     *              Q(2,1)*G(2,2,3) - Q(3,1)*G(3,2,3) )
      E1G23 = - EPSG(1,1)*G(1,2,3) -
     *          EPSG(2,1)*G(2,2,3) - EPSG(3,1)*G(3,2,3)
      D123S0 =  G(0,2,3)*(Q2EPS1+Q3EPS1) +
     *           (Q(0,1)-Q(0,2)-Q(0,3))*E1G23
      D123S1 =  G(1,2,3)*(Q2EPS1+Q3EPS1) - EPSG(1,1)*TQ1G23 +
     *           (Q(1,1)-Q(1,2)-Q(1,3))*E1G23
      D123S2 =  G(2,2,3)*(Q2EPS1+Q3EPS1) - EPSG(2,1)*TQ1G23 +
     *           (Q(2,1)-Q(2,2)-Q(2,3))*E1G23
      D123S3 =  G(3,2,3)*(Q2EPS1+Q3EPS1) - EPSG(3,1)*TQ1G23 +
     *           (Q(3,1)-Q(3,2)-Q(3,3))*E1G23
C
      TQ2G13 = 2D0 *( Q(0,2)*G(0,1,3) - Q(1,2)*G(1,1,3) -
     *              Q(2,2)*G(2,1,3) - Q(3,2)*G(3,1,3) )
      E2G13 = - EPSG(1,2)*G(1,1,3) -
     *          EPSG(2,2)*G(2,1,3) - EPSG(3,2)*G(3,1,3)
      D213S0 =  G(0,1,3)*(Q1EPS2+Q3EPS2) +
     *           (Q(0,2)-Q(0,1)-Q(0,3))*E2G13
      D213S1 =  G(1,1,3)*(Q1EPS2+Q3EPS2) - EPSG(1,2)*TQ2G13 +
     *           (Q(1,2)-Q(1,1)-Q(1,3))*E2G13
      D213S2 =  G(2,1,3)*(Q1EPS2+Q3EPS2) - EPSG(2,2)*TQ2G13 +
     *           (Q(2,2)-Q(2,1)-Q(2,3))*E2G13
      D213S3 =  G(3,1,3)*(Q1EPS2+Q3EPS2) - EPSG(3,2)*TQ2G13 +
     *           (Q(3,2)-Q(3,1)-Q(3,3))*E2G13
C
      TQ3G12 = 2D0 *( Q(0,3)*G(0,1,2) - Q(1,3)*G(1,1,2) -
     *              Q(2,3)*G(2,1,2) - Q(3,3)*G(3,1,2) )
      E3G12 = - EPSG(1,3)*G(1,1,2) -
     *          EPSG(2,3)*G(2,1,2) - EPSG(3,3)*G(3,1,2)
      D312S0 =  G(0,1,2)*(Q2EPS3+Q1EPS3) +
     *           (Q(0,3)-Q(0,2)-Q(0,1))*E3G12
      D312S1 =  G(1,1,2)*(Q2EPS3+Q1EPS3) - EPSG(1,3)*TQ3G12 +
     *           (Q(1,3)-Q(1,2)-Q(1,1))*E3G12
      D312S2 =  G(2,1,2)*(Q2EPS3+Q1EPS3) - EPSG(2,3)*TQ3G12 +
     *           (Q(2,3)-Q(2,2)-Q(2,1))*E3G12
      D312S3 =  G(3,1,2)*(Q2EPS3+Q1EPS3) - EPSG(3,3)*TQ3G12 +
     *           (Q(3,3)-Q(3,2)-Q(3,1))*E3G12
C
C  Calculate the 4-vectors V_ijk
      V123S0 =  0.0d0
      V123S1 =  2D0 *EPSG(1,2)*EPS13 - EPSG(1,1)*EPS23 - EPSG(1,3)*EPS12
      V123S2 =  2D0 *EPSG(2,2)*EPS13 - EPSG(2,1)*EPS23 - EPSG(2,3)*EPS12
      V123S3 =  2D0 *EPSG(3,2)*EPS13 - EPSG(3,1)*EPS23 - EPSG(3,3)*EPS12
C
      V213S0 =  0.0d0
      V213S1 =  2D0 *EPSG(1,1)*EPS23 - EPSG(1,2)*EPS13 - EPSG(1,3)*EPS12
      V213S2 =  2D0 *EPSG(2,1)*EPS23 - EPSG(2,2)*EPS13 - EPSG(2,3)*EPS12
      V213S3 =  2D0 *EPSG(3,1)*EPS23 - EPSG(3,2)*EPS13 - EPSG(3,3)*EPS12
C
C   Now distinguish the different channels:
      IF (CHANEL.NE.3) THEN
        CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA,K,JVLL,
     *           BRAKET(1,1,4),PQK(0,1,4))
        CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA,K,JVLL,
     *           BRAKET(1,2,4),PQK(0,2,4))
C
C  Get bras and kets [p1-qi-k,e]|q_i,1> etc.
        CALL KET2C(BRAKET(1,1,1),.FALSE.,PQK(0,1,1),SIGMA,K,JVLL,
     *           DEBK(1,1,1,4),DUMMY)
        CALL KET2C(BRAKET(1,1,2),.FALSE.,PQK(0,1,2),SIGMA,K,JVLL,
     *           DEBK(1,1,2,4),DUMMY)
        CALL KET2C(BRAKET(1,1,3),.FALSE.,PQK(0,1,3),SIGMA,K,JVLL,
     *           DEBK(1,1,3,4),DUMMY)
        CALL KET2R(BRAKET(1,1,4),.FALSE.,PQK(0,1,4),SIGMA,Q(0,1),
     *           EPSG(0,1),DEBK(1,1,4,1),DUMMY)
        CALL KET2R(BRAKET(1,1,4),.FALSE.,PQK(0,1,4),SIGMA,Q(0,2),
     *           EPSG(0,2),DEBK(1,1,4,2),DUMMY)
        CALL KET2R(BRAKET(1,1,4),.FALSE.,PQK(0,1,4),SIGMA,Q(0,3),
     *           EPSG(0,3),DEBK(1,1,4,3),DUMMY)
C
        CALL BRA2C(BRAKET(1,2,1),.FALSE.,PQK(0,2,1),SIGMA,K,JVLL,
     *           DEBK(1,2,1,4),DUMMY)
        CALL BRA2C(BRAKET(1,2,2),.FALSE.,PQK(0,2,2),SIGMA,K,JVLL,
     *           DEBK(1,2,2,4),DUMMY)
        CALL BRA2C(BRAKET(1,2,3),.FALSE.,PQK(0,2,3),SIGMA,K,JVLL,
     *           DEBK(1,2,3,4),DUMMY)
        CALL BRA2R(BRAKET(1,2,4),.FALSE.,PQK(0,2,4),SIGMA,Q(0,1),
     *           EPSG(0,1),DEBK(1,2,4,1),DUMMY)
        CALL BRA2R(BRAKET(1,2,4),.FALSE.,PQK(0,2,4),SIGMA,Q(0,2),
     *           EPSG(0,2),DEBK(1,2,4,2),DUMMY)
        CALL BRA2R(BRAKET(1,2,4),.FALSE.,PQK(0,2,4),SIGMA,Q(0,3),
     *           EPSG(0,3),DEBK(1,2,4,3),DUMMY)
C
C  Form the current E_21^mu = <2,k|mu|1>+<2|mu|k,1> )
C  a2* a1
        Z1 = BRAKET(1,2,4)*CHIF(1,1) + CHIF(1,2)*BRAKET(1,1,4)
C
C  b2* b1
        Z2 = BRAKET(2,2,4)*CHIF(2,1) + CHIF(2,2)*BRAKET(2,1,4)
C
C  a2* b1
        Z3 = BRAKET(1,2,4)*CHIF(2,1) + CHIF(1,2)*BRAKET(2,1,4)
C
C  b2* a1
        Z4 = BRAKET(2,2,4)*CHIF(1,1) + CHIF(2,2)*BRAKET(1,1,4)
C
        E21S0 = Z1+Z2
        IF (SIGMA.EQ.-1) THEN
          E21S1 = -(Z3+Z4)
          Z3M4 = Z3-Z4
          E21S2 = dcmplx(-dimag(Z3M4),dble(Z3M4))
          E21S3 = -(Z1-Z2)
        ELSE
          E21S1 = Z3+Z4
          Z3M4 = Z3-Z4
          E21S2 = dcmplx(dimag(Z3M4),-dble(Z3M4))
          E21S3 = Z1-Z2
        ENDIF
C
C  ... and form the contractions with D and V for the last line
C  of eqn 5.18 of HZ
        ED123 = E21S0*D123S0-E21S1*D123S1-E21S2*D123S2-E21S3*D123S3
        ED213 = E21S0*D213S0-E21S1*D213S1-E21S2*D213S2-E21S3*D213S3
        ED312 = E21S0*D312S0-E21S1*D312S1-E21S2*D312S2-E21S3*D312S3
C
        EV123 = - E21S1*V123S1 - E21S2*V123S2 - E21S3*V123S3
        EV213 = - E21S1*V213S1 - E21S2*V213S2 - E21S3*V213S3
        EV231 = - EV123 - EV213
        FOURG(1,2,3) = (ED123 - ED312 + EV123)*PROP
        FOURG(1,3,2) = (-ED123 - ED213 + EV231)*PROP
        FOURG(2,1,3) = (ED213 + ED312 + EV213)*PROP
        FOURG(2,3,1) = FOURG(1,3,2)
        FOURG(3,1,2) = FOURG(2,1,3)
        FOURG(3,2,1) = FOURG(1,2,3)
C
C  And now the permutations of the three gluons and the summation over
C  colors
C  pi = (1) , (12), (13), (23), (123), (132)
c  i3 =  1     3     6     2     4      5
c  there are 6 color structures which are the the 6
c  permutations of gluons
        CALL AV2Q3G(1,2,3,M(1))
        CALL AV2Q3G(1,3,2,M(2))
        CALL AV2Q3G(2,1,3,M(3))
        CALL AV2Q3G(2,3,1,M(4))
        CALL AV2Q3G(3,1,2,M(5))
        CALL AV2Q3G(3,2,1,M(6))
C
c       Calculate sum of the squares of the M(i3):

        summ1 = 0.0d0
        do i3 = 1,6
           summ1 = summ1 + dble(M(i3))**2 + dimag(M(i3))**2
        enddo

        CSAM2 = 0.0d0
C
        DO 70 NN=1,6
          AMSQ(NN) = 0.0d0
          DO 80 KK=1,6
            AMSQ(NN)=AMSQ(NN)+ZKOEFF(KK,NN)*M(KK)
 80       CONTINUE
C
          CSAM2=CSAM2+CKOEFF(NN)*(DBLE(AMSQ(NN))**2+DIMAG(AMSQ(NN))**2)
 70    CONTINUE
C
C  Get overall factor F6, forget about sign factor FSIGN(1)*FSIGN(2)
        F6HAT = CLR(3,5,1)**3 * OMFACF
C
C  And the final answer is
        CSAM2 = CSAM2 * F6HAT**2
C
        IF (BOS.EQ.1 .OR. BOS.EQ.2 .OR. BOS.EQ.10) THEN
          DO 90 IC = 1,2
            if (lcol) then
               RV(IC,2,0) = CSAM2 *
     1         ( dble(CORFAC(1,3,IC))**2+dimag(CORFAC(1,3,IC))**2 )
               RV(IC,1,0) = CSAM2 *
     1         ( dble(CORFAC(1,4,IC))**2+dimag(CORFAC(1,4,IC))**2 )
               do ifl = 1,2 
                  do i3 = 1,6
                     rv(ic,ifl,i3) = rv(ic,ifl,0) *
     1                 (dble(M(i3))**2+dimag(M(i3))**2)/summ1
                  enddo
               enddo
            else
               RVUU(IC) = CSAM2 *
     1         ( dble(CORFAC(1,3,IC))**2+dimag(CORFAC(1,3,IC))**2 )
               RVDD(IC) = CSAM2 *
     1         ( dble(CORFAC(1,4,IC))**2+dimag(CORFAC(1,4,IC))**2 )
            endif
 90       CONTINUE
        ELSE
           if (lcol) then
              RW(0)  = CSAM2 *
     1         ( dble(CORFAC(1,3,1))**2+dimag(CORFAC(1,3,1))**2 )
              do i3 = 1,6
                 RW(i3) = RW(0) *
     1             (dble(M(i3))**2+dimag(M(i3))**2)/summ1
              enddo
           else
              RWUD  = CSAM2 *
     1         ( dble(CORFAC(1,3,1))**2+dimag(CORFAC(1,3,1))**2 )
           endif
        ENDIF
C
C  Now comes the case of a virtual photon/Z coupled to charged leptons
C
      ELSE
        DO 100 MU = 1,3
          CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA,K,JVXYZ(0,MU),
     *           BRAKET(1,1,4),PQK(0,1,4))
          CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA,K,JVXYZ(0,MU),
     *           BRAKET(1,2,4),PQK(0,2,4))
C
C  Get bras and kets [p1-qi-k,e]|q_i,1> etc.
          CALL KET2R(BRAKET(1,1,1),.FALSE.,PQK(0,1,1),SIGMA,K,
     *           JVXYZ(0,MU),DEBK(1,1,1,4),DUMMY)
          CALL KET2R(BRAKET(1,1,2),.FALSE.,PQK(0,1,2),SIGMA,K,
     *           JVXYZ(0,MU),DEBK(1,1,2,4),DUMMY)
          CALL KET2R(BRAKET(1,1,3),.FALSE.,PQK(0,1,3),SIGMA,K,
     *           JVXYZ(0,MU),DEBK(1,1,3,4),DUMMY)
          CALL KET2R(BRAKET(1,1,4),.FALSE.,PQK(0,1,4),SIGMA,Q(0,1),
     *           EPSG(0,1),DEBK(1,1,4,1),DUMMY)
          CALL KET2R(BRAKET(1,1,4),.FALSE.,PQK(0,1,4),SIGMA,Q(0,2),
     *           EPSG(0,2),DEBK(1,1,4,2),DUMMY)
          CALL KET2R(BRAKET(1,1,4),.FALSE.,PQK(0,1,4),SIGMA,Q(0,3),
     *           EPSG(0,3),DEBK(1,1,4,3),DUMMY)
C
          CALL BRA2R(BRAKET(1,2,1),.FALSE.,PQK(0,2,1),SIGMA,K,
     *           JVXYZ(0,MU),DEBK(1,2,1,4),DUMMY)
          CALL BRA2R(BRAKET(1,2,2),.FALSE.,PQK(0,2,2),SIGMA,K,
     *           JVXYZ(0,MU),DEBK(1,2,2,4),DUMMY)
          CALL BRA2R(BRAKET(1,2,3),.FALSE.,PQK(0,2,3),SIGMA,K,
     *           JVXYZ(0,MU),DEBK(1,2,3,4),DUMMY)
          CALL BRA2R(BRAKET(1,2,4),.FALSE.,PQK(0,2,4),SIGMA,Q(0,1),
     *           EPSG(0,1),DEBK(1,2,4,1),DUMMY)
          CALL BRA2R(BRAKET(1,2,4),.FALSE.,PQK(0,2,4),SIGMA,Q(0,2),
     *           EPSG(0,2),DEBK(1,2,4,2),DUMMY)
          CALL BRA2R(BRAKET(1,2,4),.FALSE.,PQK(0,2,4),SIGMA,Q(0,3),
     *           EPSG(0,3),DEBK(1,2,4,3),DUMMY)
C
C  Form the current E_21^mu = <2,k|mu|1>+<2|mu|k,1> )
C  a2* a1
          Z1 = BRAKET(1,2,4)*CHIF(1,1) + CHIF(1,2)*BRAKET(1,1,4)
C
C  b2* b1
          Z2 = BRAKET(2,2,4)*CHIF(2,1) + CHIF(2,2)*BRAKET(2,1,4)
C
C  a2* b1
          Z3 = BRAKET(1,2,4)*CHIF(2,1) + CHIF(1,2)*BRAKET(2,1,4)
C
C  b2* a1
          Z4 = BRAKET(2,2,4)*CHIF(1,1) + CHIF(2,2)*BRAKET(1,1,4)
C
          E21S0 = Z1+Z2
          IF (SIGMA.EQ.-1) THEN
            E21S1 = -(Z3+Z4)
            Z3M4 = Z3-Z4
            E21S2 = dcmplx(-dimag(Z3M4),dble(Z3M4))
            E21S3 = -(Z1-Z2)
          ELSE
            E21S1 = Z3+Z4
            Z3M4 = Z3-Z4
            E21S2 = dcmplx(dimag(Z3M4),-dble(Z3M4))
            E21S3 = Z1-Z2
          ENDIF
C
C  ... and form the contractions with D and V for the last line
C  of eqn 5.18 of HZ
          ED123 = E21S0*D123S0-E21S1*D123S1-E21S2*D123S2-E21S3*D123S3
          ED213 = E21S0*D213S0-E21S1*D213S1-E21S2*D213S2-E21S3*D213S3
          ED312 = E21S0*D312S0-E21S1*D312S1-E21S2*D312S2-E21S3*D312S3
C
          EV123 = - E21S1*V123S1 - E21S2*V123S2 - E21S3*V123S3
          EV213 = - E21S1*V213S1 - E21S2*V213S2 - E21S3*V213S3
          EV231 = - EV123 - EV213
          FOURG(1,2,3) = (ED123 - ED312 + EV123)*PROP
          FOURG(1,3,2) = (-ED123 - ED213 + EV231)*PROP
          FOURG(2,1,3) = (ED213 + ED312 + EV213)*PROP
          FOURG(2,3,1) = FOURG(1,3,2)
          FOURG(3,1,2) = FOURG(2,1,3)
          FOURG(3,2,1) = FOURG(1,2,3)
C
C  And now the permutations of the three gluons and the summation over
C  colors
C  pi = (1) , (12), (13), (23), (123), (132)
c  i3 =  1     3     6     2     4      5
c  there are 6 color structures which are the the 6
c  permutations of gluons
          CALL AV2Q3G(1,2,3,M10(1,MU))
          CALL AV2Q3G(1,3,2,M10(2,MU))
          CALL AV2Q3G(2,1,3,M10(3,MU))
          CALL AV2Q3G(2,3,1,M10(4,MU))
          CALL AV2Q3G(3,1,2,M10(5,MU))
          CALL AV2Q3G(3,2,1,M10(6,MU))
C

          DO 110 NN=1,6
            AMOC10(NN,MU) = 0.0d0
            DO 110 KK=1,6
              AMOC10(NN,MU)=AMOC10(NN,MU)+ZKOEFF(KK,NN)*M10(KK,MU)
 110       CONTINUE
 100    CONTINUE
C
C  Now contract with the lepton current JV10
        DO 120 IC = 1,2 
        DO 120 IFL = 3,4
          AM10SQ(IFL,IC) = 0D0
          DO 120 IL = -1,1,2
          DO 120 NN = 1,6
            AM10(NN,IL,IFL,IC) = -AMOC10(NN,1)*JV10(1,1,IFL,IC,IL)-
     2                            AMOC10(NN,2)*JV10(2,1,IFL,IC,IL)-
     3                            AMOC10(NN,3)*JV10(3,1,IFL,IC,IL)
            AM10SQ(IFL,IC) = AM10SQ(IFL,IC) + CKOEFF(NN) * 
     1        (DBLE(AM10(NN,IL,IFL,IC))**2+DIMAG(AM10(NN,IL,IFL,IC))**2)
120     CONTINUE
C  calculate color coefficients:
        DO 121 I3 = 1,6
        DO 121 IC = 1,2 
        DO 121 IFL = 3,4
           COM10SQ(IFL,IC,I3) = 0D0
           DO 121 IL = -1,1,2
              COM10(IL,IFL,IC,I3) = -M10(i3,1)*JV10(1,1,IFL,IC,IL)-
     2                               M10(i3,2)*JV10(2,1,IFL,IC,IL)-
     3                               M10(i3,3)*JV10(3,1,IFL,IC,IL)
              COM10SQ(IFL,IC,i3) = COM10SQ(IFL,IC,I3) +  
     1     (DBLE(COM10(IL,IFL,IC,I3))**2+DIMAG(COM10(IL,IFL,IC,I3))**2)
 121    continue
C     COMPUTE SUM OF COEFFICIETS
        DO IC =1,2
           DO IFL = 3,4
              COM10SUM(IFL,IC)=0D0
              DO I3 = 1,6
                 COM10SUM(IFL,IC)=COM10SUM(IFL,IC)+COM10SQ(IFL,IC,I3)
              ENDDO
           ENDDO
        ENDDO

C           NORMALIZE COEFFICIETS
        DO IC = 1,2
           DO IFL = 3,4
              do i3 = 1,6
                 COM10SQ(IFL,IC,I3) = COM10SQ(IFL,IC,I3)/COM10SUM(IFL,IC)
              ENDDO
           ENDDO
        ENDDO
               
C  Get overall factor F6**2
        F6HAT2 = (CLR(3,5,1)**3 * OMFACF)**2
C  
C  and the final answer is
        DO 130 IC = 1,2
           if (lcol) then
              RV(IC,1,0) = AM10SQ(4,IC) * F6HAT2
              RV(IC,2,0) = AM10SQ(3,IC) * F6HAT2
              do i3 = 1,6
                 RV(IC,1,i3) = RV(IC,1,0)*COM10SQ(4,IC,I3)
                 RV(IC,2,i3) = RV(IC,2,0)*COM10SQ(3,IC,I3)
              enddo
           else   
              RVUU(IC) = AM10SQ(3,IC) * F6HAT2
              RVDD(IC) = AM10SQ(4,IC) * F6HAT2
           endif
 130    CONTINUE
      ENDIF
C
      RETURN
      END
C
C
C**********AV2Q3G**************************************************
C
      SUBROUTINE AV2Q3G(N1,N2,N3,A)
      implicit none
      INTEGER NF,NG,NF4,NGPL1
      PARAMETER (NF=2,NG=3,NF4=NF*4,NGPL1=NG+1)
C AV2Q3G calculates the coeficient M(pi) of the color factor T(pi)
C of eq.5.18 in HZ.
C
C INPUT : N1,N2,N3 identfying the three gluons
C -------
C
C OUTPUT:
C -------
C A=M(pi) of eq.5.18 in HZ
C pi denotes the permutation (N1,N2,N3) of the three gluons
C
      INTEGER N1,N2,N3,SIGMA
      real*8 P(0:3,NF), Q(0:4,NG), EPSG(0:3,NG), G(0:3,3,3)
      complex*16  CHIF(2,NF),BRAKET(2,NF,NGPL1),DEBK(2,NF,NGPL1,NGPL1),
     1          A, FOURG(3,3,3)
C
C  Further local variables
      complex*16  AH12(3,3),AH23(3,3), Z1,Z2,Z3,Z4,Z3M4
C
C  Information passed to AV2Q3G by common block:
c      COMMON /BV2Q3G/ CHIF,BRAKET, DEBK, P, SIGMA, Q, EPSG,
c     1                FOURG, G, AH12, AH23
      COMMON /BV2Q3G/ CHIF, BRAKET, DEBK, AH12, AH23, FOURG,  
     1                P, Q, G, EPSG, SIGMA 
C
C
C  Current to be contracted with eps_2 (first 4 terms of eq.5.18)
      Z1 = DEBK  (1,2,4,N1) * BRAKET(1,1,N3)   +
     1     BRAKET(1,2,N1)   * DEBK  (1,1,4,N3) +
     2     DEBK  (1,2,N1,4) * BRAKET(1,1,N3)  +
     3     BRAKET(1,2,N1)   * DEBK  (1,1,N3,4)
C
      Z2 = DEBK  (2,2,4,N1) * BRAKET(2,1,N3)   +
     1     BRAKET(2,2,N1)   * DEBK  (2,1,4,N3) +
     2     DEBK  (2,2,N1,4) * BRAKET(2,1,N3)  +
     3     BRAKET(2,2,N1)   * DEBK  (2,1,N3,4)
C
      Z3 = DEBK  (1,2,4,N1) * BRAKET(2,1,N3)   +
     1     BRAKET(1,2,N1)   * DEBK  (2,1,4,N3) +
     2     DEBK  (1,2,N1,4) * BRAKET(2,1,N3)  +
     3     BRAKET(1,2,N1)   * DEBK  (2,1,N3,4)
C
      Z4 = DEBK  (2,2,4,N1) * BRAKET(1,1,N3)   +
     1     BRAKET(2,2,N1)   * DEBK  (1,1,4,N3) +
     2     DEBK  (2,2,N1,4) * BRAKET(1,1,N3)  +
     3     BRAKET(2,2,N1)   * DEBK  (1,1,N3,4)
C
      Z3M4 = Z3-Z4
      A = (Z3+Z4)*EPSG(1,N2) +
     1    dcmplx( dimag(Z3M4),-dble(Z3M4) )*EPSG(2,N2) +
     2    (Z1-Z2)*EPSG(3,N2)
      A = -SIGMA*A
C  Now calculate the 3-gluon vertices and exploit the antisymmetry
C  of the gamma_ij^(mu)
      IF (N2.LT.N3) THEN
        Z1 = -DEBK  (1,2,4,N1) * CHIF  (1,1) -
     1        DEBK  (1,2,N1,4) * CHIF  (1,1)  -
     2        BRAKET(1,2,N1)   * BRAKET(1,1,4)
C
        Z2 = -DEBK  (2,2,4,N1) * CHIF  (2,1) -
     1        DEBK  (2,2,N1,4) * CHIF  (2,1)  -
     2        BRAKET(2,2,N1)   * BRAKET(2,1,4)
C
        Z3 = -DEBK  (1,2,4,N1) * CHIF  (2,1) -
     1        DEBK  (1,2,N1,4) * CHIF  (2,1)  -
     2        BRAKET(1,2,N1)   * BRAKET(2,1,4)
C
        Z4 = -DEBK  (2,2,4,N1) * CHIF  (1,1) -
     1        DEBK  (2,2,N1,4) * CHIF  (1,1)  -
     2        BRAKET(2,2,N1)   * BRAKET(1,1,4)
C
        Z3M4 = Z3-Z4
        AH23(N2,N3) = (Z1+Z2)*G(0,N2,N3)
        IF (SIGMA.EQ.-1) THEN
          AH23(N2,N3) = AH23(N2,N3) + (Z3+Z4)*G(1,N2,N3) +
     1                  dcmplx( dimag(Z3M4),-dble(Z3M4) )*G(2,N2,N3) +
     2                  (Z1-Z2)*G(3,N2,N3)
        ELSE
          AH23(N2,N3) = AH23(N2,N3) - (Z3+Z4)*G(1,N2,N3) -
     1                  dcmplx( dimag(Z3M4),-dble(Z3M4) )*G(2,N2,N3) -
     2                  (Z1-Z2)*G(3,N2,N3)
        ENDIF
        A=A+AH23(N2,N3)
      ELSE
        A=A-AH23(N3,N2)
      ENDIF
C
      IF (N1.LT.N2) THEN
        Z1 = - CHIF  (1,2)   * DEBK  (1,1,N3,4) -
     1         CHIF  (1,2)   * DEBK  (1,1,4,N3) -
     3         BRAKET(1,2,4) * BRAKET(1,1,N3)
C
        Z2 = - CHIF  (2,2)   * DEBK  (2,1,N3,4) -
     1         CHIF  (2,2)   * DEBK  (2,1,4,N3) -
     3         BRAKET(2,2,4) * BRAKET(2,1,N3)
C
        Z3 = - CHIF  (1,2)   * DEBK  (2,1,N3,4) -
     1         CHIF  (1,2)   * DEBK  (2,1,4,N3) -
     3         BRAKET(1,2,4) * BRAKET(2,1,N3)
C
        Z4 = - CHIF  (2,2)   * DEBK  (1,1,N3,4) -
     1         CHIF  (2,2)   * DEBK  (1,1,4,N3) -
     3         BRAKET(2,2,4) * BRAKET(1,1,N3)
C
        Z3M4 = Z3-Z4
        AH12(N1,N2) = (Z1+Z2)*G(0,N1,N2)
        IF (SIGMA.EQ.-1) THEN
          AH12(N1,N2) = AH12(N1,N2) + (Z3+Z4)*G(1,N1,N2) +
     1                  dcmplx( dimag(Z3M4),-dble(Z3M4) )*G(2,N1,N2) +
     2                  (Z1-Z2)*G(3,N1,N2)
        ELSE
          AH12(N1,N2) = AH12(N1,N2) - (Z3+Z4)*G(1,N1,N2) -
     1                  dcmplx( dimag(Z3M4),-dble(Z3M4) )*G(2,N1,N2) -
     2                  (Z1-Z2)*G(3,N1,N2)
        ENDIF
        A=A+AH12(N1,N2)
      ELSE
        A=A-AH12(N2,N1)
      ENDIF
C
C  Add the term for the 4-gluon vertices (last line in eq.5.18 of HZ):
      A = A + FOURG(N1,N2,N3)
      RETURN
      END
C
C************ RVQQQQ, AVQQQQ *******************************************
C
C
      SUBROUTINE RVQQQQ (PBAR,FSIGN,FPOL,
     1               BOS,KBAR,KSIGN,VPOL,
     2               RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
      implicit none
      INTEGER NF, NG, NF4, NGPL1
      REAL*8 CPLUS, CMINUS
      PARAMETER (NF=4, NG=0, NF4=NF*4,NGPL1=NG+1)
      PARAMETER (CPLUS=0.6666666666666667d0,CMINUS=1.3333333333333333d0)
C
C  RVQQQQ calculates the color-summed amplitude squared of the Vqqqq
C  amplitude (eqns. 4.1 to 4.10 of HZ ) for fixed fermion helicities
C  and fixed V polarization. All fermions are assumed to be massless.
C
C  OUTPUT are the 6 2-vectors RVUUUU,...,RVDDDD for Z/photon and the 
C  real numbers RW00, RW13, RW24 for W's. As before the 2 elements
C  of each Z/photon result correspond to the direct and the charge
C  conjugation related channel. The last 4 letters of the output
C  variables identify the various flavor combinations:
C
C  RVUUUU       quark1,...,quark4 are all up-type quarks of the same
C               family
C  RVUUCC       quark1,...,quark4 are all up type, but 1 and 2 belong to
C               a family different from 3 and 4
C  etc. for photon/Z
C
C               For W's the flavors of quarks 3 and 4 are always assumed
C               to be identical, while the flavors of quarks 1 and 2 are
C               always assumed to be both entries of an SU(2) doublet.
C               This leaves three possibilities for the relative flavors
C               of the first pair (1 and 2) and the second pair of
C               quarks:
C  RW00         the W couples to quarks 1 and 2, the flavors of quark 3
C               and 4 are different from either.
C  RW13         the flavors of the first and the third quark are
C               identical
C  RW24         the flavors of quarks 2 and 4 are identical
C
      real*8 PBAR(0:3,NF), KBAR(0:4), LBAR(0:3,2)
      real*8 P   (0:3,NF), K   (0:4), L   (0:3,2)
C
      INTEGER FSIGN(NF), FPOL(NF), SIGMA(NF),
     1        BOS, KSIGN, VPOL, LSIGN(2), LPOL(2), LFLAVR(2)
C
      real*8 RVUUUU(2), RVUUCC(2), RVUUDD(2), RVDDUU(2),
     1     RVDDSS(2), RVDDDD(2), RW00, RW13, RW24,
     2     EPSV(0:3), OMFACF, DUMMY(0:4)
c     Old format (new local variables):
      real*8 RVUUUUO(2),RVUUCCO(2),RVUUDDO(2),RVDDUUO(2),
     1       RVDDSSO(2),RVDDDDO(2),RW00O,RW13O,RW24O
c
c     NEW FORMAT:  RVcI(flavor,color)  flavor = flavor of the 4 quarks
c                  which is identical. color = color structure
c                  0 = color sum
c                  1 = c14q  i.e. q1 and q4 are color connected
c                  2 = c12q  i.e. q1 and q2 are color connected
c                  RVcNI(iflv1,iflv3,col)
c                  iflv1 = flavor of quark 1, 1 for down type
c                  iflv2 = flavor of quark 2, 2 for up type
c     The flavors are either up-type or down-type.
c     RWc13(i3)    Same as old but i3 is the color flow
c     i3 = 0 for the color sum
c     i3 = 1 for cw14_13
c     i3 = 2 for cw12_13
c
      real*8 RVcI(-2:2,0:2),RVcNI(-2:2,-2:2,0:2),RWc00(0:2),
     1       RWc13(0:2),RWc24(0:2)
c     declare color and flavor indices
c
      integer i3,ifl1,ifl3,sign
c
      complex*16  CHIF(2,NF), JVLL(0:3), CORFAC(NF,3:4,2),
     1         BRAKET(2,NF)
C
C  Further local variables
      INTEGER I,J,MU,IFL,ic,u,d,s,c
      real*8  F3HAT, CPMFAC
      complex*16  A1234,A3214,A1432,A3412, MSP(6,2), MSM(6,2), MH(6,2)
C      
      logical lcol
C  Information passed to AVQQQQ by common block:
      COMMON /BVQQQQ/ CHIF, BRAKET, P, SIGMA
C
C  Information on coupling constants from KOPPEL
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
c
c  color information passed to m2s subroutines. In the c variables the
c  first two digits specify the two external fermions which colors are
c  connected, e.g. c14u(1) is the weight for rvuuuu(1) with color flowing from
c  quark 1 to 4 (and 3 to 2). Similarly cw12_13 is the weight for RW13
c  for color flow conecting quarks 1 and 2.
      complex*16 M1, M2
      double precision r1, r2
      double precision  c14u,    c14d,    c12u,    c12d, 
     &                 cw14_13, cw14_24, cw12_13, cw12_24
      common /cvqqqq/ c14u(2), c14d(2), c12u(2), c12d(2),
     &               cw14_13, cw14_24, cw12_13, cw12_24
C
      DATA JVLL /4*(0.0d0, 0.0d0)/, CORFAC /NF4*(0.0d0, 0.0d0)/
      DO 10 I=1,2
        RVUUUU(I) = 0.0d0
        RVUUCC(I) = 0.0d0
        RVUUDD(I) = 0.0d0
        RVDDUU(I) = 0.0d0
        RVDDSS(I) = 0.0d0
        RVDDDD(I) = 0.0d0
10    CONTINUE
      lcol = .true.
      GOTO 20

      ENTRY RVcQQQQ(PBAR,FSIGN,FPOL,
     1               BOS,KBAR,KSIGN,VPOL,
     2               RVcI,RVcNI)
      do ifl1 = -2,2
         do i3 = 0,2
            RVcI(ifl1,i3) = 0.0d0
            do ifl3 = -2,2
               RVcNI(ifl1,ifl3,i3) = 0.0d0
            enddo
         enddo
      enddo
      lcol = .false.
      goto 20
C
C  Entry for real W
      ENTRY RWQQQQ (PBAR,FSIGN,FPOL,
     1          BOS,KBAR,KSIGN,VPOL,
     2              RW00,RW13,RW24)
      RW00 = 0.0d0
      RW13 = 0.0d0
      RW24 = 0.0d0
      lcol = .true.
      goto 20
c      
      ENTRY RWcQQQQ (PBAR,FSIGN,FPOL,
     1          BOS,KBAR,KSIGN,VPOL,
     2              RWc00,RWc13,RWc24)
      do i3 = 0,2
         RWc00(i3) = 0.0d0
         RWc13(i3) = 0.0d0
         RWc24(i3) = 0.0d0
      enddo

      lcol = .false.
C
20    CONTINUE
C
C  Get polarisation vector, momentum K and Vqq coupling for onshell
C  boson  V
      SIGMA(1) = FSIGN(1)*FPOL(1)
      SIGMA(2) = FSIGN(2)*FPOL(2)
      SIGMA(3) = FSIGN(3)*FPOL(3)
      SIGMA(4) = FSIGN(4)*FPOL(4)
      CALL POLVEC(KBAR,VPOL,EPSV)
      K(0) = KSIGN*KBAR(0)
      DO 30 MU = 1,3
        K(MU) = KSIGN*KBAR(MU)
30      JVLL(MU) = EPSV(MU)
      K(4) = KBAR(4)**2
      DO 40 IFL = 3,4
        DO 40 I = 1,3,2
          CORFAC(I,IFL,1) = CLR(IFL,BOS,SIGMA(I))
40        CORFAC(I,IFL,2) = CLR(IFL,BOS,-SIGMA(I))
      GOTO 50
C
C  Now the entry for offshell electroweak bosons
      ENTRY LVQQQQ (PBAR,FSIGN,FPOL,
     1          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2          RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
      DO 60 I=1,2
        RVUUUU(I) = 0.0d0
        RVUUCC(I) = 0.0d0
        RVUUDD(I) = 0.0d0
        RVDDUU(I) = 0.0d0
        RVDDSS(I) = 0.0d0
        RVDDDD(I) = 0.0d0
60    CONTINUE
      lcol = .true.
      GOTO 70
c
      ENTRY LVcQQQQ (PBAR,FSIGN,FPOL,
     1          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2          RVcI,RVcNI) 
      do ifl1 = -2,2
         do i3 = 0,2
            RVcI(ifl1,i3) = 0.0d0
            do ifl3 = -2,2
               RVcNI(ifl1,ifl3,i3) = 0.0d0
            enddo
         enddo
      enddo
      lcol = .false.
      goto 70
C
      ENTRY LWQQQQ (PBAR,FSIGN,FPOL,
     1          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2              RW00,RW13,RW24)
      RW00 = 0.0d0
      RW13 = 0.0d0
      RW24 = 0.0d0
      lcol = .true.
      goto 70
C
      ENTRY LWcQQQQ (PBAR,FSIGN,FPOL,
     1          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2              RWc00,RWc13,RWc24)
      do i3 = 0,2
         RWc00(i3) = 0.0d0
         RWc13(i3) = 0.0d0
         RWc24(i3) = 0.0d0
      enddo
      lcol = .false.
      
70    CONTINUE
C
      SIGMA(1) = FSIGN(1)*FPOL(1)
      SIGMA(2) = FSIGN(2)*FPOL(2)
      SIGMA(3) = FSIGN(3)*FPOL(3)
      SIGMA(4) = FSIGN(4)*FPOL(4)
C
C  Get the decay current JVLL
      CALL DECAYC(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMA,BOS,
     *                  JVLL,CORFAC,K)
C
C  Now virtual and real bosons merge again
50    CONTINUE

C
C  Get the unbared quark and gluon momenta
      DO 80 MU = 0,3
        P(MU,1) = FSIGN(1)*PBAR(MU,1)
        P(MU,2) = FSIGN(2)*PBAR(MU,2)
        P(MU,3) = FSIGN(3)*PBAR(MU,3)
80      P(MU,4) = FSIGN(4)*PBAR(MU,4)
C
C  Get the 4 quark spinors CHIF
      CALL CHIOM(NF,PBAR,SIGMA,CHIF,OMFACF)
C
      DO 90 I = 1,2
        DO 90 J = 1,6
          MSP(J,I) = 0.0d0
          MSM(J,I) = 0.0d0
90    CONTINUE
C
C  Distinguishing cases.....
      IF ( BOS.EQ.1 .OR. BOS.EQ.2 .OR. BOS.EQ.10 ) THEN
C
C           This is the part for photon and Z
C
        CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVLL,
     *             BRAKET(1,1),DUMMY)
        CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVLL,
     *             BRAKET(1,2),DUMMY)
        CALL KET2C(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVLL,
     *             BRAKET(1,3),DUMMY)
        CALL BRA2C(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVLL,
     *             BRAKET(1,4),DUMMY)
C
        IF ( SIGMA(1).EQ.SIGMA(2) .AND. SIGMA(3).EQ.SIGMA(4) ) THEN
          CALL AVQQQQ(1,2,3,4,A1234)
          CALL AVQQQQ(3,4,1,2,A3412)
C
          DO 100 I = 1,2
            MSP(1,I) = A1234*CORFAC(1,3,I) + A3412*CORFAC(3,3,I)
            MSP(2,I) = MSP(1,I)
            MSP(3,I) = A1234*CORFAC(1,3,I) + A3412*CORFAC(3,4,I)
            MSP(4,I) = A1234*CORFAC(1,4,I) + A3412*CORFAC(3,3,I)
            MSP(5,I) = A1234*CORFAC(1,4,I) + A3412*CORFAC(3,4,I)
            MSP(6,I) = MSP(5,I)
            DO 100 J = 1,6
              MSM(J,I) = MSP(J,I)
100       CONTINUE
C
          IF ( SIGMA(1).EQ.SIGMA(3) ) THEN
            CALL AVQQQQ(3,2,1,4,A3214)
            CALL AVQQQQ(1,4,3,2,A1432)
            DO 110 I = 1,2
              MH(1,I) = A3214*CORFAC(3,3,I) + A1432*CORFAC(1,3,I)
              MH(6,I) = A3214*CORFAC(3,4,I) + A1432*CORFAC(1,4,I)
              DO 110 J = 1,6,5
                MSP(J,I) = MSP(J,I) - MH(J,I)
                MSM(J,I) = MSM(J,I) + MH(J,I)
110         CONTINUE
          ENDIF
C
        ELSEIF ( SIGMA(1).EQ.SIGMA(4) .AND. SIGMA(3).EQ.SIGMA(2) ) THEN
          CALL AVQQQQ(1,4,3,2,A1432)
          CALL AVQQQQ(3,2,1,4,A3214)
          DO 120 I = 1,2
            MSP(1,I) = A1432*CORFAC(1,3,I) + A3214*CORFAC(3,3,I)
            MSP(6,I) = A1432*CORFAC(1,4,I) + A3214*CORFAC(3,4,I)
            MSM(1,I) = MSP(1,I)
            MSM(6,I) = MSP(6,I)
120       CONTINUE
C
        ELSE
          WRITE(*,*) ' Helicity nonconserving amplitude called ',
     1               'in VQQQQ (V = photon,Z):'
          WRITE(*,*) ' SIGMA  = ',SIGMA
          RETURN
        ENDIF
C
      ELSEIF ( BOS.EQ.3 .OR. BOS.EQ.4 .OR. BOS.EQ.30 .OR.
     1         BOS.EQ.40 ) THEN
C
C                 This part is for V = W+-
C
        IF ( SIGMA(1).EQ.-1 .AND. SIGMA(2).EQ.-1 .AND.
     1       SIGMA(3).EQ.SIGMA(4) ) THEN
C
          CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVLL,
     *               BRAKET(1,1),DUMMY)
          CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVLL,
     *               BRAKET(1,2),DUMMY)
          CALL AVQQQQ(1,2,3,4,A1234)
          A1234 = A1234*CORFAC(1,3,1)
          DO 130 J = 1,3
            MSP(J,1) = A1234
130         MSM(J,1) = A1234
C
          IF ( SIGMA(3).EQ.-1 ) THEN
            CALL KET2C(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVLL,
     *                 BRAKET(1,3),DUMMY)
            CALL BRA2C(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVLL,
     *               BRAKET(1,4),DUMMY)
            CALL AVQQQQ(1,4,3,2,A1432)
            CALL AVQQQQ(3,2,1,4,A3214)
            A1432 = A1432*CORFAC(1,3,1)
            A3214 = A3214*CORFAC(3,3,1)
            MSP(2,1) = MSP(2,1) - A3214
            MSM(2,1) = MSM(2,1) + A3214
            MSP(3,1) = MSP(3,1) - A1432
            MSM(3,1) = MSM(3,1) + A1432
          ENDIF
C
        ELSEIF ( SIGMA(1).EQ.-1 .AND. SIGMA(4).EQ.-1 ) THEN
          CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVLL,
     *               BRAKET(1,1),DUMMY)
          CALL BRA2C(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVLL,
     *               BRAKET(1,4),DUMMY)
          CALL AVQQQQ(1,4,3,2,A1432)
          MSP(3,1) = A1432*CORFAC(1,3,1)
          MSM(3,1) = MSP(3,1)
C
        ELSEIF ( SIGMA(3).EQ.-1 .AND. SIGMA(2).EQ.-1 ) THEN
          CALL KET2C(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVLL,
     *               BRAKET(1,3),DUMMY)
          CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVLL,
     *               BRAKET(1,2),DUMMY)
          CALL AVQQQQ(3,2,1,4,A3214)
          MSP(2,1) = A3214*CORFAC(3,3,1)
          MSM(2,1) = MSP(2,1)
C
        ELSE
          WRITE(*,*) ' Helicity nonconserving amplitude called',
     *               ' in VQQQQ (V = W+,W-):'
          WRITE(*,*) ' SIGMA  = ',SIGMA
          RETURN
        ENDIF
C
      ELSE
        WRITE(*,*) ' Unrecognized vector boson V in VQQQQ.'
        WRITE(*,*) ' BOS = ',BOS
        RETURN
      ENDIF
C
C  Get overall factor F3, forget about sign factor FSIGN(1)*FSIGN(2)*
C                                                  FSIGN(3)*FSIGN(4)
      F3HAT = CLR(3,5,1)**2 * OMFACF
C
C  ... and now the final answer. The color factors are for N = 3 in
C  table 1 of HZ
C
      IF ( BOS.EQ.1 .OR. BOS.EQ.2 .OR. BOS.EQ.10 ) THEN
        CPMFAC = (CPLUS+CMINUS) * F3HAT**2
        DO 140 I = 1,2
          RVUUUUO(I) = CPLUS * (dble(MSP(1,I))**2+dimag(MSP(1,I))**2 )
     1                +CMINUS* (dble(MSM(1,I))**2+dimag(MSM(1,I))**2 )
          RVUUUUO(I) = RVUUUUO(I) * F3HAT**2
C  assign color weights
          M1 = MSP(1,I)+MSM(1,I)
          M2 = MSP(1,I)-MSM(1,I)
          r1 = dble(M1)**2 + dimag(M1)**2
          r2 = dble(M2)**2 + dimag(M2)**2
          if ( (r1+r2).gt.0d0 ) then
             c14u(I) = r1/(r1+r2)
             c12u(I) = r2/(r1+r2)
          else
             c14u(I) = 0.5d0 
             c12u(I) = 0.5d0
          endif
C
          RVUUCCO(I) = CPMFAC *
     1                (dble(MSP(2,I))**2+dimag(MSP(2,I))**2 )
          RVUUDDO(I) = CPMFAC *
     1                (dble(MSP(3,I))**2+dimag(MSP(3,I))**2 )
          RVDDUUO(I) = CPMFAC *
     1                (dble(MSP(4,I))**2+dimag(MSP(4,I))**2 )
          RVDDSSO(I) = CPMFAC *
     1                (dble(MSP(5,I))**2+dimag(MSP(5,I))**2 )
C
          RVDDDDO(I) = CPLUS * (dble(MSP(6,I))**2+dimag(MSP(6,I))**2 )
     1               +CMINUS* (dble(MSM(6,I))**2+dimag(MSM(6,I))**2 )
          RVDDDDO(I) = RVDDDDO(I) * F3HAT**2
C  assign color weights
          M1 = MSP(6,I)+MSM(6,I)
          M2 = MSP(6,I)-MSM(6,I)
          r1 = dble(M1)**2 + dimag(M1)**2
          r2 = dble(M2)**2 + dimag(M2)**2
          if ( (r1+r2).gt.0d0 ) then
             c14d(I) = r1/(r1+r2)
             c12d(I) = r2/(r1+r2)
          else
             c14d(I) = 0.5d0 
             c12d(I) = 0.5d0
          endif

140     CONTINUE
        if (lcol) then
           do I =1,2
              RVUUUU(I) = RVUUUUO(I)
              RVUUCC(I) = RVUUCCO(I)
              RVUUDD(I) = RVUUDDO(I)
              RVDDUU(I) = RVDDUUO(I)
              RVDDSS(I) = RVDDSSO(I)
              RVDDDD(I) = RVDDDDO(I)
           enddo
        else
c
           do ic = 1,2
              sign = -2*ic + 3
              u = 2*sign        ! up quark
              c = u             ! charm
              d = sign          ! down
              s = d             ! strange
              do i3 = 0,1       !one color structure
                 RVcNI(u,c,i3) = RVUUCCO(ic)
                 RVcNI(u,d,i3) = RVUUDDO(ic)
                 RVcNI(d,u,i3) = RVDDUUO(ic)
                 RVcNI(d,s,i3) = RVDDSSO(ic)
              enddo
              RVcI(d,0) = RVDDDDO(ic)
              RVcI(d,1) = c14d(ic) * RVcI(d,0)
              RVcI(d,2) = c12d(ic) * RVcI(d,0)
              RVcI(u,0) = RVUUUUO(ic)
              RVcI(u,1) = c14u(ic) * RVcI(u,0)
              RVcI(u,2) = c12u(ic) * RVcI(u,0)
           enddo
c
        endif
            
      ELSE
        RW00O = (CPLUS+CMINUS) * F3HAT**2 *
     1          (dble(MSP(1,1))**2+dimag(MSP(1,1))**2 )
C
        RW13O = CPLUS * (dble(MSP(2,1))**2+dimag(MSP(2,1))**2 )
     1         +CMINUS* (dble(MSM(2,1))**2+dimag(MSM(2,1))**2 )
        RW13O = RW13O * F3HAT**2
C  assign color weights
        M1 = MSP(2,1)+MSM(2,1)
        M2 = MSP(2,1)-MSM(2,1)
        r1 = dble(M1)**2 + dimag(M1)**2
        r2 = dble(M2)**2 + dimag(M2)**2
        if ( (r1+r2).gt.0d0 ) then
           cw14_13 = r1/(r1+r2)
           cw12_13 = r2/(r1+r2)
        else
           cw14_13 = 0.5d0
           cw12_13 = 0.5d0
        endif
C
        RW24O = CPLUS * (dble(MSP(3,1))**2+dimag(MSP(3,1))**2 )
     1        +CMINUS* (dble(MSM(3,1))**2+dimag(MSM(3,1))**2 )
        RW24O = RW24O * F3HAT**2
C  assign color weights
        M1 = MSP(3,1)+MSM(3,1)
        M2 = MSP(3,1)-MSM(3,1)
        r1 = dble(M1)**2 + dimag(M1)**2
        r2 = dble(M2)**2 + dimag(M2)**2
        if ( (r1+r2).gt.0d0 ) then
           cw14_24 = r1/(r1+r2)
           cw12_24 = r2/(r1+r2)
        else
           cw14_24 = 0.5d0
           cw12_24 = 0.5d0
        endif
c      ENDIF
c
      if(lcol) then
         RW00 = RW00O
         RW13 = RW13O
         RW24 = RW24O
      else
         RWc00(0) = RW00O
         RWc00(1) = RWc00(0)
         RWc13(0) = RW13O
         RWc13(1) = RWc13(0)*cw14_13
         RWc13(2) = RWc13(0)*cw12_13
         RWc24(0) = RW24O
         RWc24(1) = RWc24(0)*cw14_24
         RWc24(2) = RWc24(0)*cw12_24   
      endif
      endif
C
      RETURN
      END
C
C************ AVQQQQ ***************************************************
C
      SUBROUTINE AVQQQQ(N1,N2,N3,N4,A)
      implicit none
      INTEGER NF, NG, NF4, NGPL1
      PARAMETER (NF=4, NG=0, NF4=NF*4,NGPL1=NG+1)
C
C  AVQQQQ calculates the basic Wqqqq amplitude    A(1,2,3,4)/F_3
C  of eq.4.2 in HZ.
C
C  INPUT : Indices N1,N2,N3,N4 identifying the four quarks
C  ------
C
C  OUTPUT: A, the complex value of the amplitude
C  -------
C
C             The full information on the quarks is transferred via
C             common block / BVQQQQ / from subroutine VQQQQ
C
      INTEGER N1, N2, N3, N4, SIGMA(NF)
      real*8 P(0:3,NF)
      complex*16  CHIF(2,NF), BRAKET(2,NF,NGPL1), A
C
C  Information passed to AVQQQQ by common block:
      COMMON /BVQQQQ/ CHIF, BRAKET, P, SIGMA
C
C  Further local variables
      real*8  PROP
      complex*16  Z1,Z2,Z3,Z4, ZJ0,ZJ1,ZJ2,ZJ3, J43S0,J43S1,J43S2,J43S3
C
C  Check helicity conservation
      IF ( SIGMA(N1).NE.SIGMA(N2).OR.SIGMA(N3).NE.SIGMA(N4) ) THEN
        WRITE(*,*) ' Wrong helicity assignment in AVQQQQ: '
        WRITE(*,*) ' SIGMA = ',SIGMA(N1),SIGMA(N2),' ; ',
     1                         SIGMA(N3),SIGMA(N4)
        A = 0.0d0
        RETURN
      ENDIF
C
C  Form the current  <2,k|mu|1>+<2|mu|k,1> ).......
      Z1 = BRAKET(1,N2,1)*CHIF(1,N1) + CHIF(1,N2)*BRAKET(1,N1,1)
      Z2 = BRAKET(2,N2,1)*CHIF(2,N1) + CHIF(2,N2)*BRAKET(2,N1,1)
      Z3 = BRAKET(1,N2,1)*CHIF(2,N1) + CHIF(1,N2)*BRAKET(2,N1,1)
      Z4 = BRAKET(2,N2,1)*CHIF(1,N1) + CHIF(2,N2)*BRAKET(1,N1,1)
C
      ZJ0 = Z1+Z2
C  The missing factors sigma(n1) and -i (for zj2) (according to
C  eqn.2.27 in HZ) is absorbed in the contraction with J43:
      ZJ1 = Z3+Z4
      ZJ2 = Z3-Z4
      ZJ3 = Z1-Z2
C
C  Form the current J43 * (p_4 - p_3)**2:
      Z1 = CHIF(1,N4)*CHIF(1,N3)
      Z2 = CHIF(2,N4)*CHIF(2,N3)
      Z3 = CHIF(1,N4)*CHIF(2,N3)
      Z4 = CHIF(2,N4)*CHIF(1,N3)
C
      J43S0 = Z1+Z2
      J43S1 = Z3+Z4
      J43S2 = Z3-Z4
      J43S3 = Z1-Z2
C
C  ......and contract the two
      IF ( SIGMA(N1).EQ.SIGMA(N3) ) THEN
        Z1 = ZJ0*J43S0 - ZJ1*J43S1 + ZJ2*J43S2 - ZJ3*J43S3
      ELSE
        Z1 = ZJ0*J43S0 + ZJ1*J43S1 - ZJ2*J43S2 + ZJ3*J43S3
      ENDIF
C
C  calculate the gluon propagator:
      PROP = -P(0,N4)*P(0,N3)+P(1,N4)*P(1,N3)+P(2,N4)*P(2,N3)+
     *        P(3,N4)*P(3,N3)
      PROP = 0.5d0 / PROP
C
C  and now the result:
      A = Z1*PROP
      RETURN
      END
C
C************ RV4Q1G, AV4Q1G *******************************************
C
      SUBROUTINE RV4Q1G (PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,KBAR,KSIGN,VPOL,
     3               RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
      IMPLICIT NONE
      INTEGER NF, NG, NF4, NGPL1, NF48
      PARAMETER (NF=4, NG=1, NF4=NF*4,NF48=NF*48,NGPL1=NG+1)
C
C  RV4Q1G calculates the color-summed amplitude squared of the Vqqqqg
C  amplitude (eqns. 5.1 to 5.11 of HZ ) for fixed fermion helicities
C  and fixed V polarization. All fermions are assumed to be massless,
C  except in the case BOS=10 and LFLAVR=2, i.e. for NC processes with
C  incoming e+ or e-. In this case electron mass effects are included
C  in order to allow a reliable simulation of ep scattering at extremely
C  low Q**2 ( << m_e**2 ).
C
C  5 different channels are considered:
C     CHANEL = 1:   real photon or Z
C     CHANEL = 2:   real W+ or W-
C     CHANEL = 3:   virtual photon and Z coupled to e+e-
C     CHANEL = 4:   virtual photon and Z coupled to nu nu-bar
C     CHANEL = 5:   virtual W+- coupled to nu e
C
C  OUTPUT are the 6 2-vectors RVUUUU,...,RVDDDD for Z/photon and the
C  real numbers RW00, RW13, RW24 for W's. As before the 2 elements
C  of each Z/photon result correspond to the direct and the charge
C  conjugation related channel. The last 4 letters of the output
C  variables identify the various flavor combinations:
C
C  RVUUUU       quark1,...,quark4 are all up-type quarks of the same
C               family
C  RVUUCC       quark1,...,quark4 are all up type, but 1 and 2 belong to
C               a family different from 3 and 4
C  etc. for photon/Z
C
C               For W's the flavors of quarks 3 and 4 are always assumed
C               to be identical, while the flavors of quarks 1 and 2 are
C               always assumed to be both entries of an SU(2) doublet.
C               This leaves three possibilities for the relative flavors
C               of the first pair (1 and 2) and the second pair of
C               quarks:
C  RW00         the W couples to quarks 1 and 2, the flavors of quark 3
C               and 4 are different from either.
C  RW13         the flavors of the first and the third quark are
C               identical
C  RW24         the flavors of quarks 2 and 4 are identical
C
      real*8  PBAR(0:3,NF), QBAR(0:4), KBAR(0:4), LBAR(0:3,2)
      real*8  P   (0:3,NF), Q   (0:4), K   (0:4), L   (0:3,2)
C
      INTEGER FSIGN(NF), FPOL(NF), SIGMA(NF),
     1        GSIGN, GPOL,
     2        BOS, KSIGN, VPOL, LSIGN(2), LPOL(2), LFLAVR(2)
C
      real*8 RVUUUU(2), RVUUCC(2), RVUUDD(2), RVDDUU(2),
     1     RVDDSS(2), RVDDDD(2), RW00, RW13, RW24,
     2     EPSG(0:3), EPSV(0:3), JVXYZ(0:3,3), OMFACF, DUMMY(0:4)
C    New local variables:
      real*8  RVUUUUO(2), RVUUCCO(2), RVUUDDO(2), RVDDUUO(2),
     1     RVDDSSO(2), RVDDDDO(2), RW00O, RW13O, RW24O
      complex*16  CHIF(2,NF), JVLL(0:3), CORFAC(NF,3:4,2),
     1         BRAKET(2,NF,NGPL1), JV10(0:3,NF,3:4,2,-1:1)
C    New format
      real*8 RVcI(-2:2,0:4),RVcNI(-2:2,-2:2,0:4),
     1       RWc00(0:4),RWc13(0:4),RWc24(0:4)
c
      integer u,d,s,c,sign ! quark flavor integers
      logical lcol
c
C  Further local variables
      INTEGER MU,IFL,I,J,IS1,IS2,IL,CHANEL
      integer ifl1,ifl3,i3      ! flavor and color indices
      real*8  F5HAT, FINFAC, PQ(0:4,NF), PK(0:4,NF)
      complex*16  A1234(-1:1,-1:1), A3214(-1:1,-1:1),
     1         A1432(-1:1,-1:1), A3412(-1:1,-1:1),
     2         HA1234(-1:1,-1:1,3:4,2), HA3214(-1:1,-1:1,3:4,2),
     3         HA1432(-1:1,-1:1,3:4,2), HA3412(-1:1,-1:1,3:4,2),
     4         MSPP(6,2), MSPM(6,2), MSMP(6,2), MSMM(6,2),
     5         DEBK(2,NF,NGPL1,NGPL1)
      complex*16  B1234(-1:1,-1:1,3), B3214(-1:1,-1:1,3),
     1       B1432(-1:1,-1:1,3), B3412(-1:1,-1:1,3),
     2       HB1234(-1:1,-1:1,3:4,2,-1:1), HB3214(-1:1,-1:1,3:4,2,-1:1),
     3       HB1432(-1:1,-1:1,3:4,2,-1:1), HB3412(-1:1,-1:1,3:4,2,-1:1),
     4       MSPP10(6,2,-1:1), MSPM10(6,2,-1:1), 
     5       MSMP10(6,2,-1:1), MSMM10(6,2,-1:1)
      complex*16 MSC(6,2,4)
      real*8 CO(6,2,4),COI(-2:2,0:4),CONI(-2:2,-2:2,0:4),
     1       COSUM(6,2),CW(3,4),CWSUM(3)
      
C
C  DEBK stores "Double Emission Bra Kets".
C      NF index:   fermion index as in BRAKET
C      3rd index:  index of boson emitted closest to fermion
C      4th index:  index of second boson emitted
C
C  Information passed to AV4Q1G by common block:
c      COMMON /BV4Q1G/ CHIF, BRAKET, DEBK, P, SIGMA, Q, EPSG, PK
      COMMON /BV4Q1G/ CHIF, BRAKET, DEBK, P, Q, EPSG, PK, SIGMA
C
C  Information on coupling constants from KOPPEL
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
C
      DATA JVLL /4*(0.0d0, 0.0d0)/, CORFAC /NF4*(0.0d0, 0.0d0)/
      DATA JVXYZ /0D0,1D0,0D0,0D0,  0D0,0D0,1D0,0D0,  0D0,0D0,0D0,1D0/  
      DATA JV10 /NF48*(0D0,0D0)/
C
      DO 10 I=1,2
        RVUUUU(I) = 0.0d0
        RVUUCC(I) = 0.0d0
        RVUUDD(I) = 0.0d0
        RVDDUU(I) = 0.0d0
        RVDDSS(I) = 0.0d0
        RVDDDD(I) = 0.0d0
10    CONTINUE
      IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
        CHANEL = 1
      ELSE 
        CHANEL = -10
      ENDIF
      lcol = .true.
      GOTO 20
c      
      ENTRY RVc4Q1G(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,KBAR,KSIGN,VPOL,
     3               RVcI,RVcNI)
      do ifl1 = -2,2
         do i3 = 0,4
            RVcI(ifl1,i3) = 0.0d0
            do ifl3 = -2,2
               RVcNI(ifl1,ifl3,i3) = 0.0d0
            enddo
         enddo
      enddo
      IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
         CHANEL = 1
      ELSE 
         CHANEL = -10
      ENDIF
      lcol = .false.
      goto 20
C
C  Entry for real W
      ENTRY RW4Q1G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,KBAR,KSIGN,VPOL,
     3              RW00,RW13,RW24)
      RW00 = 0.0d0
      RW13 = 0.0d0
      RW24 = 0.0d0
      IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
        CHANEL = 2
      ELSE 
        CHANEL = -20
      ENDIF
      lcol = .true.
      goto 20

      ENTRY RWc4Q1G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,KBAR,KSIGN,VPOL,
     3              RWc00,RWc13,RWc24)
      do i3 = 0,4
         RWc00(i3) = 0.0d0
         RWc13(i3) = 0.0d0
         RWc24(i3) = 0.0d0
      enddo
      IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
        CHANEL = 2
      ELSE 
        CHANEL = -20
      ENDIF
      lcol = .false.
C
20    CONTINUE
C
C  Get polarisation vector, momentum K and Vqq coupling for onshell
C  boson  V
      SIGMA(1) = FSIGN(1)*FPOL(1)
      SIGMA(2) = FSIGN(2)*FPOL(2)
      SIGMA(3) = FSIGN(3)*FPOL(3)
      SIGMA(4) = FSIGN(4)*FPOL(4)
      CALL POLVEC(KBAR,VPOL,EPSV)
      K(0) = KSIGN*KBAR(0)
      JVLL(0) = 0.0d0
      DO 30 MU = 1,3
        K(MU) = KSIGN*KBAR(MU)
30       JVLL(MU) = EPSV(MU)
      K(4) = KBAR(4)**2
      DO 40 IFL = 3,4
        DO 40 I = 1,3,2
          CORFAC(I,IFL,1) = CLR(IFL,BOS,SIGMA(I))
40        CORFAC(I,IFL,2) = CLR(IFL,BOS,-SIGMA(I))
      GOTO 50
C
C  Now the entry for offshell electroweak bosons
      ENTRY LV4Q1G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3          RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
      DO 60 I=1,2
        RVUUUU(I) = 0.0d0
        RVUUCC(I) = 0.0d0
        RVUUDD(I) = 0.0d0
        RVDDUU(I) = 0.0d0
        RVDDSS(I) = 0.0d0
        RVDDDD(I) = 0.0d0
60    CONTINUE
c      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.2.AND.LFLAVR(2)/10.EQ.2 ) THEN
      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.20.AND.LFLAVR(2)/10.EQ.20 ) THEN
        CHANEL = 3
      ELSEIF (BOS.EQ.10) THEN 
        CHANEL = 4
      ELSE 
        CHANEL = -30
      ENDIF
      lcol = .true.
      GOTO 70
      ENTRY LVc4Q1G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3          RVcI,RVcNI)
c     
      do ifl1 = -2,2
         do i3 = 0,4
            RVcI(ifl1,i3) = 0.0d0
            do ifl3 = -2,2
               RVcNI(ifl1,ifl3,i3) = 0.0d0
            enddo
         enddo
      enddo
c
c      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.2.AND.LFLAVR(2)/10.EQ.2 ) THEN
      IF (BOS.EQ.10.AND.LFLAVR(1)/10.EQ.20.AND.LFLAVR(2)/10.EQ.20 ) THEN
        CHANEL = 3
      ELSEIF (BOS.EQ.10) THEN 
        CHANEL = 4
      ELSE 
        CHANEL = -30
      ENDIF
      lcol = .false.
      GOTO 70
      
C
      ENTRY LW4Q1G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3              RW00,RW13,RW24)
     
      RW00 = 0.0d0
      RW13 = 0.0d0
      RW24 = 0.0d0    
      IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
        CHANEL = 5
      ELSE 
        CHANEL = -50
      ENDIF
      lcol = .true.
      goto 70

      ENTRY LWc4Q1G (PBAR,FSIGN,FPOL,
     1              QBAR,GSIGN,GPOL,
     2          BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3              RWc00,RWc13,RWc24)
      do i3 = 0,4
         RWc00(i3) = 0.0d0
         RWc13(i3) = 0.0d0
         RWc24(i3) = 0.0d0
      enddo
      IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
        CHANEL = 5
      ELSE 
        CHANEL = -50
      ENDIF
      lcol = .false.
C
70    CONTINUE
C
      SIGMA(1) = FSIGN(1)*FPOL(1)
      SIGMA(2) = FSIGN(2)*FPOL(2)
      SIGMA(3) = FSIGN(3)*FPOL(3)
      SIGMA(4) = FSIGN(4)*FPOL(4)
C
C  Get the decay current JVLL
      IF ( CHANEL.EQ.3 ) THEN
        CALL DECAYN(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMA,JV10,K)
      ELSE
        CALL DECAYC(LBAR,LSIGN,LPOL,LFLAVR,NF,SIGMA,BOS,
     *            JVLL,CORFAC,K)
      ENDIF
C
C  Now virtual and real bosons merge again
50    CONTINUE
C
C  Get the unbared quark and gluon momenta
      DO 80 MU = 0,3
        P(MU,1) = FSIGN(1)*PBAR(MU,1)
        P(MU,2) = FSIGN(2)*PBAR(MU,2)
        P(MU,3) = FSIGN(3)*PBAR(MU,3)
        P(MU,4) = FSIGN(4)*PBAR(MU,4)
80      Q(MU)   = GSIGN*QBAR(MU)
C
C  Get the 4 quark spinors CHIF and the gluon polarization vector
      CALL CHIOM(NF,PBAR,SIGMA,CHIF,OMFACF)
      CALL POLVEC(QBAR,GPOL,EPSG)
C
C  Get bras and kets <i,q| and |q,i> for gluon emission
      CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),Q,EPSG,
     *           BRAKET(1,1,1),PQ(0,1))
      CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),Q,EPSG,
     *           BRAKET(1,2,1),PQ(0,2))
      CALL KET2R(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),Q,EPSG,
     *           BRAKET(1,3,1),PQ(0,3))
      CALL BRA2R(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),Q,EPSG,
     *           BRAKET(1,4,1),PQ(0,4))
C
      IF ( CHANEL.NE.3 ) THEN
        DO 90 I = 1,2
          DO 90 J = 1,6
            MSPP(J,I) = 0.0d0
            MSPM(J,I) = 0.0d0
            MSMP(J,I) = 0.0d0
            MSMM(J,I) = 0.0d0
90      CONTINUE
      ENDIF
C
C  Distinguishing cases.....
      IF ( CHANEL.EQ.1 .OR. CHANEL.EQ.4 ) THEN
C
C---- This is the part for real photon and Z or decay into neutrinos
C
C  Get bras and kets <i,k| and |k,i>
        CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVLL,
     *             BRAKET(1,1,2),PK(0,1))
        CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVLL,
     *             BRAKET(1,2,2),PK(0,2))
        CALL KET2C(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVLL,
     *             BRAKET(1,3,2),PK(0,3))
        CALL BRA2C(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVLL,
     *             BRAKET(1,4,2),PK(0,4))
C
C  Get bras and kets [pi-q-k,e]|q,i> etc.
        CALL KET2C(BRAKET(1,1,1),.FALSE.,PQ(0,1),SIGMA(1),K,JVLL,
     *             DEBK(1,1,1,2),DUMMY)
        CALL KET2R(BRAKET(1,1,2),.FALSE.,PK(0,1),SIGMA(1),Q,EPSG,
     *             DEBK(1,1,2,1),DUMMY)
        CALL BRA2C(BRAKET(1,2,1),.FALSE.,PQ(0,2),SIGMA(2),K,JVLL,
     *             DEBK(1,2,1,2),DUMMY)
        CALL BRA2R(BRAKET(1,2,2),.FALSE.,PK(0,2),SIGMA(2),Q,EPSG,
     *             DEBK(1,2,2,1),DUMMY)
        CALL KET2C(BRAKET(1,3,1),.FALSE.,PQ(0,3),SIGMA(3),K,JVLL,
     *             DEBK(1,3,1,2),DUMMY)
        CALL KET2R(BRAKET(1,3,2),.FALSE.,PK(0,3),SIGMA(3),Q,EPSG,
     *             DEBK(1,3,2,1),DUMMY)
        CALL BRA2C(BRAKET(1,4,1),.FALSE.,PQ(0,4),SIGMA(4),K,JVLL,
     *             DEBK(1,4,1,2),DUMMY)
        CALL BRA2R(BRAKET(1,4,2),.FALSE.,PK(0,4),SIGMA(4),Q,EPSG,
     *             DEBK(1,4,2,1),DUMMY)
C
        IF ( SIGMA(1).EQ.SIGMA(2) .AND. SIGMA(3).EQ.SIGMA(4) ) THEN
C
          CALL AV4Q1G(1,2,3,4,A1234)
          CALL AV4Q1G(3,4,1,2,A3412)
C
          DO 100 I = 1,2
            DO 110 IFL = 3,4
            DO 110 IS2 = -1,1,2
            DO 110 IS1 = -1,1,2
              HA1234(IS1,IS2,IFL,I) = A1234(IS1,IS2) * CORFAC(1,IFL,I)
              HA3412(IS1,IS2,IFL,I) = A3412(IS1,IS2) * CORFAC(3,IFL,I)
110         CONTINUE
C
            MSPP(1,I) = HA1234( 1, 1,3,I) + HA3412( 1, 1,3,I)
            MSPM(1,I) = HA1234( 1,-1,3,I) - HA3412( 1,-1,3,I)
            MSMP(1,I) = HA1234(-1, 1,3,I) - HA3412(-1, 1,3,I)
            MSMM(1,I) = HA1234(-1,-1,3,I) + HA3412(-1,-1,3,I)
C
            MSPP(2,I) = MSPP(1,I)
            MSPM(2,I) = MSPM(1,I)
            MSMP(2,I) = MSMP(1,I)
            MSMM(2,I) = MSMM(1,I)
C
            MSPP(3,I) = HA1234( 1, 1,3,I) + HA3412( 1, 1,4,I)
            MSPM(3,I) = HA1234( 1,-1,3,I) - HA3412( 1,-1,4,I)
            MSMP(3,I) = HA1234(-1, 1,3,I) - HA3412(-1, 1,4,I)
            MSMM(3,I) = HA1234(-1,-1,3,I) + HA3412(-1,-1,4,I)
C
            MSPP(4,I) = HA1234( 1, 1,4,I) + HA3412( 1, 1,3,I)
            MSPM(4,I) = HA1234( 1,-1,4,I) - HA3412( 1,-1,3,I)
            MSMP(4,I) = HA1234(-1, 1,4,I) - HA3412(-1, 1,3,I)
            MSMM(4,I) = HA1234(-1,-1,4,I) + HA3412(-1,-1,3,I)
C
            MSPP(5,I) = HA1234( 1, 1,4,I) + HA3412( 1, 1,4,I)
            MSPM(5,I) = HA1234( 1,-1,4,I) - HA3412( 1,-1,4,I)
            MSMP(5,I) = HA1234(-1, 1,4,I) - HA3412(-1, 1,4,I)
            MSMM(5,I) = HA1234(-1,-1,4,I) + HA3412(-1,-1,4,I)
C
            MSPP(6,I) = MSPP(5,I)
            MSPM(6,I) = MSPM(5,I)
            MSMP(6,I) = MSMP(5,I)
            MSMM(6,I) = MSMM(5,I)
C
100       CONTINUE
C
          IF ( SIGMA(1).EQ.SIGMA(3) ) THEN
            CALL AV4Q1G(3,2,1,4,A3214)
            CALL AV4Q1G(1,4,3,2,A1432)
C
            DO 120 I = 1,2
C
              MSPP(1,I) = MSPP(1,I) -
     1         A3214(1,1)*CORFAC(3,3,I) - A1432(1,1)*CORFAC(1,3,I)
              MSPM(1,I) = MSPM(1,I) -
     1         A3214(1,-1)*CORFAC(3,3,I) + A1432(1,-1)*CORFAC(1,3,I)
              MSMP(1,I) = MSMP(1,I) +
     1         A3214(-1,1)*CORFAC(3,3,I) - A1432(-1,1)*CORFAC(1,3,I)
              MSMM(1,I) = MSMM(1,I) +
     1         A3214(-1,-1)*CORFAC(3,3,I) + A1432(-1,-1)*CORFAC(1,3,I)
C
              MSPP(6,I) = MSPP(6,I) -
     1         A3214(1,1)*CORFAC(3,4,I) - A1432(1,1)*CORFAC(1,4,I)
              MSPM(6,I) = MSPM(6,I) -
     1         A3214(1,-1)*CORFAC(3,4,I) + A1432(1,-1)*CORFAC(1,4,I)
              MSMP(6,I) = MSMP(6,I) +
     1         A3214(-1,1)*CORFAC(3,4,I) - A1432(-1,1)*CORFAC(1,4,I)
              MSMM(6,I) = MSMM(6,I) +
     1         A3214(-1,-1)*CORFAC(3,4,I) + A1432(-1,-1)*CORFAC(1,4,I)
C
120         CONTINUE
C
          ENDIF
C
        ELSEIF ( SIGMA(1).EQ.SIGMA(4) .AND. SIGMA(3).EQ.SIGMA(2) ) THEN
          CALL AV4Q1G(1,4,3,2,A1432)
          CALL AV4Q1G(3,2,1,4,A3214)
          DO 130 I = 1,2
C
            MSPP(1,I) =
     1         A3214(1,1)*CORFAC(3,3,I) + A1432(1,1)*CORFAC(1,3,I)
            MSPM(1,I) =
     1         A3214(1,-1)*CORFAC(3,3,I) - A1432(1,-1)*CORFAC(1,3,I)
            MSMP(1,I) =
     1       - A3214(-1,1)*CORFAC(3,3,I) + A1432(-1,1)*CORFAC(1,3,I)
            MSMM(1,I) =
     1       - A3214(-1,-1)*CORFAC(3,3,I) - A1432(-1,-1)*CORFAC(1,3,I)
C
            MSPP(6,I) =
     1         A3214(1,1)*CORFAC(3,4,I) + A1432(1,1)*CORFAC(1,4,I)
            MSPM(6,I) =
     1         A3214(1,-1)*CORFAC(3,4,I) - A1432(1,-1)*CORFAC(1,4,I)
            MSMP(6,I) =
     1       - A3214(-1,1)*CORFAC(3,4,I) + A1432(-1,1)*CORFAC(1,4,I)
            MSMM(6,I) =
     1       - A3214(-1,-1)*CORFAC(3,4,I) - A1432(-1,-1)*CORFAC(1,4,I)
C
130       CONTINUE
        ELSE
          WRITE(*,*) ' Helicity nonconserving amplitude called',
     1               ' in V4Q1G (V = photon,Z):'
          WRITE(*,*) ' SIGMA  = ',SIGMA
          RETURN
        ENDIF

      ELSEIF ( CHANEL.EQ.3 ) THEN
C
C   This is the part for VIRTUAL photon and Z. Amplitudes are
C   calculated for both the spin nonflip and flip part of the 
C   lepton current.
C
        DO 91 I = 1,2
          DO 91 J = 1,6
            DO 91 IL=-1,1,2
              MSPP10(J,I,IL) = 0.0d0
              MSPM10(J,I,IL) = 0.0d0
              MSMP10(J,I,IL) = 0.0d0
              MSMM10(J,I,IL) = 0.0d0
91      CONTINUE
C  Do the matrixelements for 3 components of lepton vector:
        DO 92 MU = 1,3

C  Get bras and kets <i,k| and |k,i>
        CALL KET2R(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVXYZ(0,MU),
     *             BRAKET(1,1,2),PK(0,1))
        CALL BRA2R(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVXYZ(0,MU),
     *             BRAKET(1,2,2),PK(0,2))
        CALL KET2R(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVXYZ(0,MU),
     *             BRAKET(1,3,2),PK(0,3))
        CALL BRA2R(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVXYZ(0,MU),
     *             BRAKET(1,4,2),PK(0,4))
C
C  Get bras and kets [pi-q-k,e]|q,i> etc.
        CALL KET2R(BRAKET(1,1,1),.FALSE.,PQ(0,1),SIGMA(1),K,
     *             JVXYZ(0,MU),DEBK(1,1,1,2),DUMMY)
        CALL KET2R(BRAKET(1,1,2),.FALSE.,PK(0,1),SIGMA(1),Q,EPSG,
     *             DEBK(1,1,2,1),DUMMY)
        CALL BRA2R(BRAKET(1,2,1),.FALSE.,PQ(0,2),SIGMA(2),K,
     *             JVXYZ(0,MU),DEBK(1,2,1,2),DUMMY)
        CALL BRA2R(BRAKET(1,2,2),.FALSE.,PK(0,2),SIGMA(2),Q,EPSG,
     *             DEBK(1,2,2,1),DUMMY)
        CALL KET2R(BRAKET(1,3,1),.FALSE.,PQ(0,3),SIGMA(3),K,
     *             JVXYZ(0,MU),DEBK(1,3,1,2),DUMMY)
        CALL KET2R(BRAKET(1,3,2),.FALSE.,PK(0,3),SIGMA(3),Q,EPSG,
     *             DEBK(1,3,2,1),DUMMY)
        CALL BRA2R(BRAKET(1,4,1),.FALSE.,PQ(0,4),SIGMA(4),K,
     *             JVXYZ(0,MU),DEBK(1,4,1,2),DUMMY)
        CALL BRA2R(BRAKET(1,4,2),.FALSE.,PK(0,4),SIGMA(4),Q,EPSG,
     *             DEBK(1,4,2,1),DUMMY)
C
        IF ( SIGMA(1).EQ.SIGMA(2) .AND. SIGMA(3).EQ.SIGMA(4) ) THEN
          CALL AV4Q1G(1,2,3,4,B1234(-1,-1,MU) )
          CALL AV4Q1G(3,4,1,2,B3412(-1,-1,MU) )
C
          IF ( SIGMA(1).EQ.SIGMA(3) ) THEN
            CALL AV4Q1G(3,2,1,4,B3214(-1,-1,MU) )
            CALL AV4Q1G(1,4,3,2,B1432(-1,-1,MU) )
          ENDIF
C
        ELSEIF ( SIGMA(1).EQ.SIGMA(4) .AND. SIGMA(3).EQ.SIGMA(2) ) THEN
          CALL AV4Q1G(1,4,3,2,B1432(-1,-1,MU) )
          CALL AV4Q1G(3,2,1,4,B3214(-1,-1,MU) )
        ENDIF
C
92      CONTINUE

        IF ( SIGMA(1).EQ.SIGMA(2) .AND. SIGMA(3).EQ.SIGMA(4) ) THEN
          DO 101 IL = -1,1,2
          DO 101 I = 1,2
            DO 111 IFL = 3,4
            DO 111 IS2 = -1,1,2
            DO 111 IS1 = -1,1,2
              HB1234(IS1,IS2,IFL,I,IL) = 
     *               -B1234(IS1,IS2,1) * JV10(1, 1,IFL,I,IL)-
     *                B1234(IS1,IS2,2) * JV10(2, 1,IFL,I,IL)-
     *                B1234(IS1,IS2,3) * JV10(3, 1,IFL,I,IL)
              HB3412(IS1,IS2,IFL,I,IL) = 
     *               -B3412(IS1,IS2,1) * JV10(1, 3,IFL,I,IL)-
     *                B3412(IS1,IS2,2) * JV10(2, 3,IFL,I,IL)-
     *                B3412(IS1,IS2,3) * JV10(3, 3,IFL,I,IL)
111         CONTINUE
C                                 
            MSPP10(1,I,IL) = HB1234( 1, 1,3,I,IL) + HB3412( 1, 1,3,I,IL)
            MSPM10(1,I,IL) = HB1234( 1,-1,3,I,IL) - HB3412( 1,-1,3,I,IL)
            MSMP10(1,I,IL) = HB1234(-1, 1,3,I,IL) - HB3412(-1, 1,3,I,IL)
            MSMM10(1,I,IL) = HB1234(-1,-1,3,I,IL) + HB3412(-1,-1,3,I,IL)
C
            MSPP10(2,I,IL) = MSPP10(1,I,IL)
            MSPM10(2,I,IL) = MSPM10(1,I,IL)
            MSMP10(2,I,IL) = MSMP10(1,I,IL)
            MSMM10(2,I,IL) = MSMM10(1,I,IL)
C
            MSPP10(3,I,IL) = HB1234( 1, 1,3,I,IL) + HB3412( 1, 1,4,I,IL)
            MSPM10(3,I,IL) = HB1234( 1,-1,3,I,IL) - HB3412( 1,-1,4,I,IL)
            MSMP10(3,I,IL) = HB1234(-1, 1,3,I,IL) - HB3412(-1, 1,4,I,IL)
            MSMM10(3,I,IL) = HB1234(-1,-1,3,I,IL) + HB3412(-1,-1,4,I,IL)
C
            MSPP10(4,I,IL) = HB1234( 1, 1,4,I,IL) + HB3412( 1, 1,3,I,IL)
            MSPM10(4,I,IL) = HB1234( 1,-1,4,I,IL) - HB3412( 1,-1,3,I,IL)
            MSMP10(4,I,IL) = HB1234(-1, 1,4,I,IL) - HB3412(-1, 1,3,I,IL)
            MSMM10(4,I,IL) = HB1234(-1,-1,4,I,IL) + HB3412(-1,-1,3,I,IL)
C
            MSPP10(5,I,IL) = HB1234( 1, 1,4,I,IL) + HB3412( 1, 1,4,I,IL)
            MSPM10(5,I,IL) = HB1234( 1,-1,4,I,IL) - HB3412( 1,-1,4,I,IL)
            MSMP10(5,I,IL) = HB1234(-1, 1,4,I,IL) - HB3412(-1, 1,4,I,IL)
            MSMM10(5,I,IL) = HB1234(-1,-1,4,I,IL) + HB3412(-1,-1,4,I,IL)
C
            MSPP10(6,I,IL) = MSPP10(5,I,IL)
            MSPM10(6,I,IL) = MSPM10(5,I,IL)
            MSMP10(6,I,IL) = MSMP10(5,I,IL)
            MSMM10(6,I,IL) = MSMM10(5,I,IL)
C
101       CONTINUE
C
          IF ( SIGMA(1).EQ.SIGMA(3) ) THEN
C
            DO 122 IL = -1,1,2
            DO 122 I = 1,2
              DO 121 IFL = 3,4
              DO 121 IS2 = -1,1,2
              DO 121 IS1 = -1,1,2
                HB3214(IS1,IS2,IFL,I,IL) = 
     *                 -B3214(IS1,IS2,1) * JV10(1, 3,IFL,I,IL)-
     *                  B3214(IS1,IS2,2) * JV10(2, 3,IFL,I,IL)-
     *                  B3214(IS1,IS2,3) * JV10(3, 3,IFL,I,IL)
                HB1432(IS1,IS2,IFL,I,IL) = 
     *                 -B1432(IS1,IS2,1) * JV10(1, 1,IFL,I,IL)-
     *                  B1432(IS1,IS2,2) * JV10(2, 1,IFL,I,IL)-
     *                  B1432(IS1,IS2,3) * JV10(3, 1,IFL,I,IL)
121         CONTINUE
C
              MSPP10(1,I,IL) = MSPP10(1,I,IL) -
     1                       HB3214(1,1,3,I,IL) - HB1432(1,1,3,I,IL)
              MSPM10(1,I,IL) = MSPM10(1,I,IL) -
     1                       HB3214(1,-1,3,I,IL) + HB1432(1,-1,3,I,IL)
              MSMP10(1,I,IL) = MSMP10(1,I,IL) +
     1                       HB3214(-1,1,3,I,IL) - HB1432(-1,1,3,I,IL)
              MSMM10(1,I,IL) = MSMM10(1,I,IL) +
     1                       HB3214(-1,-1,3,I,IL) + HB1432(-1,-1,3,I,IL)
C
              MSPP10(6,I,IL) = MSPP10(6,I,IL) -
     1                       HB3214(1,1,4,I,IL) - HB1432(1,1,4,I,IL)
              MSPM10(6,I,IL) = MSPM10(6,I,IL) -
     1                       HB3214(1,-1,4,I,IL) + HB1432(1,-1,4,I,IL)
              MSMP10(6,I,IL) = MSMP10(6,I,IL) +
     1                       HB3214(-1,1,4,I,IL) - HB1432(-1,1,4,I,IL)
              MSMM10(6,I,IL) = MSMM10(6,I,IL) +
     1                       HB3214(-1,-1,4,I,IL) + HB1432(-1,-1,4,I,IL)
C
122         CONTINUE
C
          ENDIF
C
        ELSEIF ( SIGMA(1).EQ.SIGMA(4) .AND. SIGMA(3).EQ.SIGMA(2) ) THEN
C
          DO 132 IL = -1,1,2
          DO 132 I = 1,2
            DO 131 IFL = 3,4
            DO 131 IS2 = -1,1,2
            DO 131 IS1 = -1,1,2
              HB3214(IS1,IS2,IFL,I,IL) = 
     *                 -B3214(IS1,IS2,1) * JV10(1, 3,IFL,I,IL)-
     *                  B3214(IS1,IS2,2) * JV10(2, 3,IFL,I,IL)-
     *                  B3214(IS1,IS2,3) * JV10(3, 3,IFL,I,IL)
              HB1432(IS1,IS2,IFL,I,IL) = 
     *                 -B1432(IS1,IS2,1) * JV10(1, 1,IFL,I,IL)-
     *                  B1432(IS1,IS2,2) * JV10(2, 1,IFL,I,IL)-
     *                  B1432(IS1,IS2,3) * JV10(3, 1,IFL,I,IL)
131         CONTINUE
C
            MSPP10(1,I,IL) = 
     1                       HB3214(1,1,3,I,IL) + HB1432(1,1,3,I,IL)
            MSPM10(1,I,IL) = 
     1                       HB3214(1,-1,3,I,IL) - HB1432(1,-1,3,I,IL)
            MSMP10(1,I,IL) = 
     1                     - HB3214(-1,1,3,I,IL) + HB1432(-1,1,3,I,IL)
            MSMM10(1,I,IL) = 
     1                     - HB3214(-1,-1,3,I,IL) - HB1432(-1,-1,3,I,IL)
C
            MSPP10(6,I,IL) =
     1                       HB3214(1,1,4,I,IL) + HB1432(1,1,4,I,IL)
            MSPM10(6,I,IL) = 
     1                       HB3214(1,-1,4,I,IL) - HB1432(1,-1,4,I,IL)
            MSMP10(6,I,IL) = 
     1                     - HB3214(-1,1,4,I,IL) + HB1432(-1,1,4,I,IL)
            MSMM10(6,I,IL) =
     1                     - HB3214(-1,-1,4,I,IL) - HB1432(-1,-1,4,I,IL)
C
132       CONTINUE
        ELSE
          WRITE(*,*) ' Helicity nonconserving amplitude called',
     1               ' in V4Q1G (V = photon,Z):'
          WRITE(*,*) ' SIGMA  = ',SIGMA
          RETURN
        ENDIF
C
      ELSEIF ( CHANEL.EQ.2 .OR. CHANEL.EQ.5) THEN
C
C                 This part is for V = W+-
C
        IF ( SIGMA(1).EQ.-1 .AND. SIGMA(2).EQ.-1 .AND.
     1       SIGMA(3).EQ.SIGMA(4) ) THEN
C
C  Get bras and kets <i,k| and |k,i>
          CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVLL,
     *               BRAKET(1,1,2),PK(0,1))
          CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVLL,
     *               BRAKET(1,2,2),PK(0,2))
C
C  Get bras and kets [pi-q-k,e]|q,i> etc.
          CALL KET2C(BRAKET(1,1,1),.FALSE.,PQ(0,1),SIGMA(1),K,JVLL,
     *               DEBK(1,1,1,2),DUMMY)
          CALL KET2R(BRAKET(1,1,2),.FALSE.,PK(0,1),SIGMA(1),Q,EPSG,
     *               DEBK(1,1,2,1),DUMMY)
          CALL BRA2C(BRAKET(1,2,1),.FALSE.,PQ(0,2),SIGMA(2),K,JVLL,
     *               DEBK(1,2,1,2),DUMMY)
          CALL BRA2R(BRAKET(1,2,2),.FALSE.,PK(0,2),SIGMA(2),Q,EPSG,
     *               DEBK(1,2,2,1),DUMMY)
C
          CALL AV4Q1G(1,2,3,4,A1234)
C
          DO 140 IS2 = -1,1,2
            DO 140 IS1 = -1,1,2
              HA1234(IS1,IS2,3,1) = A1234(IS1,IS2) * CORFAC(1,3,1)
140       CONTINUE
C
          DO 150 I = 1,3
            MSPP(I,1) = HA1234(1,1,3,1)
            MSPM(I,1) = HA1234(1,-1,3,1)
            MSMP(I,1) = HA1234(-1,1,3,1)
            MSMM(I,1) = HA1234(-1,-1,3,1)
150       CONTINUE
C
          IF ( SIGMA(3).EQ.-1 ) THEN
            CALL KET2C(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVLL,
     *                 BRAKET(1,3,2),PK(0,3))
            CALL KET2C(BRAKET(1,3,1),.FALSE.,PQ(0,3),SIGMA(3),K,JVLL,
     *                 DEBK(1,3,1,2),DUMMY)
            CALL KET2R(BRAKET(1,3,2),.FALSE.,PK(0,3),SIGMA(3),Q,EPSG,
     *                 DEBK(1,3,2,1),DUMMY)
            CALL AV4Q1G(3,2,1,4,A3214)
C
            MSPP(2,1) = MSPP(2,1) - A3214(1,1) * CORFAC(3,3,1)
            MSPM(2,1) = MSPM(2,1) - A3214(1,-1) * CORFAC(3,3,1)
            MSMP(2,1) = MSMP(2,1) + A3214(-1,1) * CORFAC(3,3,1)
            MSMM(2,1) = MSMM(2,1) + A3214(-1,-1) * CORFAC(3,3,1)
C
            CALL BRA2C(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVLL,
     *                 BRAKET(1,4,2),PK(0,4))
            CALL BRA2C(BRAKET(1,4,1),.FALSE.,PQ(0,4),SIGMA(4),K,JVLL,
     *                 DEBK(1,4,1,2),DUMMY)
            CALL BRA2R(BRAKET(1,4,2),.FALSE.,PK(0,4),SIGMA(4),Q,EPSG,
     *                 DEBK(1,4,2,1),DUMMY)
            CALL AV4Q1G(1,4,3,2,A1432)
            MSPP(3,1) = MSPP(3,1) - A1432(1,1) * CORFAC(1,3,1)
            MSPM(3,1) = MSPM(3,1) + A1432(1,-1) * CORFAC(1,3,1)
            MSMP(3,1) = MSMP(3,1) - A1432(-1,1) * CORFAC(1,3,1)
            MSMM(3,1) = MSMM(3,1) + A1432(-1,-1) * CORFAC(1,3,1)
C
          ENDIF
C
        ELSEIF ( SIGMA(1).EQ.-1 .AND. SIGMA(4).EQ.-1 .AND.
     1           SIGMA(2).EQ.SIGMA(3) ) THEN
C
C  Get bras and kets <i,k| and |k,i>
          CALL KET2C(CHIF(1,1),.TRUE.,P(0,1),SIGMA(1),K,JVLL,
     *               BRAKET(1,1,2),PK(0,1))
          CALL BRA2C(CHIF(1,4),.TRUE.,P(0,4),SIGMA(4),K,JVLL,
     *               BRAKET(1,4,2),PK(0,4))
C
C  Get bras and kets [pi-q-k,e]|q,i> etc.
          CALL KET2C(BRAKET(1,1,1),.FALSE.,PQ(0,1),SIGMA(1),K,JVLL,
     *               DEBK(1,1,1,2),DUMMY)
          CALL KET2R(BRAKET(1,1,2),.FALSE.,PK(0,1),SIGMA(1),Q,EPSG,
     *               DEBK(1,1,2,1),DUMMY)
          CALL BRA2C(BRAKET(1,4,1),.FALSE.,PQ(0,4),SIGMA(4),K,JVLL,
     *               DEBK(1,4,1,2),DUMMY)
          CALL BRA2R(BRAKET(1,4,2),.FALSE.,PK(0,4),SIGMA(4),Q,EPSG,
     *               DEBK(1,4,2,1),DUMMY)
C
          CALL AV4Q1G(1,4,3,2,A1432)
C
          MSPP(3,1) = A1432(1,1) * CORFAC(1,3,1)
          MSPM(3,1) = A1432(1,-1) * CORFAC(1,3,1)
          MSMP(3,1) = A1432(-1,1) * CORFAC(1,3,1)
          MSMM(3,1) = A1432(-1,-1) * CORFAC(1,3,1)
C
        ELSEIF ( SIGMA(3).EQ.-1 .AND. SIGMA(2).EQ.-1 .AND.
     1           SIGMA(1).EQ.SIGMA(4) ) THEN
C
C  Get bras and kets <i,k| and |k,i>
          CALL KET2C(CHIF(1,3),.TRUE.,P(0,3),SIGMA(3),K,JVLL,
     *               BRAKET(1,3,2),PK(0,3))
          CALL BRA2C(CHIF(1,2),.TRUE.,P(0,2),SIGMA(2),K,JVLL,
     *               BRAKET(1,2,2),PK(0,2))
C
C  Get bras and kets [pi-q-k,e]|q,i> etc.
          CALL KET2C(BRAKET(1,3,1),.FALSE.,PQ(0,3),SIGMA(3),K,JVLL,
     *               DEBK(1,3,1,2),DUMMY)
          CALL KET2R(BRAKET(1,3,2),.FALSE.,PK(0,3),SIGMA(3),Q,EPSG,
     *               DEBK(1,3,2,1),DUMMY)
          CALL BRA2C(BRAKET(1,2,1),.FALSE.,PQ(0,2),SIGMA(2),K,JVLL,
     *               DEBK(1,2,1,2),DUMMY)
          CALL BRA2R(BRAKET(1,2,2),.FALSE.,PK(0,2),SIGMA(2),Q,EPSG,
     *               DEBK(1,2,2,1),DUMMY)
C
          CALL AV4Q1G(3,2,1,4,A3214)
          MSPP(2,1) = A3214(1,1) * CORFAC(3,3,1)
          MSPM(2,1) = A3214(1,-1) * CORFAC(3,3,1)
          MSMP(2,1) = A3214(-1,1) * CORFAC(3,3,1)
          MSMM(2,1) = A3214(-1,-1) * CORFAC(3,3,1)
C
        ELSE
          WRITE(*,*) ' Helicity nonconserving amplitude called',
     1               ' in V4Q1G (V = W+-):'
          WRITE(*,*) ' SIGMA  = ',SIGMA
          RETURN
        ENDIF
C
      ELSE
        WRITE(*,*) ' Unrecognized vector boson V in V4Q1G.'
        WRITE(*,*) ' BOS = ',BOS
        RETURN
      ENDIF
C
C  Get overall factor F5, forget about sign factor FSIGN(1)*FSIGN(2)*
C                                                  FSIGN(3)*FSIGN(4)
      F5HAT = CLR(3,5,1)**3 * OMFACF
C
C  ... and now the final answer. The color factors are 2**2=4 times
C  larger than the ones for N = 3 in table 3 of HZ. This factor 4
C  is absorbed in the output of RV4Q1G.
C
c
      FINFAC = F5HAT**2 / 9.0d0
      do I = 1,2
         RVUUUUO(I) = 0.0d0
         RVUUCCO(I) = 0.0d0
         RVUUDDO(I) = 0.0d0
         RVDDUUO(I) = 0.0d0
         RVDDSSO(I) = 0.0d0
         RVDDDDO(I) = 0.0d0
      enddo
      RW00O = 0.0d0
      RW13O = 0.0d0
      RW24O = 0.0d0

      IF ( CHANEL.EQ.1 .OR. CHANEL.EQ.4 ) THEN
          
      DO 170 I = 1,2
         RVUUUUO(I) = 5.0d0 * ( dble(MSPP(1,I))**2+dimag(MSPP(1,I))**2 )
     1        + 3.0d0 * ( dble(MSPM(1,I))**2+dimag(MSPM(1,I))**2 )
     2        + 3.0d0 * ( dble(MSMP(1,I))**2+dimag(MSMP(1,I))**2 )
     3        +           dble(MSMM(1,I))**2+dimag(MSMM(1,I))**2
         RVUUUUO(I) = RVUUUUO(I) * FINFAC
         RVUUCCO(I) = 5.0d0 * ( dble(MSPP(2,I))**2+dimag(MSPP(2,I))**2 )
     1        + 3.0d0 * ( dble(MSPM(2,I))**2+dimag(MSPM(2,I))**2 )
     2        + 3.0d0 * ( dble(MSMP(2,I))**2+dimag(MSMP(2,I))**2 )
     3        +           dble(MSMM(2,I))**2+dimag(MSMM(2,I))**2
         RVUUCCO(I) = RVUUCCO(I) * FINFAC
         RVUUDDO(I) = 5.0d0 * ( dble(MSPP(3,I))**2+dimag(MSPP(3,I))**2 )
     1        + 3.0d0 * ( dble(MSPM(3,I))**2+dimag(MSPM(3,I))**2 )
     2        + 3.0d0 * ( dble(MSMP(3,I))**2+dimag(MSMP(3,I))**2 )
     3        +           dble(MSMM(3,I))**2+dimag(MSMM(3,I))**2
         RVUUDDO(I) = RVUUDDO(I) * FINFAC
         RVDDUUO(I) = 5.0d0 * ( dble(MSPP(4,I))**2+dimag(MSPP(4,I))**2 )
     1        + 3.0d0 * ( dble(MSPM(4,I))**2+dimag(MSPM(4,I))**2 )
     2        + 3.0d0 * ( dble(MSMP(4,I))**2+dimag(MSMP(4,I))**2 )
     3        +           dble(MSMM(4,I))**2+dimag(MSMM(4,I))**2
         RVDDUUO(I) = RVDDUUO(I) * FINFAC
         RVDDSSO(I) = 5.0d0 * ( dble(MSPP(5,I))**2+dimag(MSPP(5,I))**2 )
     1        + 3.0d0 * ( dble(MSPM(5,I))**2+dimag(MSPM(5,I))**2 )
     2        + 3.0d0 * ( dble(MSMP(5,I))**2+dimag(MSMP(5,I))**2 )
     3        +           dble(MSMM(5,I))**2+dimag(MSMM(5,I))**2
         RVDDSSO(I) = RVDDSSO(I) * FINFAC
         RVDDDDO(I) = 5.0d0 * ( dble(MSPP(6,I))**2+dimag(MSPP(6,I))**2 )
     1        + 3.0d0 * ( dble(MSPM(6,I))**2+dimag(MSPM(6,I))**2 )
     2        + 3.0d0 * ( dble(MSMP(6,I))**2+dimag(MSMP(6,I))**2 )
     3        +           dble(MSMM(6,I))**2+dimag(MSMM(6,I))**2
         RVDDDDO(I) = RVDDDDO(I) * FINFAC
 170  CONTINUE
      if(lcol) then
         do I =1,2
            RVUUUU(I) = RVUUUUO(I)
            RVUUCC(I) = RVUUCCO(I)
            RVUUDD(I) = RVUUDDO(I)
            RVDDUU(I) = RVDDUUO(I)
            RVDDSS(I) = RVDDSSO(I) 
            RVDDDD(I) = RVDDDDO(I)
         enddo
      else
         do I = 1,2
            do J = 1,6  ! the third index refers to the colstr
               MSC(J,I,1) = MSPP(J,I) + MSPM(J,I)
     1                    + MSMP(J,I) + MSMM(J,I)
c          The color flow connects q3 and q4  
               MSC(J,I,2) = MSPP(J,I) + MSPM(J,I)
     1                    - MSMM(J,I) - MSMP(J,I)
c          the color flow connects q2 and q4
               MSC(J,I,3) = MSPP(J,I) + MSMP(J,I)
     1                    - MSMM(J,I) - MSPM(J,I)
c          the color flow connects q2 and q3
               MSC(J,I,4) = MSPP(J,I) + MSMM(J,I)
     1                    - MSMP(J,I) - MSPM(J,I)
c          the color flow connects q2 and q1
            enddo
         enddo
c     See HZ (3.11b)
c     i3 = 1 -> gamma_i2i1 delta_i4i3
c        = 2 -> gamma_i2i3 delta_i4i1
c        = 3 -> gamma_i4i1 delta_i2i3         
c        = 4 -> gamma_i4i3 delta_i2i1
c
         do I = 1,2
            do J = 1,6
               COSUM(J,I) = 0.0d0
               do i3 = 1,4
                  CO(J,I,i3) = dble(MSC(J,I,i3))**2 + 
     1                        dimag(MSC(J,I,i3))**2
                  COSUM(J,I) = COSUM(J,I) + CO(J,I,i3) ! find sum
               enddo
               do i3 = 1,4
                  if(COSUM(J,I).gt. 0d0) then
                     CO(J,I,i3) = CO(J,I,i3)/COSUM(J,I) ! normalize
                  else
                     CO(J,I,i3) = 1.0d0/4.0d0
                  endif
               enddo
            enddo
         enddo
c
c
c
c     Change into new format:
c     COI(fl1,i3) for identical particles
c     CONI(fl1,fl3,i3) for non-identical particles
c
c  The result in the new format:
         do i = 1,2
            sign = -2*i +3
            u = 2*sign
            c = u
            d = sign
            s = d
            RVcNI(u,c,0) = RVUUCCO(i)
            RVcNI(u,d,0) = RVUUDDO(i)
            RVcNI(d,u,0) = RVDDUUO(i)
            RVcNI(d,s,0) = RVDDSSO(i)
            RVcI(u,0) =    RVUUUUO(i)
            RVcI(d,0) =    RVDDDDO(i)
c
            do i3 = 1,4
               COI(u,i3)      = CO(1,i,i3) !vuuuu
               CONI(u,c,i3)   = CO(2,i,i3) !vuucc         
               CONI(u,d,i3)   = CO(3,i,i3) !vuudd           
               CONI(d,u,i3)   = CO(4,i,i3) !vdduu          
               CONI(d,s,i3)   = CO(5,i,i3) !vddss           
               COI(d,i3)      = CO(6,i,i3) !vdddd
            enddo
         enddo
c
         do ifl1 = -2,2
            do i3 = 1,4
               RVcI(ifl1,i3) = COI(ifl1,i3)*RVcI(ifl1,0)
               do ifl3 = -2,2
                  RVcNI(ifl1,ifl3,i3) = CONI(ifl1,ifl3,i3)*
     1                                  RVcNI(ifl1,ifl3,0)
               enddo
            enddo
         enddo
c        
      endif

      ELSEIF ( CHANEL.EQ.3 ) THEN 
       
         DO 171 I = 1,2
          DO 172 IL = -1,1,2
            RVUUUUO(I) = RVUUUUO(I) +
     1      5.0d0 * ( dble(MSPP10(1,I,IL))**2+dimag(MSPP10(1,I,IL))**2 )
     2    + 3.0d0 * ( dble(MSPM10(1,I,IL))**2+dimag(MSPM10(1,I,IL))**2 )
     3    + 3.0d0 * ( dble(MSMP10(1,I,IL))**2+dimag(MSMP10(1,I,IL))**2 )
     4    +           dble(MSMM10(1,I,IL))**2+dimag(MSMM10(1,I,IL))**2
            RVUUCCO(I) = RVUUCCO(I) +
     1      5.0d0 * ( dble(MSPP10(2,I,IL))**2+dimag(MSPP10(2,I,IL))**2 )
     2    + 3.0d0 * ( dble(MSPM10(2,I,IL))**2+dimag(MSPM10(2,I,IL))**2 )
     3    + 3.0d0 * ( dble(MSMP10(2,I,IL))**2+dimag(MSMP10(2,I,IL))**2 )
     4    +           dble(MSMM10(2,I,IL))**2+dimag(MSMM10(2,I,IL))**2
            RVUUDDO(I) = RVUUDDO(I) +
     1      5.0d0 * ( dble(MSPP10(3,I,IL))**2+dimag(MSPP10(3,I,IL))**2 )
     2    + 3.0d0 * ( dble(MSPM10(3,I,IL))**2+dimag(MSPM10(3,I,IL))**2 )
     3    + 3.0d0 * ( dble(MSMP10(3,I,IL))**2+dimag(MSMP10(3,I,IL))**2 )
     4    +           dble(MSMM10(3,I,IL))**2+dimag(MSMM10(3,I,IL))**2
            RVDDUUO(I) = RVDDUUO(I) + 
     1      5.0d0 * ( dble(MSPP10(4,I,IL))**2+dimag(MSPP10(4,I,IL))**2 )
     2    + 3.0d0 * ( dble(MSPM10(4,I,IL))**2+dimag(MSPM10(4,I,IL))**2 )
     3    + 3.0d0 * ( dble(MSMP10(4,I,IL))**2+dimag(MSMP10(4,I,IL))**2 )
     4    +           dble(MSMM10(4,I,IL))**2+dimag(MSMM10(4,I,IL))**2
            RVDDSSO(I) = RVDDSSO(I) + 
     1      5.0d0 * ( dble(MSPP10(5,I,IL))**2+dimag(MSPP10(5,I,IL))**2 )
     2    + 3.0d0 * ( dble(MSPM10(5,I,IL))**2+dimag(MSPM10(5,I,IL))**2 )
     3    + 3.0d0 * ( dble(MSMP10(5,I,IL))**2+dimag(MSMP10(5,I,IL))**2 )
     4    +           dble(MSMM10(5,I,IL))**2+dimag(MSMM10(5,I,IL))**2
            RVDDDDO(I) = RVDDDDO(I) + 
     1      5.0d0 * ( dble(MSPP10(6,I,IL))**2+dimag(MSPP10(6,I,IL))**2 )
     2    + 3.0d0 * ( dble(MSPM10(6,I,IL))**2+dimag(MSPM10(6,I,IL))**2 )
     3    + 3.0d0 * ( dble(MSMP10(6,I,IL))**2+dimag(MSMP10(6,I,IL))**2 )
     4    +           dble(MSMM10(6,I,IL))**2+dimag(MSMM10(6,I,IL))**2
172       CONTINUE
          RVUUUUO(I) = RVUUUUO(I) * FINFAC
          RVUUCCO(I) = RVUUCCO(I) * FINFAC
          RVUUDDO(I) = RVUUDDO(I) * FINFAC
          RVDDUUO(I) = RVDDUUO(I) * FINFAC
          RVDDSSO(I) = RVDDSSO(I) * FINFAC
          RVDDDDO(I) = RVDDDDO(I) * FINFAC
171     CONTINUE
        if(lcol) then
           do I =1,2
              RVUUUU(I) = RVUUUUO(I)
              RVUUCC(I) = RVUUCCO(I)
              RVUUDD(I) = RVUUDDO(I)
              RVDDUU(I) = RVDDUUO(I)
              RVDDSS(I) = RVDDSSO(I) 
              RVDDDD(I) = RVDDDDO(I)
           enddo
        else
c     Not applicable:
           write(6,*)'FATAL CRASH in RV4Q1G'
           write(6,*)'color distinction not implemented yet'
           stop
        endif

      ELSEIF (CHANEL.EQ.2 .OR.CHANEL.EQ.5 ) THEN
        
         RW00O = 5.0d0 * ( dble(MSPP(1,1))**2+dimag(MSPP(1,1))**2 )
     1         + 3.0d0 * ( dble(MSPM(1,1))**2+dimag(MSPM(1,1))**2 )
     2         + 3.0d0 * ( dble(MSMP(1,1))**2+dimag(MSMP(1,1))**2 )
     3         +           dble(MSMM(1,1))**2+dimag(MSMM(1,1))**2
            RW00O = RW00O * FINFAC
         RW13O = 5.0d0 * ( dble(MSPP(2,1))**2+dimag(MSPP(2,1))**2 )
     1         + 3.0d0 * ( dble(MSPM(2,1))**2+dimag(MSPM(2,1))**2 )
     2         + 3.0d0 * ( dble(MSMP(2,1))**2+dimag(MSMP(2,1))**2 )
     3         +           dble(MSMM(2,1))**2+dimag(MSMM(2,1))**2
         RW13O = RW13O * FINFAC
         RW24O = 5.0d0 * ( dble(MSPP(3,1))**2+dimag(MSPP(3,1))**2 )
     1         + 3.0d0 * ( dble(MSPM(3,1))**2+dimag(MSPM(3,1))**2 )
     2         + 3.0d0 * ( dble(MSMP(3,1))**2+dimag(MSMP(3,1))**2 )
     3         +           dble(MSMM(3,1))**2+dimag(MSMM(3,1))**2
         RW24O = RW24O * FINFAC
         if(lcol) then
            RW00 = RW00O
            RW13 = RW13O
            RW24 = RW24O
         else
c     New format:
            J = 1               ! no charge conj for W's
            do I = 1,3
               MSC(I,J,1) = MSPP(I,J) + MSPM(I,J)
     1                    + MSMP(I,J) + MSMM(I,J)  
               MSC(I,J,2) = MSPP(I,J) + MSPM(I,J)
     1                    - MSMM(I,J) - MSMP(I,J)
               MSC(I,J,3) = MSPP(I,J) + MSMP(I,J)
     1                    - MSMM(I,J) - MSPM(I,J)
               MSC(I,J,4) = MSPP(I,J) + MSMM(I,J)
     1                    - MSMP(I,J) - MSPM(I,J)
            enddo
            
c
            do I = 1,3
               CWSUM(I) = 0.0d0
               do i3 = 1,4
                  CW(I,i3) = dble(MSC(I,J,i3))**2 +
     1                      dimag(MSC(I,J,i3))**2
                  CWSUM(I) = CWSUM(I) + CW(I,i3)
 
               enddo
               do i3 = 1,4
                  if(CWSUM(I).gt.0d0) then
                     CW(I,i3) = CW(I,i3)/CWSUM(I)
                  else
                     CW(I,i3) = 0.25d0  
                  endif
               enddo
            enddo
c
c
            RWc00(0) = RW00O
            RWc13(0) = RW13O
            RWc24(0) = RW24O
            do i3 = 1,4
               RWc00(i3) = CW(1,i3) * RWc00(0)
               RWc13(i3) = CW(2,i3) * RWc13(0)
               RWc24(i3) = CW(3,i3) * RWc24(0)
            enddo
c        
         endif
      ELSE
        WRITE (*,*) ' Illegal channel assignment in V4Q1G: CHANEL = ',
     1                CHANEL
      ENDIF
C
      RETURN
      END
C
C************ AV4Q1G ***************************************************
C
      SUBROUTINE AV4Q1G(N1,N2,N3,N4,A)
      implicit none
      INTEGER NF, NG, NF4, NGPL1
      PARAMETER (NF=4, NG=1, NF4=NF*4,NGPL1=NG+1)
C
C  AV4Q1G calculates the basic Wqqqqg amplitude
C  1/2 * A^(+-,+-)(1,2,3,4)/F_5 of eq.4.2 in HZ.
C
C  INPUT : Indices N1,N2,N3,N4 identifying the four quarks
C  ------
C
C  OUTPUT:
C  -------
C  A(-1:1,-1:1)    The complex values of the amplitudes A(+,+), A(+,-),
C                  A(-,+), and A(-,-) of eqn. 5.7 in HZ.
C
C
C             The full information on the quarks and the gluon is
C             transferred via common block / BV4Q1G / from
C             subroutine V4Q1G
C
      INTEGER N1, N2, N3, N4, SIGMA(NF)
      real*8 P(0:3,NF), Q(0:4), EPSG(0:3), PK(0:4,NF)
      complex*16  CHIF(2,NF), BRAKET(2,NF,NGPL1), DEBK(2,NF,NGPL1,NGPL1),
     1          A(-1:1,-1:1)
C
C  Information passed to AV4Q1G by common block:
C      COMMON /BV4Q1G/ CHIF, BRAKET, DEBK, P, SIGMA, Q, EPSG, PK
      COMMON /BV4Q1G/ CHIF, BRAKET, DEBK, P, Q, EPSG, PK, SIGMA
C
C  Further local variables
      real*8  PROP, EPS4M3
      complex*16  Z1,Z2,Z3,Z4,Z3M4, ZJ0,ZJ1,ZJ2,ZJ3,
     1          J43S0,J43S1,J43S2,J43S3,
     2          EPSJ43,QJ43,E21J43, E21S0,E21S1,E21S2,E21S3,
     3          S1C, CHIA(2), J43(0:3), E21(0:3), MS1,MS2,MS3,MS4
      EXTERNAL S1C
C
C  Check helicity conservation
      IF ( SIGMA(N1).NE.SIGMA(N2).OR.SIGMA(N3).NE.SIGMA(N4) ) THEN
        WRITE(*,*) ' Wrong helicity assignment in AV4Q1G: '
        WRITE(*,*) ' SIGMA = ',SIGMA(N1),SIGMA(N2),' ; ',
     1                         SIGMA(N3),SIGMA(N4)
        A(1,1)   = 0.0d0
        A(1,-1)  = 0.0d0
        A(-1,1)  = 0.0d0
        A(-1,-1) = 0.0d0
        RETURN
      ENDIF
C
C  Form the current E_21^mu = <2,k|mu|1>+<2|mu|k,1> ).......
      Z1 = BRAKET(1,N2,2)*CHIF(1,N1) + CHIF(1,N2)*BRAKET(1,N1,2)
      Z2 = BRAKET(2,N2,2)*CHIF(2,N1) + CHIF(2,N2)*BRAKET(2,N1,2)
      Z3 = BRAKET(1,N2,2)*CHIF(2,N1) + CHIF(1,N2)*BRAKET(2,N1,2)
      Z4 = BRAKET(2,N2,2)*CHIF(1,N1) + CHIF(2,N2)*BRAKET(1,N1,2)
C
      PROP = ( PK(0,N1)-P(0,N2) )**2 - ( PK(1,N1)-P(1,N2) )**2 -
     1       ( PK(2,N1)-P(2,N2) )**2 - ( PK(3,N1)-P(3,N2) )**2
      PROP = 1.0d0 / PROP
C
      E21S0 = (Z1+Z2)*PROP
      IF (SIGMA(N1).EQ.-1) THEN
        E21S1 = -(Z3+Z4)*PROP
        Z3M4 = Z3-Z4
        E21S2 = dcmplx(-dimag(Z3M4),dble(Z3M4))*PROP
        E21S3 = -(Z1-Z2)*PROP
      ELSE
        E21S1 = (Z3+Z4)*PROP
        Z3M4 = Z3-Z4
        E21S2 = dcmplx(dimag(Z3M4),-dble(Z3M4))*PROP
        E21S3 = (Z1-Z2)*PROP
      ENDIF
      E21(0) = E21S0
      E21(1) = E21S1
      E21(2) = E21S2
      E21(3) = E21S3
C
C  Form the current J43:
      Z1 = CHIF(1,N4)*CHIF(1,N3)
      Z2 = CHIF(2,N4)*CHIF(2,N3)
      Z3 = CHIF(1,N4)*CHIF(2,N3)
      Z4 = CHIF(2,N4)*CHIF(1,N3)
C
      PROP = -P(0,N4)*P(0,N3)+P(1,N4)*P(1,N3)+P(2,N4)*P(2,N3)+
     *        P(3,N4)*P(3,N3)
      PROP = 0.5d0 / PROP
C
      J43S0 = (Z1+Z2)*PROP
      IF (SIGMA(N3).EQ.-1) THEN
        J43S1 = -(Z3+Z4)*PROP
        Z3M4 = Z3-Z4
        J43S2 = dcmplx(-dimag(Z3M4),dble(Z3M4))*PROP
        J43S3 = -(Z1-Z2)*PROP
      ELSE
        J43S1 = (Z3+Z4)*PROP
        Z3M4 = Z3-Z4
        J43S2 = dcmplx(dimag(Z3M4),-dble(Z3M4))*PROP
        J43S3 = (Z1-Z2)*PROP
      ENDIF
      J43(0) = J43S0
      J43(1) = J43S1
      J43(2) = J43S2
      J43(3) = J43S3
C
C  Get the 3-gluon vertex contribution....
      EPSJ43 = -EPSG(1)*J43S1-EPSG(2)*J43S2-EPSG(3)*J43S3
      EPS4M3 = -EPSG(1)*(P(1,N4)-P(1,N3))-
     1          EPSG(2)*(P(2,N4)-P(2,N3))-EPSG(3)*(P(3,N4)-P(3,N3))
      QJ43 = Q(0)*J43S0 -Q(1)*J43S1-Q(2)*J43S2-Q(3)*J43S3
C
      ZJ0 = Q(0)*EPSJ43 + J43S0*EPS4M3
      ZJ1 = Q(1)*EPSJ43 + J43S1*EPS4M3 - EPSG(1)*QJ43
      ZJ2 = Q(2)*EPSJ43 + J43S2*EPS4M3 - EPSG(2)*QJ43
      ZJ3 = Q(3)*EPSJ43 + J43S3*EPS4M3 - EPSG(3)*QJ43
C
      E21J43 = 2D0 *(E21S0*ZJ0 -E21S1*ZJ1-E21S2*ZJ2-E21S3*ZJ3)
C
C  ......and the rest...
      CHIA(1) = DEBK(1,N2,1,2) + DEBK(1,N2,2,1)
      CHIA(2) = DEBK(2,N2,1,2) + DEBK(2,N2,2,1)
      MS1 = S1C(BRAKET(1,N2,1),J43,.TRUE.,SIGMA(N1),BRAKET(1,N1,2)) +
     1      S1C(CHIA,J43,.TRUE.,SIGMA(N1),CHIF(1,N1)) -
     2      E21J43
C
      CHIA(1) = DEBK(1,N1,1,2) + DEBK(1,N1,2,1)
      CHIA(2) = DEBK(2,N1,1,2) + DEBK(2,N1,2,1)
      MS2 = S1C(BRAKET(1,N2,2),J43,.TRUE.,SIGMA(N1),BRAKET(1,N1,1))
     1    + S1C(CHIF(1,N2),J43,.TRUE.,SIGMA(N1),CHIA)
     2    + E21J43
C
      MS3 = S1C(BRAKET(1,N4,1),E21,.TRUE.,SIGMA(N3),CHIF(1,N3))
C
      MS4 = S1C(CHIF(1,N4),E21,.TRUE.,SIGMA(N3),BRAKET(1,N3,1))
C
C  ... and map it onto A(+-,+-):
      A(1,1)   = MS1 + MS2 + MS3 + MS4
      A(-1,1)  = -2D0 *MS1 + MS2 + 2D0 *MS3 - MS4
      A(1,-1)  = MS1 - 2D0 *MS2 - MS3 + 2D0 *MS4
      A(-1,-1) = -2D0 *A(1,1)
C
      RETURN
      END
C
C****************  HSRW2Q  *********************************************
C
      SUBROUTINE HSRW2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1                  BOS,KBAR,KSIGN,RWUD)
      implicit none
C
C---------------------------------------------------------------------
C      Helicity Summed, Squared Matrix Element Evaluations
C       for 2 Quarks + Ng Gluons + W(--> l neu) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Random Helicity Selection;
C               'S' for Direct Sum over Helicities.
C        R    ---  Input Random Number to Specify the Helicities
C                (Relevant only when Mode = 'R')
C        NG   ---  Number of Gluons
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities of 2  Quarks
C        GPOL ---  Physical Helicities of NG Gluons
C        LPOL ---  Physical Helicities of 2 Leptons
C        VPOL ---  Physical Helicity of the  real W
C      OUTPUT:
C        RWUD ---  Spin & Color Summed, Squared Matrix Element
C
C  For more information see the general comments on the package
C
      CHARACTER*1 MODE
      INTEGER NG, FSIGN(2), GSIGN(NG), KSIGN, LSIGN(2), LFLAVR(2), BOS
      real*8  PBAR(0:3,2), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2),
     1      R, RWUD
C
      real*8  RN, HWUD, WGT
      INTEGER FPOL(2), GPOL(3), VPOL, LPOL(2), I,I1,I2,I3
C
C---------------------------------------------------------------------
C Entry for a Virtual W:
C
      ENTRY HSLW2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1               BOS,LBAR,LSIGN,LFLAVR,RWUD)
C
C---------------------------------------------------------------------
      RWUD = 0.0d0
C
C For a W Coupling, Fix the Fermion Helicities:
      DO 10 I = 1, 2
         IF (BOS .EQ. 30 .OR. BOS .EQ. 40) LPOL(I)  = -LSIGN(I)
         FPOL(I)  = -FSIGN(I)
10    CONTINUE
C
      IF (MODE .EQ. 'R') THEN
C Monte Carlo Summation over Polarizations:
        IF (R.LT. 0.0d0 .OR. R.GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in HSRW2Q: ', R
           RETURN
        ENDIF
        IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
           RN   = 3.0d0 * R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
        ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
           WGT  = 1.0d0
           RN   = R
        ELSE
           PRINT *, ' Wrong BOS assignment in HSRW2Q: ', BOS
           RETURN
        END IF
C
C  get random gluon polarizations
        DO 20 I = 1, NG
           RN  = 2D0 * RN
           IF (RN .GT. 1.0d0 ) THEN
              GPOL(I) = 1
              RN = RN - 1.0d0
           ELSE
              GPOL(I) = 2
           END IF
           WGT = 2D0 * WGT
 20      CONTINUE
C
         IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
           IF (NG .EQ. 1) THEN
              CALL RWQQG(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,KBAR,KSIGN,VPOL,HWUD)
           ELSE IF (NG .EQ. 2) THEN
              CALL RWQQGG(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,HWUD)
           ELSE IF (NG .EQ. 3) THEN
              CALL RW2Q3G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,HWUD)
           ELSE
              PRINT *, ' Wrong NG assignment in HSRW2Q: ', NG
              RETURN
           END IF
         ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
           IF (NG .EQ. 1) THEN
              CALL LWQQG(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,LBAR,LSIGN,LPOL,LFLAVR,HWUD)
           ELSE IF (NG .EQ. 2) THEN
              CALL LWQQGG(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,HWUD)
           ELSE IF (NG .EQ. 3) THEN
              CALL LW2Q3G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,HWUD)
           ELSE
              PRINT *, ' Wrong NG assignment in HSRW2Q: ', NG
           END IF
         END IF
         RWUD = WGT*HWUD
C
      ELSE IF (MODE .EQ. 'S') THEN
C Direct Summation for Helicities:
         IF (NG .EQ. 1) THEN
            DO 30 I1  = 1, 2
               GPOL(1) = I1
               IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
                  DO 31 VPOL = 1, 3
                     CALL RWQQG(PBAR,FSIGN,FPOL,
     1                          QBAR,GSIGN,GPOL,
     2                      BOS,KBAR,KSIGN,VPOL,HWUD)
                     RWUD = RWUD + HWUD
31                CONTINUE
               ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
                  CALL LWQQG(PBAR,FSIGN,FPOL,
     1                       QBAR,GSIGN,GPOL,
     2                   BOS,LBAR,LSIGN,LPOL,LFLAVR,HWUD)
                  RWUD = RWUD + HWUD
               END IF
30          CONTINUE
         ELSE IF (NG .EQ. 2) THEN
            DO 40 I1  = 1, 2
               GPOL(1) = I1
               DO 40 I2  = 1, 2
                  GPOL(2) = I2
                  IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
                     DO 41 VPOL = 1, 3
                        CALL RWQQGG(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,KBAR,KSIGN,VPOL,HWUD)
                        RWUD = RWUD + HWUD
41                   CONTINUE
                  ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
                     CALL LWQQGG(PBAR,FSIGN,FPOL,
     1                           QBAR,GSIGN,GPOL,
     2                       BOS,LBAR,LSIGN,LPOL,LFLAVR,HWUD)
                     RWUD = RWUD + HWUD
                  END IF
40          CONTINUE
         ELSE IF (NG .EQ. 3) THEN
            DO 50 I1  = 1, 2
               GPOL(1) = I1
               DO 50 I2  = 1, 2
                  GPOL(2) = I2
                  DO 50 I3  = 1, 2
                     GPOL(3) = I3
                     IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
                        DO 51 VPOL = 1, 3
                           CALL RW2Q3G(PBAR,FSIGN,FPOL,
     1                                 QBAR,GSIGN,GPOL,
     2                             BOS,KBAR,KSIGN,VPOL,HWUD)
                           RWUD = RWUD + HWUD
51                      CONTINUE
                     ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
                        CALL LW2Q3G(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,LBAR,LSIGN,LPOL,LFLAVR,HWUD)
                        RWUD = RWUD + HWUD
                     END IF
50          CONTINUE
         ELSE
              PRINT *, ' Wrong NG assignment in HSRW2Q: ', NG
         END IF
C
      ELSE
         PRINT *, ' Wrong MODE assignment in HSRW2Q: ', MODE
      END IF
C
      RETURN
      END

C
C***************  HSRV2Q *********************************************
C
      SUBROUTINE HSRV2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1                    BOS,KBAR,KSIGN,RVUU,RVDD)
      implicit none
C
C---------------------------------------------------------------------
C      Helicity Summed, Squared Matrix Element Evaluations
C       for 2 Quarks + Ng Gluons + Z,gamma(--> ll) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Randam Helicity Selection;
C               'S' for Direct Sum over Helicities.
C        R    ---  Input Random Number to Specify the Helicities
C                (Relevant only when Mode = 'R')
C        NG   ---  Number of Gluons
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities of 2  Quarks
C        GPOL ---  Physical Helicities of NG Gluons
C        LPOL ---  Physical Helicities of 2 Leptons
C        VPOL ---  Physical Helicity of a real Z (gamma)
C      OUTPUT:
C        RVUU(2) ---  Spin & Color Summed, Squared Matrix Element;
C        RVDD(2) ---  Letters 3 and 4 give the flavors of the
C                      quarks assumed for a particular matrixelement**2.
C                      2 values are given for each flavor combination:
C                        1: the basic amplitude**2 as given in HZ
C                        2: the amplitude**2 of the corresponding
C                        charge conjugation transformed reaction, i.e.
C                        for quarks replaced by antiquarks of the SAME
C                        momentum.
C
C                        For more information see the individual
C                        subroutines RVQQG, RVQQGG and RV2Q3G and
C                        the general information on the package.
C
      CHARACTER*1 MODE
      INTEGER NG, FSIGN(2), GSIGN(NG), KSIGN, LSIGN(2), LFLAVR(2), BOS
      real*8  PBAR(0:3,2), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2),
     1      R, RVUU(2), RVDD(2)
C
      real*8  RN, HVUU(2), HVDD(2), WGT
      INTEGER FPOL(2), GPOL(3), VPOL, LPOL(2), I,I1,I2,I3,IQ,IL
C
C---------------------------------------------------------------------
C Entry for a Virtual Z,Gamma:
C
      ENTRY HSLV2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1             BOS,LBAR,LSIGN,LFLAVR,RVUU,RVDD)
C
C---------------------------------------------------------------------
C
         DO 10 I = 1,2
           RVUU(I) = 0.0d0
10         RVDD(I) = 0.0d0
      IF (MODE .EQ. 'R') THEN
C Monte Carlo Summation for Polarizations:
        IF (R.LT. 0.0d0 .OR. R .GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in HSRW2Q: ', R
           RETURN
        ENDIF
C Real Photon:
         IF (BOS .EQ. 1) THEN
            RN   = 2D0 * R
            VPOL = MIN0((1 + INT(RN)), 2)
            WGT  = 2D0
            RN   = RN + 1 - VPOL
C Real Z:
         ELSE IF (BOS .EQ. 2) THEN
            RN   = 3.0d0 * R
            VPOL = MIN0((1 + INT(RN)), 3)
            WGT  = 3.0d0
            RN   = RN + 1 - VPOL
C Virtual gamma & Z :
         ELSE IF (BOS .EQ. 10) THEN
            IF (LFLAVR(1)/10 .EQ. 1) THEN
              RN = R
              WGT = 1.0d0
              LPOL(1) = -LSIGN(1)
              LPOL(2) = -LSIGN(2)
            ELSE
              RN  = 2D0 * R
              IF (RN .GT. 1.0d0 ) THEN
                 LPOL(1) =  LSIGN(1)
                 LPOL(2) =  LSIGN(2)
                 RN = RN -  1.0d0
              ELSE IF(RN .LE. 1.0d0 ) THEN
                 LPOL(1) = -LSIGN(1)
                 LPOL(2) = -LSIGN(2)
              END IF
              WGT = 2D0
           ENDIF
         ELSE
            PRINT *, ' Wrong BOS Assignment in HSRV2Q:', BOS
            RETURN
         END IF
C
C   For fermions:
C
         RN  = 2D0 * RN
         IF (RN .GT. 1.0d0 ) THEN
            FPOL(1) =  FSIGN(1)
            FPOL(2) =  FSIGN(2)
            RN = RN -  1.0d0
         ELSE IF(RN .LE. 1.0d0 ) THEN
            FPOL(1) = -FSIGN(1)
            FPOL(2) = -FSIGN(2)
         END IF
         WGT = 2D0 * WGT
C   For gluons:
         DO 20 I = 1, NG
           RN  = 2D0 * RN
           IF (RN .GT. 1.0d0 ) THEN
              GPOL(I) = 1
              RN = RN - 1.0d0
           ELSE
              GPOL(I) = 2
           END IF
           WGT = 2D0 * WGT
 20      CONTINUE
C
C now call the subroutines for matrix elemenent squared at
C fixed polarizations
C
         IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
           IF (NG .EQ. 1) THEN
              CALL RVQQG(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,KBAR,KSIGN,VPOL,HVUU,HVDD)
           ELSE IF (NG .EQ. 2) THEN
              CALL RVQQGG(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,HVUU,HVDD)
           ELSE IF (NG .EQ. 3) THEN
              CALL RV2Q3G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,HVUU,HVDD)
           ELSE
              PRINT *, ' Wrong NG assignment in HSRV2Q:', NG
              RETURN
           END IF
         ELSE IF (BOS .EQ. 10) THEN
           IF (NG .EQ. 1) THEN
              CALL LVQQG(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,LBAR,LSIGN,LPOL,LFLAVR,HVUU,HVDD)
           ELSE IF (NG .EQ. 2) THEN
              CALL LVQQGG(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,HVUU,HVDD)
           ELSE IF (NG .EQ. 3) THEN
              CALL LV2Q3G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,HVUU,HVDD)
           ELSE
              PRINT *, ' Wrong NG assignment in HSRV2Q:', NG
              RETURN
           END IF
         END IF
C
         DO 21 I = 1,2
           RVUU(I) = WGT*HVUU(I)
21         RVDD(I) = WGT*HVDD(I)
         RETURN
C
C  now comes the direct summation over polarizations
      ELSE IF (MODE .EQ. 'S') THEN
         IF (NG .EQ. 1) THEN
            DO 30 I1  = 1, 2
               GPOL(1) = I1
               DO 30 IQ = -1, 1, 2
                  FPOL(1) =  IQ*FSIGN(1)
                  FPOL(2) =  IQ*FSIGN(2)
                  IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
                     DO 31 VPOL = 1, BOS+1
                        CALL RVQQG(PBAR,FSIGN,FPOL,
     1                             QBAR,GSIGN,GPOL,
     2                         BOS,KBAR,KSIGN,VPOL,HVUU,HVDD)
                        DO 32 I = 1,2
                           RVUU(I) = RVUU(I) + HVUU(I)
32                         RVDD(I) = RVDD(I) + HVDD(I)
31                   CONTINUE
                  ELSE IF (BOS .EQ. 10) THEN
                     DO 33 IL = -1, 1, 2
                        IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 33
                        LPOL(1) =  IL*LSIGN(1)
                        LPOL(2) =  IL*LSIGN(2)
                        CALL LVQQG(PBAR,FSIGN,FPOL,
     1                             QBAR,GSIGN,GPOL,
     2                  BOS,LBAR,LSIGN,LPOL,LFLAVR,HVUU,HVDD)
                        DO 34 I = 1,2
                           RVUU(I) = RVUU(I) + HVUU(I)
34                         RVDD(I) = RVDD(I) + HVDD(I)
33                CONTINUE
               END IF
30          CONTINUE
         ELSE IF (NG .EQ. 2) THEN
            DO 40 I1  = 1, 2
              GPOL(1) = I1
              DO 40 I2  = 1, 2
                GPOL(2) = I2
                DO 40 IQ = -1, 1, 2
                  FPOL(1) =  IQ*FSIGN(1)
                  FPOL(2) =  IQ*FSIGN(2)
                  IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
                     DO 41 VPOL = 1, BOS+1
                        CALL RVQQGG(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,KBAR,KSIGN,VPOL,HVUU,HVDD)
                        DO 42 I = 1,2
                           RVUU(I) = RVUU(I) + HVUU(I)
42                         RVDD(I) = RVDD(I) + HVDD(I)
41                   CONTINUE
                  ELSE IF (BOS .EQ. 10) THEN
                     DO 43 IL = -1, 1, 2
                        IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 43
                        LPOL(1) =  IL*LSIGN(1)
                        LPOL(2) =  IL*LSIGN(2)
                        CALL LVQQGG(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                  BOS,LBAR,LSIGN,LPOL,LFLAVR,HVUU,HVDD)
                        DO 44 I = 1,2
                           RVUU(I) = RVUU(I) + HVUU(I)
44                         RVDD(I) = RVDD(I) + HVDD(I)
43                   CONTINUE
                  END IF
40          CONTINUE
         ELSE IF (NG .EQ. 3) THEN
            DO 50 I1  = 1, 2
              GPOL(1) = I1
              DO 50 I2  = 1, 2
                GPOL(2) = I2
                DO 50 I3  = 1, 2
                  GPOL(3) = I3
                  DO 50 IQ = -1, 1, 2
                    FPOL(1) =  IQ*FSIGN(1)
                    FPOL(2) =  IQ*FSIGN(2)
                    IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
                      DO 51 VPOL = 1, BOS+1
                        CALL RV2Q3G(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,KBAR,KSIGN,VPOL,HVUU,HVDD)
                        DO 52 I = 1,2
                           RVUU(I) = RVUU(I) + HVUU(I)
52                         RVDD(I) = RVDD(I) + HVDD(I)
51                    CONTINUE
                    ELSE IF (BOS .EQ. 10) THEN
                      DO 53 IL = -1, 1, 2
                        IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 53
                        LPOL(1) =  IL*LSIGN(1)
                        LPOL(2) =  IL*LSIGN(2)
                        CALL LV2Q3G(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                  BOS,LBAR,LSIGN,LPOL,LFLAVR,HVUU,HVDD)
                        DO 54 I = 1,2
                           RVUU(I) = RVUU(I) + HVUU(I)
54                         RVDD(I) = RVDD(I) + HVDD(I)
53                   CONTINUE
                   END IF
50          CONTINUE
C
         ELSE
              PRINT *, ' Wrong NG assignment in HSRV2Q:', NG
         END IF
C
      ELSE
         PRINT *, ' Wrong MODE assignment in HSRV2Q:', MODE
      END IF
C
      RETURN
      END
C
C********************  SRW4Q,SLW4Q, SRW4QG,SLW4QG **********************
C
      SUBROUTINE SRW4Q(MODE,R,PBAR,FSIGN,BOS,KBAR,KSIGN,
     1                 RW00,RW13,RW24)
      implicit none
C Entry for a real W
C
C---------------------------------------------------------------------
C      Polarization Summed, Squared Matrix Element Evaluations
C       for 4 Quarks + 0 or 1 Gluon + W(--> l neu) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Random Polarization Selection;
C               'S' for Direct Sum over Polarizations.
C        R    ---  Input Random Number to Specify the Polarizations
C                (Relevant only when Mode = 'R')
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities   of 4  Quarks
C        GPOL ---  Physical Polarization of the Gluon
C        LPOL ---  Physical Helicities   of 2 Leptons
C        VPOL ---  Physical Polarization of the  real W
C      OUTPUT:
C        RW00, RW13, RW24
C              ---  Spin & Color Summed, Squared Matrix Elements.
C                   The meaning of the 3 is the same as in entries
C                   RWQQQQ and RW4Q1G of the subroutines RVQQQQ and
C                   RV4Q1G except that here all polarizations
C                   are summed over (not averaged!).
C
        CHARACTER*1 MODE
        real*8  PBAR(0:3,4), QBAR(0:4), KBAR(0:4), LBAR(0:3,2),
     1          R, RW00, RW13, RW24
        INTEGER FSIGN(4), GSIGN, KSIGN, LSIGN(2),
     1          LFLAVR(2), BOS
C
        real*8  RN, PERXW, PERXV, WGT, W00, W13, W24
      INTEGER FPOL(4), GPOL, VPOL, LPOL(2), NG, I, IQ, II
C  Percentage Wgt for Quark Helicity configurations. For random helicity
C  selection a fraction PERXW of all events will be generated with
C  helicities which are possible also for nonidentical flavors. 1-PERXW
C  is reserved for helicity combinations particular to the identical
C  flavor case.
C
      DATA PERXW  /0.7d0/
      SAVE PERXW
C      COMMON /PERTG/ PERXW, PERXV
C
C---------------------------------------------------------------------
C ENTRY FOR A VIRTUAL W
      ENTRY SLW4Q(MODE,R,PBAR,FSIGN,BOS,LBAR,LSIGN,LFLAVR,
     1            RW00,RW13,RW24)
C
C---------------------------------------------------------------------
      NG  = 0
      GOTO  5
C---------------------------------------------------------------------
C ENTRY FOR A REAL W + GLUON
      ENTRY SRW4QG(MODE,R,PBAR,FSIGN,
     1             QBAR,GSIGN,BOS,KBAR,KSIGN,
     2             RW00,RW13,RW24)
C
C---------------------------------------------------------------------
C ENTRY FOR A VIRTUAL W + GLUON
      ENTRY  SLW4QG(MODE,R,PBAR,FSIGN,
     1              QBAR,GSIGN,BOS,LBAR,LSIGN,LFLAVR,
     2              RW00,RW13,RW24)
C
C---------------------------------------------------------------------
      NG   = 1
 5    CONTINUE
      RW00 = 0.0d0
      RW13 = 0.0d0
      RW24 = 0.0d0
C
      IF (MODE.EQ.'R') THEN
C Monte Carlo Summation for Polarizations:
        IF (R .LT. 0.0d0 .OR. R .GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in SRW4QG:', R
           RETURN
        ENDIF
        IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
           RN   = 3.0d0 * R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
        ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
           DO 10 I = 1, 2
              LPOL(I)  = -LSIGN(I)
 10        CONTINUE
           WGT  = 1.0d0
           RN   = R
        ELSE
           PRINT *, ' Wrong BOS assignment in SRW4Q', BOS
        END IF
C
C Gluon polarization:
        IF (NG.EQ.1) THEN
           RN  = 2D0 * RN
           IF (RN .GT. 1.0d0 ) THEN
              GPOL = 1
              RN   = RN - 1.0d0
           ELSE
              GPOL = 2
           END IF
           WGT = 2D0 * WGT
        END IF
C
        IF (RN .LE. 0.5d0 * PERXW) THEN
           FPOL(1)  = -FSIGN(1)
           FPOL(2)  = -FSIGN(2)
           FPOL(3)  = -FSIGN(3)
           FPOL(4)  = -FSIGN(4)
           WGT  = 2D0 * WGT/PERXW
        ELSE IF (RN .LE. PERXW) THEN
           FPOL(1)  = -FSIGN(1)
           FPOL(2)  = -FSIGN(2)
           FPOL(3)  =  FSIGN(3)
           FPOL(4)  =  FSIGN(4)
           WGT  = 2D0 *WGT/PERXW
        ELSE
           RN   = 2D0 *(RN - PERXW)/(1.0d0 -PERXW)
           IF ( RN .LE. 1.0d0 ) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           END IF
           WGT  = 2D0 *WGT/(1.0d0 - PERXW)
        END IF
C
        IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
           IF (NG.EQ.0) THEN
              CALL RWQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,KBAR,KSIGN,VPOL,RW00,RW13,RW24)
           ELSE IF (NG.EQ.1) THEN
              CALL RW4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,RW00,RW13,RW24)
           ELSE
              PRINT *, ' Wrong NG Assignment in SRW4Q: ', NG
           END IF
        ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
           IF (NG.EQ.0) THEN
              CALL LWQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,LBAR,LSIGN,LPOL,LFLAVR,RW00,RW13,RW24)
           ELSE IF (NG.EQ.1) THEN
              CALL LW4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,RW00,RW13,RW24)
           ELSE
              PRINT *, ' Wrong NG assignment in SLW4Q: ', NG
           END IF
        END IF
C
        RW00 = WGT*RW00
        RW13 = WGT*RW13
        RW24 = WGT*RW24
        RETURN
C
      ELSE IF (MODE.EQ.'S') THEN
C
C  Direct Summation over Polarizations:
        IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
           LPOL(1) = -LSIGN(1)
           LPOL(2) = -LSIGN(2)
        ENDIF
C
        FPOL(1) = -FSIGN(1)
        FPOL(2) = -FSIGN(2)
        DO 20 IQ   = -1, 1, 2
           FPOL(3) = IQ*FSIGN(3)
           FPOL(4) = IQ*FSIGN(4)
           IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
              DO 30 VPOL = 1, 3
                 IF (NG.EQ.0) THEN
                    CALL RWQQQQ(PBAR,FSIGN,FPOL,
     1                      BOS,KBAR,KSIGN,VPOL,W00,W13,W24)
                    RW00 = RW00 + W00
                    RW13 = RW13 + W13
                    RW24 = RW24 + W24
                 ELSE IF (NG.EQ.1) THEN
                    DO 31 GPOL = 1,2
                       CALL RW4Q1G(PBAR,FSIGN,FPOL,
     1                             QBAR,GSIGN,GPOL,
     2                         BOS,KBAR,KSIGN,VPOL,W00,W13,W24)
                       RW00 = RW00 + W00
                       RW13 = RW13 + W13
                       RW24 = RW24 + W24
31                  CONTINUE
                 END IF
30            CONTINUE
           ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
              IF (NG.EQ.0) THEN
                 CALL LWQQQQ(PBAR,FSIGN,FPOL,
     1                   BOS,LBAR,LSIGN,LPOL,LFLAVR,W00,W13,W24)
                 RW00 = RW00 + W00
                 RW13 = RW13 + W13
                 RW24 = RW24 + W24
              ELSE IF (NG.EQ.1) THEN
                 DO 32 GPOL = 1,2
                    CALL LW4Q1G(PBAR,FSIGN,FPOL,
     1                          QBAR,GSIGN,GPOL,
     2                 BOS,LBAR,LSIGN,LPOL,LFLAVR,W00,W13,W24)
                    RW00 = RW00 + W00
                    RW13 = RW13 + W13
                    RW24 = RW24 + W24
32               CONTINUE
              END IF
           END IF
20      CONTINUE
C Identical Flavor Channels:
        DO 40 II = 1, 2
           IF (II.EQ.1) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ENDIF
           IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
              DO 50 VPOL = 1, 3
                 IF (NG.EQ.0) THEN
                    CALL RWQQQQ(PBAR,FSIGN,FPOL,
     1                      BOS,KBAR,KSIGN,VPOL,W00,W13,W24)
                    RW13 = RW13 + W13
                    RW24 = RW24 + W24
                 ELSE IF (NG.EQ.1) THEN
                    DO 51 GPOL = 1,2
                       CALL RW4Q1G(PBAR,FSIGN,FPOL,
     1                             QBAR,GSIGN,GPOL,
     2                         BOS,KBAR,KSIGN,VPOL,W00,W13,W24)
                       RW13 = RW13 + W13
                       RW24 = RW24 + W24
51                  CONTINUE
                 END IF
50            CONTINUE
           ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
              IF (NG.EQ.0) THEN
                 CALL LWQQQQ(PBAR,FSIGN,FPOL,
     1                   BOS,LBAR,LSIGN,LPOL,LFLAVR,W00,W13,W24)
                 RW13 = RW13 + W13
                 RW24 = RW24 + W24
              ELSE IF (NG.EQ.1) THEN
                 DO 52 GPOL = 1,2
                    CALL LW4Q1G(PBAR,FSIGN,FPOL,
     1                          QBAR,GSIGN,GPOL,
     2                 BOS,LBAR,LSIGN,LPOL,LFLAVR,W00,W13,W24)
                    RW13 = RW13 + W13
                    RW24 = RW24 + W24
52               CONTINUE
              END IF
           END IF
40      CONTINUE
C
      ELSE
           PRINT *, ' Wrong MODE assignment in HSRW4Q: ', MODE
      END IF
C
      RETURN
      END
C
C********************  SRV4Q,SLV4Q, SRV4QG,SLV4QG **********************
C
      SUBROUTINE SRV4Q(MODE,R,PBAR,FSIGN,BOS,KBAR,KSIGN,
     1                 RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
      implicit none
C Entry for a real Z/photon
C
C---------------------------------------------------------------------
C      Polarization Summed, Squared Matrix Element Evaluations
C       for 4 Quarks + 0 or 1 Gluon + Z/photon(--> l lbar) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Random Polarization Selection;
C               'S' for Direct Sum over Polarizations.
C        R    ---  Input Random Number to Specify the Polarizations
C                (Relevant only when Mode = 'R')
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities   of 4  Quarks
C        GPOL ---  Physical Polarization of the Gluon
C        LPOL ---  Physical Helicities   of 2 Leptons
C        VPOL ---  Physical Polarization of the  real Z/photon
C      OUTPUT:
C        RVUUUU(2),RVUUCC(2), ... , RVDDDD(2)
C              ---  Spin & Color Summed, Squared Matrix Elements.
C                   The meaning of the 6 is the same as in the
C                   subroutines RVQQQQ and RV4Q1G except that here
C                   all polarizations are summed over (not averaged!).
C
        CHARACTER*1 MODE
        real*8  PBAR(0:3,4), QBAR(0:4), KBAR(0:4), LBAR(0:3,2), R,
     1        RVUUUU(2), RVUUCC(2), RVUUDD(2),
     2        RVDDUU(2), RVDDSS(2), RVDDDD(2)
        INTEGER FSIGN(4), GSIGN, KSIGN, LSIGN(2),
     1          LFLAVR(2), BOS
C
        real*8  RN, PERXW, PERXV, WGT,
     1        VUUUU(2), VUUCC(2), VUUDD(2),
     2        VDDUU(2), VDDSS(2), VDDDD(2)
      INTEGER FPOL(4), GPOL, VPOL, LPOL(2), NG, I, IQ, II, IL
C  Percentage Wgt for Quark Helicity configurations. For random helicity
C  selection a fraction PERXW of all events will be generated with helic
C  helicitieswhich are possible also for nonidentical flavors. 1-PERXW
C  is reserved for helicity combinations particular to the identical
C  flavor cases.
C
      DATA PERXV  /0.7d0/
      SAVE PERXV
C      COMMON /PERTG/ PERXW, PERXV
C
C---------------------------------------------------------------------
C Entry for a virtual Z/photon
      ENTRY SLV4Q(MODE,R,PBAR,FSIGN,BOS,LBAR,LSIGN,LFLAVR,
     1            RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
C
C---------------------------------------------------------------------
      NG  = 0
      GOTO  5
C---------------------------------------------------------------------
C ENTRY FOR A REAL Z/PHOTON + GLUON
      ENTRY SRV4QG(MODE,R,PBAR,FSIGN,
     1             QBAR,GSIGN,BOS,KBAR,KSIGN,
     2             RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
C
C---------------------------------------------------------------------
C ENTRY FOR A VIRTUAL Z/PHOTON + GLUON
      ENTRY  SLV4QG(MODE,R,PBAR,FSIGN,
     1              QBAR,GSIGN,BOS,LBAR,LSIGN,LFLAVR,
     2              RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
C
C---------------------------------------------------------------------
      NG   = 1
5     CONTINUE
      DO 6 I = 1,2
         RVUUUU(I) = 0.0d0
         RVUUCC(I) = 0.0d0
         RVUUDD(I) = 0.0d0
         RVDDUU(I) = 0.0d0
         RVDDSS(I) = 0.0d0
         RVDDDD(I) = 0.0d0
6     CONTINUE
C
C Random (Monte Carlo) summation over Polarizations:
      IF (MODE.EQ.'R') THEN
        IF (R .LT. 0.0d0 .OR. R .GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in SRV4QG: ', R
           RETURN
        ENDIF
C Real Photon:
        IF (BOS.EQ.1) THEN
           RN   = 2D0 *R
           VPOL = MIN0((1 + INT(RN)), 2)
           WGT  = 2D0
           RN   = RN + 1 - VPOL
C Real Z:
        ELSE IF (BOS.EQ.2) THEN
           RN   = 3.0d0 *R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
C Virtual gamma & Z :
        ELSE IF (BOS .EQ. 10) THEN
           IF (LFLAVR(1)/10 .EQ. 1) THEN
              RN = R
              WGT = 1.0d0
              LPOL(1) = -LSIGN(1)
              LPOL(2) = -LSIGN(2)
           ELSE
              RN  = 2D0 *R
              IF (RN .GT. 1.0d0 ) THEN
                 LPOL(1) =  LSIGN(1)
                 LPOL(2) =  LSIGN(2)
                 RN = RN -  1.0d0
              ELSE IF(RN .LE. 1.0d0 ) THEN
                 LPOL(1) = -LSIGN(1)
                 LPOL(2) = -LSIGN(2)
              END IF
              WGT = 2D0
           ENDIF
        ELSE
           PRINT *, ' Wrong BOS Assignment in SRV4Q: ', BOS
           RETURN
        END IF
C
C Gluon polarization:
        IF (NG.EQ.1) THEN
           RN  = 2D0 *RN
           IF (RN .GT. 1.0d0) THEN
              GPOL = 1
              RN   = RN - 1.0d0
           ELSE
              GPOL = 2
           END IF
           WGT = 2D0 * WGT
        END IF
C
C  Quark polarizations:
        IF (RN .LE. PERXV) THEN
           RN = 4.0d0 * RN/PERXV
           IF (RN.LE.1) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (RN.LE.2) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE IF (RN.LE.3) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ENDIF
           WGT  = 4.0d0 *WGT/PERXV
        ELSE
           RN = 2D0 * (RN-PERXV)/(1.0d0 -PERXV)
           IF (RN.LE.1) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (RN.LE.2) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE
              PRINT*,' WRONG RN IN SV4Q: ',RN
           ENDIF
           WGT  = 2D0 *WGT/(1.0d0 -PERXV)
        ENDIF
C
        IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
           IF (NG.EQ.0) THEN
              CALL RVQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,KBAR,KSIGN,VPOL,
     2                RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
           ELSE IF (NG.EQ.1) THEN
              CALL RV4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,
     3                RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
           END IF
        ELSE IF (BOS.EQ.10) THEN
           IF (NG.EQ.0) THEN
              CALL LVQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2                RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
           ELSE IF (NG.EQ.1) THEN
              CALL LV4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3                RVUUUU,RVUUCC,RVUUDD,RVDDUU,RVDDSS,RVDDDD)
           END IF
        END IF
        DO 10 I = 1,2
           RVUUUU(I) = RVUUUU(I)*WGT
           RVUUCC(I) = RVUUCC(I)*WGT
           RVUUDD(I) = RVUUDD(I)*WGT
           RVDDUU(I) = RVDDUU(I)*WGT
           RVDDSS(I) = RVDDSS(I)*WGT
           RVDDDD(I) = RVDDDD(I)*WGT
10      CONTINUE
C
      ELSE IF (MODE.EQ.'S') THEN
C
C  Direct Summation over Polarizations:
         DO 20 II = 1,6
           IF (II.EQ.1) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (II.EQ.2) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE IF (II.EQ.3) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (II.EQ.4) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE IF (II.EQ.5) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (II.EQ.6) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           END IF
           IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
             DO 30 VPOL = 1, BOS+1
               IF (NG.EQ.0) THEN
                  CALL RVQQQQ(PBAR,FSIGN,FPOL,
     1                    BOS,KBAR,KSIGN,VPOL,
     2                    VUUUU,VUUCC,VUUDD,VDDUU,VDDSS,VDDDD)
                  DO 31 I = 1,2
                    RVUUUU(I) = RVUUUU(I) + VUUUU(I)
                    RVUUCC(I) = RVUUCC(I) + VUUCC(I)
                    RVUUDD(I) = RVUUDD(I) + VUUDD(I)
                    RVDDUU(I) = RVDDUU(I) + VDDUU(I)
                    RVDDSS(I) = RVDDSS(I) + VDDSS(I)
                    RVDDDD(I) = RVDDDD(I) + VDDDD(I)
31                CONTINUE
               ELSE IF (NG.EQ.1) THEN
                  DO 32 GPOL = 1,2
                     CALL RV4Q1G(PBAR,FSIGN,FPOL,
     1                           QBAR,GSIGN,GPOL,
     2                       BOS,KBAR,KSIGN,VPOL,
     3                       VUUUU,VUUCC,VUUDD,VDDUU,VDDSS,VDDDD)
                     DO 32 I = 1,2
                       RVUUUU(I) = RVUUUU(I) + VUUUU(I)
                       RVUUCC(I) = RVUUCC(I) + VUUCC(I)
                       RVUUDD(I) = RVUUDD(I) + VUUDD(I)
                       RVDDUU(I) = RVDDUU(I) + VDDUU(I)
                       RVDDSS(I) = RVDDSS(I) + VDDSS(I)
                       RVDDDD(I) = RVDDDD(I) + VDDDD(I)
32                CONTINUE
               END IF
30           CONTINUE
          ELSE IF (BOS.EQ.10) THEN
             DO 40 IL  = -1, 1, 2
               IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 40
               LPOL(1) = IL*LSIGN(1)
               LPOL(2) = IL*LSIGN(2)
               IF (NG.EQ.0) THEN
                  CALL LVQQQQ(PBAR,FSIGN,FPOL,
     1                    BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2                    VUUUU,VUUCC,VUUDD,VDDUU,VDDSS,VDDDD)
                  DO 41 I = 1,2
                    RVUUUU(I) = RVUUUU(I) + VUUUU(I)
                    RVUUCC(I) = RVUUCC(I) + VUUCC(I)
                    RVUUDD(I) = RVUUDD(I) + VUUDD(I)
                    RVDDUU(I) = RVDDUU(I) + VDDUU(I)
                    RVDDSS(I) = RVDDSS(I) + VDDSS(I)
                    RVDDDD(I) = RVDDDD(I) + VDDDD(I)
41                CONTINUE
               ELSE IF (NG.EQ.1) THEN
                  DO 42 GPOL = 1,2
                     CALL LV4Q1G(PBAR,FSIGN,FPOL,
     1                           QBAR,GSIGN,GPOL,
     2                       BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3                       VUUUU,VUUCC,VUUDD,VDDUU,VDDSS,VDDDD)
                     DO 42 I = 1,2
                       RVUUUU(I) = RVUUUU(I) + VUUUU(I)
                       RVUUCC(I) = RVUUCC(I) + VUUCC(I)
                       RVUUDD(I) = RVUUDD(I) + VUUDD(I)
                       RVDDUU(I) = RVDDUU(I) + VDDUU(I)
                       RVDDSS(I) = RVDDSS(I) + VDDSS(I)
                       RVDDDD(I) = RVDDDD(I) + VDDDD(I)
42                CONTINUE
               END IF
40           CONTINUE
          END IF
20      CONTINUE
      ELSE
        PRINT *, ' Wrong MODE assignment in HSRW4Q: ', MODE
      END IF
C
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE DECAYC(LBAR,LSIGN,LPOL,LFLAVR,
     1                  NF,SIGMA,BOS,
     2                  JVLL,CORFAC,K)
      implicit none
C
C  DECAYC calculates the "decay current" of Hagiwara, Zeppenfeld,
C    Nucl.Phys.B313 (1989) 560, eq. 2.20
C
C  INPUT:
C  ------
C
C  LBAR(0:3,2), LSIGN(2), LPOL(2), LFLAVR(2), NF, SIGMA(NF), BOS
C  are all explained in, e.g. subroutine VQQG
C
C  OUTPUT:
C  -------
C
C  JVLL(0:3)         the decay current as defined in eq.2.20 without
C                    coupling constants and the BOS-propagator(s),
C                    which are stored as the correction factor
C
C  CORFAC(NF,3:4,2)  for the incoming fermion lines (odd NF). For
C                    Z/photon the second index corresponds to coupling
C                    to a 3=u quark or 4=d quark. Third index = 1 are
C                    the normal couplings while third index = 2 contains
C                    the appropriate couplings for the charge
C                    conjugation related process (chiralities reversed
C                    for leptons and quarks).
C                    Current conservation is used to eliminate the time
C                    component of the decay current JVLL:
C                    A term proportional to the 4 momentum of the
C                    virtual boson is subtracted. This procedure is only
C                    valid for photons or MASSLESS FERMIONS.
C
C  K(0:4)            virtual boson momentum. k**2 is stored in K(4)
C
      INTEGER NF,  LSIGN(2), LPOL(2), LFLAVR(2), RHO(2),i,j,hl
      INTEGER SIGMA(NF), BOS, NSIGN, IF1, QFL, LFL
      complex*16  JVLL(0:3), CORFAC(NF,3:4,2), CHIL(2,2), ZJ0,ZJ1,ZJ2,ZJ3
      complex*16  Z1, Z2, Z3, Z4, ZFAC, PROPW, PROPZ(2)
      real*8  LBAR(0:3,2), K(0:4), OMFACL, PROPPH, A1, A2
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
      logical ldebug
      parameter (ldebug=.false.)
C
C  Get V 4-momentum
      K(0) = LSIGN(2)*LBAR(0,2) - LSIGN(1)*LBAR(0,1)
      K(1) = LSIGN(2)*LBAR(1,2) - LSIGN(1)*LBAR(1,1)
      K(2) = LSIGN(2)*LBAR(2,2) - LSIGN(1)*LBAR(2,1)
      K(3) = LSIGN(2)*LBAR(3,2) - LSIGN(1)*LBAR(3,1)
      K(4) = K(0)**2 - K(1)**2-K(2)**2-K(3)**2
C
C  Calculate the 4 vector <l2| (sigma^mu) |l1>
      RHO(1) = LSIGN(1)*LPOL(1)
      RHO(2) = LSIGN(2)*LPOL(2)
      CALL CHIOM(2,LBAR, RHO,CHIL,OMFACL)
      Z1 = CHIL(1,2) * CHIL(1,1)
      Z2 = CHIL(2,2) * CHIL(2,1)
      Z3 = CHIL(1,2) * CHIL(2,1)
      Z4 = CHIL(2,2) * CHIL(1,1)
      IF (RHO(1).EQ.1) THEN
        ZJ0 = (Z1 + Z2)/K(0)
      ELSE
        ZJ0 = -(Z1 + Z2)/K(0)
      ENDIF
      ZJ1 = Z3 + Z4                   - ZJ0*K(1)
      ZJ2 = (Z3 - Z4)*(0.0d0, -1.0d0) - ZJ0*K(2)
      ZJ3 = Z1 - Z2                   - ZJ0*K(3)
C  Overall sign factor:
      NSIGN = LSIGN(1)*LSIGN(2)*RHO(1)
      OMFACL = OMFACL*NSIGN
      JVLL(0) = 0.0d0
      JVLL(1) = OMFACL*ZJ1
      JVLL(2) = OMFACL*ZJ2
      JVLL(3) = OMFACL*ZJ3
C  Get correction factors. First calculate V propagator factors
      LFL = LFLAVR(1)/10
      IF (BOS.EQ.10) THEN
        PROPPH = CLR(LFL,1,RHO(1))/K(4)
        A1 = K(4)-XM2(2)
        A2 = MAX(K(4),0D0)*XMG(2)/XM2(2)
        PROPZ(1) = CLR(LFL,2,RHO(1))/dcmplx( A1,A2 )
        PROPZ(2) = CLR(LFL,2,-RHO(1))/dcmplx( A1,A2 )
      ELSEIF (BOS.EQ.30.OR.BOS.EQ.40) THEN
        A1 = K(4)-XM2(3)
        A2 = MAX(K(4),0D0)*XMG(3)/XM2(3)
        PROPW = CLR(LFL,3,RHO(1))/dcmplx( A1,A2 )
      ELSE
        WRITE(*,*) ' Complaint from DECCUR: ',
     1  '  What kind of boson is THAT??   BOS = ',BOS,' ) '
        RETURN
      ENDIF
      if (ldebug) then
         print*,' '
         print*,' DECAYC info: bos = ',bos,' propph = ',propph
         print*,' propz = ',propz
         print*,' K = ',K
         do i=1,4
            do j=1,5
               print*,' CLR(',i,j,'*) = ',(clr(i,j,hl),hl=-1,1)
            enddo
         enddo
         print*,' LFL = ',LFL,' lflavr = ',lflavr
         print*,' lsign = ',lsign,' rho = ',rho,' lpol = ',lpol
      endif
C  Combine things for complete correction factors
      DO 10 IF1 = 1,NF,2
        DO 10 QFL = 3,4
          IF (BOS.EQ.10) THEN
            CORFAC(IF1,QFL,1) =
     1                  CLR(QFL,1,SIGMA(IF1))*PROPPH +
     2                  CLR(QFL,2,SIGMA(IF1))*PROPZ(1)
            CORFAC(IF1,QFL,2) =
     1                  CLR(QFL,1,-SIGMA(IF1))*PROPPH +
     2                  CLR(QFL,2,-SIGMA(IF1))*PROPZ(2)
          ELSE
            CORFAC(IF1,QFL,1) = CLR(QFL,3,SIGMA(IF1))*PROPW
            CORFAC(IF1,QFL,2) = CLR(QFL,3,-SIGMA(IF1))*PROPW
          ENDIF
10    CONTINUE
      RETURN
      END
c
c****************  decayn  *****************************************
c
C     This subroutine is a variant of DECAYC for massive leptons
c     in the decay current. It calculates the neutral electroweak 
C     current of electrons coupling to gamma, Z. It includes a finite
c     electron mass and assumes that the hadronic current it couples
c     to is conserved (valid for massless quarks) in order to 
c     suppress the 1/t**2 singularity for photon exchange: a piece
C     proportional to the 4-mometum of the virtual gauge boson k_mu
C     is subtracted to make the time component of JVLL zero.
c
c     Adam Duff
c     1991 February 12
c     
c
      subroutine decayn( lbar,	      !in: lepton 4-momenta
     +                   lsign,	      !in: lepton sign
     +                   lpol,	      !in: lepton helicity
     +                   lflavr,      !in: lepton flavour
     +                   nf,	      !in: number of fermions
     +                   sigma,	      !in: helicity of fermion lines
     +                   jvll,	      !out: decay current
     +                   k )	      !out: virtual boson 4-momentum
c
      implicit none
      integer nf
      complex*16 jvll(0:3,nf,3:4,2,-1:1)
      complex*16 chil(-1:1,2), chir(-1:1,2)
      complex*16 z11, z12, z21, z22
      complex*16 zj0(-1:1), zj1(-1:1), zj2(-1:1), zj3(-1:1)
      complex*16 propz, corfac
      integer lsign(2), lpol(2), lflavr(2), sigma(nf), cfac
      integer rho(2), rho1, rho2, isigf, ialpha, ivqq, inf, imu, lfl
      real*8 lbar(0:3,2), k(0:4)
      real*8 p1abs, p2abs, opp1, opp2, omp1, omp2, p2apz, p1apz
      real*8 me, eps, norml, normr, e1p, e1m, e2p, e2m, k0p, k0m
      real*8 ofc, propp
      double precision CLR, XM2, XMG
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6)
c
      parameter( me=5.109991d-4 )     !electron mass in GeV
      parameter( eps=1.0d-30 )	      !a carefully selected small number
c
      k(0) = lsign(2) * lbar(0,2) - lsign(1) * lbar(0,1)
      k(1) = lsign(2) * lbar(1,2) - lsign(1) * lbar(1,1)
      k(2) = lsign(2) * lbar(2,2) - lsign(1) * lbar(2,1)
      k(3) = lsign(2) * lbar(3,2) - lsign(1) * lbar(3,1)
c
      rho(1) = lsign(1) * lpol(1)
      rho(2) = lsign(2) * lpol(2)
      p1abs = sqrt( lbar(1,1)**2 + lbar(2,1)**2 + lbar(3,1)**2 )
      p2abs = sqrt( lbar(1,2)**2 + lbar(2,2)**2 + lbar(3,2)**2 )
      opp1 = sqrt( lbar(0,1) + p1abs )
      opp2 = sqrt( lbar(0,2) + p2abs )
      omp1 = me / opp1
      omp2 = me / opp2
c
      if ( lbar(3,2) .ge. 0.0d0 ) then
         p2apz = p2abs + lbar(3,2)
      else
         p2apz = ( lbar(1,2)**2 + lbar(2,2)**2 ) / ( p2abs - lbar(3,2) )
      end if
      if ( lbar(3,1) .ge. 0.0d0 ) then
         p1apz = p1abs + lbar(3,1)
      else
         p1apz = ( lbar(1,1)**2 + lbar(2,1)**2 ) / ( p1abs - lbar(3,1) )
      end if
c
      if ( p2apz .le. eps * p2abs ) then
         chil(-1,1) = dcmplx( -1.0d0, 0.0d0 )
         chil(-1,2) = dcmplx( 0.0d0, 0.0d0 )
         chil(1,1) = dcmplx( 0.0d0, 0.0d0 )
         chil(1,2) = dcmplx( 1.0d0, 0.0d0 )
      else
         norml = 1.0d0 / sqrt( 2.0d0 * p2abs * p2apz )
         chil(-1,1) = dcmplx( -norml * lbar(1,2), -norml * lbar(2,2) )
         chil(-1,2) = dcmplx( norml * p2apz, 0.0d0 )
         chil(1,1) = dcmplx( norml * p2apz, 0.0d0 )
         chil(1,2) = dcmplx( norml * lbar(1,2), -norml * lbar(2,2) )
      end if
      if ( p1apz .le. eps * p1abs ) then
         chir(-1,1) = dcmplx( -1.0d0, 0.0d0 )
         chir(-1,2) = dcmplx( 0.0d0, 0.0d0 )
         chir(1,1) = dcmplx( 0.0d0, 0.0d0 )
         chir(1,2) = dcmplx( 1.0d0, 0.0d0 )
      else
         normr = 1.0d0 / sqrt( 2.0d0 * p1abs * p1apz )
         chir(-1,1) = dcmplx( -normr * lbar(1,1), normr * lbar(2,1) )
         chir(-1,2) = dcmplx( normr * p1apz, 0.0d0 )
         chir(1,1) = dcmplx( normr * p1apz, 0.0d0 )
         chir(1,2) = dcmplx( normr * lbar(1,1), normr * lbar(2,1) )
      end if
c
      if ( lbar(3,1) .gt. 0.0d0 ) then
         e1p = lbar(0,1) + lbar(3,1)
         e1m = ( me**2 + lbar(1,1)**2 + lbar(2,1)**2 ) / e1p
      else
         e1m = lbar(0,1) - lbar(3,1)
         e1p = ( me**2 + lbar(1,1)**2 + lbar(2,1)**2 ) / e1m
      end if
      if ( lbar(3,2) .gt. 0.0d0 ) then
         e2p = lbar(0,2) + lbar(3,2)
         e2m = ( me**2 + lbar(1,2)**2 + lbar(2,2)**2 ) / e2p
      else
         e2m = lbar(0,2) - lbar(3,2)
         e2p = ( me**2 + lbar(1,2)**2 + lbar(2,2)**2 ) / e2m
      end if
      k0p = lsign(2) * e2p - lsign(1) * e1p
      k0m = lsign(2) * e2m - lsign(1) * e1m
      k(4) = k0p * k0m - k(1)**2 - k(2)**2
c
c	calculate the 4-vectors <l2| (sigma^mu) |l1>
c
      do 10 isigf = -1, 1, 2
         do 10 ivqq = 3, 4
            do 10 inf = 1, nf, 2
               do 10 imu = 0, 3
                  jvll(imu,inf,ivqq,1,isigf) = dcmplx( 0.0d0, 0.0d0 )
                  jvll(imu,inf,ivqq,2,isigf) = dcmplx( 0.0d0, 0.0d0 )
10    continue
c
      do 20 isigf = -1, 1, 2
         if ( lsign(1) .eq. 1 ) then   ! incoming e-
            rho1 = rho(1)
            rho2 = isigf
         else                      ! incoming e+ or outgoing e-
            rho1 = isigf
            rho2 = rho(2)
         end if
         z11 = chil(rho2,1) * chir(rho1,1)
         z12 = chil(rho2,1) * chir(rho1,2)
         z21 = chil(rho2,2) * chir(rho1,1)
         z22 = chil(rho2,2) * chir(rho1,2)
c
         zj0(1) = ( z11 + z22 ) / k(0)
         zj1(1) = z12 + z21 - zj0(1) * k(1)
         zj2(1) = ( z12 - z21 ) * ( 0.0d0, -1.0d0 ) - zj0(1) * k(2)
         zj3(1) = ( z11 * k0m - z22 * k0p ) / k(0)
         zj0(-1) = - ( z11 + z22 ) / k(0)
         zj1(-1) = z12 + z21 - zj0(-1) * k(1)
         zj2(-1) = ( z12 - z21 ) * ( 0.0d0, -1.0d0 ) - zj0(-1) * k(2)
         zj3(-1) = ( z11 * k0p - z22 * k0m ) / k(0)
c
         lfl = lflavr(1) / 10
         do 20 ialpha = -1, 1, 2
            propp = clr(lfl,1,ialpha) / k(4)
            if ( k(4) .gt. 0.0d0 ) then
               propz = clr(lfl,2,ialpha)
     +               / dcmplx( k(4) - xm2(2), k(4) * xmg(2) / xm2(2) )
            else
               propz = clr(lfl,2,ialpha)
     +               / dcmplx( k(4) - xm2(2), 0.0d0 )
            end if
c
            cfac = -ialpha
            if (lsign(1).eq.-1) cfac = cfac * rho1*(-ialpha)
            if (lsign(2).eq.-1) cfac = cfac * rho2*(-ialpha)
c
            if ( ialpha .eq. rho1 ) then
               if ( ialpha .eq. rho2 ) then
                  ofc = opp1 * opp2 * cfac
               else
                  ofc = opp1 * omp2 * cfac
               end if
            else
               if ( ialpha .eq. rho2 ) then
                  ofc = omp1 * opp2 * cfac
               else
                  ofc = omp1 * omp2 * cfac
               end if
            end if
c
            do 20 ivqq = 3, 4
               do 20 inf = 1, nf, 2
                  corfac = ofc * ( clr(ivqq,1,sigma(inf)) * propp
     +                           + clr(ivqq,2,sigma(inf)) * propz )
                  jvll(1,inf,ivqq,1,isigf) = jvll(1,inf,ivqq,1,isigf)
     +                                     + corfac * zj1(ialpha)
                  jvll(2,inf,ivqq,1,isigf) = jvll(2,inf,ivqq,1,isigf)
     +                                     + corfac * zj2(ialpha)
                  jvll(3,inf,ivqq,1,isigf) = jvll(3,inf,ivqq,1,isigf)
     +                                     + corfac * zj3(ialpha)
20    continue
c
      do 30 isigf = -1,1,2
         do 30 ivqq = 3,4
            do 30 inf = 1,nf,2
              do 30 imu = 1,3
                  jvll(imu,inf,ivqq,2,isigf) = 
     +              conjg( jvll(imu,inf,ivqq,1,isigf) )
30    continue
      return
      end


C
C***************  HCRV2Q *********************************************
C
      SUBROUTINE HCRV2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1                  BOS,KBAR,KSIGN,RV)
      implicit none
C
C---------------------------------------------------------------------
C      Helicity Summed, Squared Matrix Element Evaluations
C       for 2 Quarks + Ng Gluons + Z,gamma(--> ll) Processes
C
c      Changes made on format of output by Terrance Figy. 2-23-01
C      SPECIAL INPUT:
C        MODE --- 'R' for Randam Helicity Selection;
C               'S' for Direct Sum over Helicities.
C        R    ---  Input Random Number to Specify the Helicities
C                (Relevant only when Mode = 'R')
C        NG   ---  Number of Gluons
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities of 2  Quarks
C        GPOL ---  Physical Helicities of NG Gluons
C        VPOL ---  Physical Helicity of a real Z (gamma)
c     Output:
c     RV(ic,ifl,j3)      ic = 1,2: direct and charge conjugate process
c                        ifl = 1: vdd 
c                        ifl = 2: vuu 
c                        j3=0: color sum result of, e.g.,
c                        j3=1,2,..,6 color structures
C        LPOL ---  Physical Helicities of 2 Leptons
C
      CHARACTER*1 MODE
      INTEGER NG, FSIGN(2), GSIGN(NG), KSIGN, LSIGN(2), LFLAVR(2), BOS
      real*8  PBAR(0:3,2), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2),
     1        R, RV(2,2,0:6) 
C
      real*8  RN, HV(2,2,0:6),  WGT
      INTEGER FPOL(2), GPOL(3), VPOL, LPOL(2), I,I1,I2,I3,IQ,IL
      integer ifl,ic,j3,jn
C
C---------------------------------------------------------------------
C Entry for a Virtual Z,Gamma:
C
      ENTRY HCLV2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1             BOS,LBAR,LSIGN,LFLAVR,LPOL,RV)
     
C
C---------------------------------------------------------------------
C    
      if(ng.eq.1) then 
         jn = 1                 ! If 1 gluon.
      else
         jn = ng*(ng-1)         !  ng > 1 number of color structures
      end if

      do ic = 1,2
         do ifl = 1,2
            do j3 = 0,jn
               RV(ic,ifl,j3) = 0d0
            enddo
         enddo
      enddo

      IF (MODE .EQ. 'R') THEN
C Monte Carlo Summation for Polarizations:
        IF (R.LT. 0.0d0 .OR. R .GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in HCRV2Q: ', R
           RETURN
        ENDIF
C Real Photon:
        IF (BOS .EQ. 1) THEN
           RN   = 2D0 * R
           VPOL = MIN0((1 + INT(RN)), 2)
           WGT  = 2D0
           RN   = RN + 1 - VPOL
C Real Z:
        ELSE IF (BOS .EQ. 2) THEN
           RN   = 3.0d0 * R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
C Virtual gamma & Z :
        ELSE IF (BOS .EQ. 10) THEN
c           IF (LFLAVR(1)/10 .EQ. 1) THEN
c              RN = R
c              WGT = 1.0d0
c              LPOL(1) = -LSIGN(1)
c              LPOL(2) = -LSIGN(2)
c           ELSE
              RN  = 2D0 * R
              IF (RN .GT. 1.0d0 ) THEN
                 LPOL(1) =  LSIGN(1)
                 LPOL(2) =  LSIGN(2)
                 RN = RN -  1.0d0
              ELSE IF(RN .LE. 1.0d0 ) THEN
                 LPOL(1) = -LSIGN(1)
                 LPOL(2) = -LSIGN(2)
              END IF
              WGT = 2D0
c           ENDIF
        ELSE
           PRINT *, ' Wrong BOS Assignment in HCRV2Q:', BOS
           RETURN
        END IF
C
C   For fermions:
C
        RN  = 2D0 * RN
        IF (RN .GT. 1.0d0 ) THEN
           FPOL(1) =  FSIGN(1)
           FPOL(2) =  FSIGN(2)
           RN = RN -  1.0d0
        ELSE IF(RN .LE. 1.0d0 ) THEN
           FPOL(1) = -FSIGN(1)
           FPOL(2) = -FSIGN(2)
        END IF
        WGT = 2D0 * WGT
C   For gluons:
        DO 20 I = 1, NG
           RN  = 2D0 * RN
           IF (RN .GT. 1.0d0 ) THEN
              GPOL(I) = 1
              RN = RN - 1.0d0
           ELSE
              GPOL(I) = 2
           END IF
           WGT = 2D0 * WGT
 20      CONTINUE
C
C now call the subroutines for matrix elemenent squared at
C fixed polarizations
C
         IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
            IF (NG .EQ. 1) THEN
               CALL RVcQQG(PBAR,FSIGN,FPOL,
     1                     QBAR,GSIGN,GPOL,
     2                 BOS,KBAR,KSIGN,VPOL,HV)
            ELSE IF (NG .EQ. 2) THEN
               CALL RVcQQGG(PBAR,FSIGN,FPOL,
     1                      QBAR,GSIGN,GPOL,
     2                  BOS,KBAR,KSIGN,VPOL,HV)
            ELSE IF (NG .EQ. 3) THEN
               CALL RVc2Q3G(PBAR,FSIGN,FPOL,
     1                      QBAR,GSIGN,GPOL,
     2                  BOS,KBAR,KSIGN,VPOL,HV)
            ELSE
               PRINT *, ' Wrong NG assignment in HCRV2Q:', NG
               RETURN
            END IF
         ELSE IF (BOS .EQ. 10) THEN
            IF (NG .EQ. 1) THEN
               CALL LVcQQG(PBAR,FSIGN,FPOL,
     1                     QBAR,GSIGN,GPOL,
     2                 BOS,LBAR,LSIGN,LPOL,LFLAVR,HV)
            ELSE IF (NG .EQ. 2) THEN
               CALL LVcQQGG(PBAR,FSIGN,FPOL,
     1                      QBAR,GSIGN,GPOL,
     2                  BOS,LBAR,LSIGN,LPOL,LFLAVR,HV)
            ELSE IF (NG .EQ. 3) THEN
               CALL LVc2Q3G(PBAR,FSIGN,FPOL,
     1                      QBAR,GSIGN,GPOL,
     2                  BOS,LBAR,LSIGN,LPOL,LFLAVR,HV)
            ELSE
               PRINT *, ' Wrong NG assignment in HCRV2Q:', NG
               RETURN
            END IF
         END IF
C
         DO ic = 1,2
            do ifl = 1,2
               do j3 = 0,jn
                  RV(ic,ifl,j3) = WGT*HV(ic,ifl,j3)
               enddo
            enddo
         enddo
         RETURN
C
C  now comes the direct summation over polarizations
      ELSE IF (MODE .EQ. 'S') THEN
         IF (NG .EQ. 1) THEN
            DO 30 I1  = 1, 2
               GPOL(1) = I1
               DO 30 IQ = -1, 1, 2
                  FPOL(1) =  IQ*FSIGN(1)
                  FPOL(2) =  IQ*FSIGN(2)
                  IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
                     DO 31 VPOL = 1, BOS+1
                        CALL RVcQQG(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,KBAR,KSIGN,VPOL,HV)
                        
                        DO ic = 1,2
                           do ifl = 1,2
                              do j3 = 0,jn
                                 RV(ic,ifl,j3) = RV(ic,ifl,j3) + 
     1                                           HV(ic,ifl,j3)
                              enddo
                           enddo
                        enddo
 31                  CONTINUE
                  ELSE IF (BOS .EQ. 10) THEN
                     DO 33 IL = -1, 1, 2
                        IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 33
                        LPOL(1) =  IL*LSIGN(1)
                        LPOL(2) =  IL*LSIGN(2)
                        CALL LVcQQG(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,LBAR,LSIGN,LPOL,LFLAVR,HV)
                                                
                        do ic = 1,2
                           do ifl = 1,2
                              do j3 = 0,jn
                                 RV(ic,ifl,j3) = RV(ic,ifl,j3) + 
     1                                           HV(ic,ifl,j3)
                              enddo
                           enddo
                        enddo
 33                  CONTINUE
                  END IF
 30         CONTINUE
         ELSE IF (NG .EQ. 2) THEN
            DO 40 I1  = 1, 2
               GPOL(1) = I1
               DO 40 I2  = 1, 2
                  GPOL(2) = I2
                  DO 40 IQ = -1, 1, 2
                     FPOL(1) =  IQ*FSIGN(1)
                     FPOL(2) =  IQ*FSIGN(2)
                     IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
                        DO 41 VPOL = 1, BOS+1
                           CALL RVcQQGG(PBAR,FSIGN,FPOL,
     1                                  QBAR,GSIGN,GPOL,
     2                              BOS,KBAR,KSIGN,VPOL,HV)
                           do ic = 1,2
                              do ifl = 1,2
                                 do j3 = 0,jn
                                    RV(ic,ifl,j3) = RV(ic,ifl,j3) + 
     1                                              HV(ic,ifl,j3)
                                 enddo
                              enddo
                           enddo
 41                     CONTINUE
                     ELSE IF (BOS .EQ. 10) THEN
                        DO 43 IL = -1, 1, 2
                           IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 43
                           LPOL(1) =  IL*LSIGN(1)
                           LPOL(2) =  IL*LSIGN(2)
                           CALL LVcQQGG(PBAR,FSIGN,FPOL,
     1                                  QBAR,GSIGN,GPOL,
     2                              BOS,LBAR,LSIGN,LPOL,LFLAVR,HV)
                           do ic = 1,2
                              do ifl = 1,2
                                 do j3 = 0,jn
                                    RV(ic,ifl,j3) = RV(ic,ifl,j3) + 
     1                                              HV(ic,ifl,j3)
                                 enddo
                              enddo
                           enddo
                       
 43                     CONTINUE
                     END IF
 40         CONTINUE
         ELSE IF (NG .EQ. 3) THEN
            DO 50 I1  = 1, 2
               GPOL(1) = I1
               DO 50 I2  = 1, 2
                  GPOL(2) = I2
                  DO 50 I3  = 1, 2
                     GPOL(3) = I3
                     DO 50 IQ = -1, 1, 2
                        FPOL(1) =  IQ*FSIGN(1)
                        FPOL(2) =  IQ*FSIGN(2)
                        IF (BOS .EQ. 1 .OR. BOS .EQ. 2) THEN
                           DO 51 VPOL = 1, BOS+1
                              CALL RVc2Q3G(PBAR,FSIGN,FPOL,
     1                                     QBAR,GSIGN,GPOL,
     2                                 BOS,KBAR,KSIGN,VPOL,HV)
                              do ic = 1,2
                                 do ifl = 1,2
                                    do j3 = 0,jn
                                       RV(ic,ifl,j3) = RV(ic,ifl,j3) + 
     1                                                 HV(ic,ifl,j3)
                                    enddo
                                 enddo
                              enddo
                        
 51                        CONTINUE
                        ELSE IF (BOS .EQ. 10) THEN
                           DO 53 IL = -1, 1, 2
                             IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 53
                             LPOL(1) =  IL*LSIGN(1)
                             LPOL(2) =  IL*LSIGN(2)
                             CALL LVc2Q3G(PBAR,FSIGN,FPOL,
     1                                    QBAR,GSIGN,GPOL,
     2                                BOS,LBAR,LSIGN,LPOL,LFLAVR,HV)
                             do ic = 1,2
                                do ifl = 1,2
                                   do j3 = 0,jn
                                      RV(ic,ifl,j3) = RV(ic,ifl,j3) + 
     1                                                HV(ic,ifl,j3)
                                   enddo
                                enddo
                             enddo
                       
 53                        CONTINUE
                        END IF
 50         CONTINUE
C
         ELSE
            PRINT *, ' Wrong NG assignment in HCRV2Q:', NG
         END IF
C
      ELSE
         PRINT *, ' Wrong MODE assignment in HCRV2Q:', MODE
      END IF
C
      RETURN
      END

C********************  ScRW4Q,ScLW4Q, ScRW4QG,ScLW4QG **********************
C
      SUBROUTINE ScRW4Q(MODE,R,PBAR,FSIGN,BOS,KBAR,KSIGN,
     1                 RWc00,RWc13,RWc24)
      implicit none
C Entry for a real W
C
C---------------------------------------------------------------------
C      Polarization Summed, Squared Matrix Element Evaluations
C       for 4 Quarks + 0 or 1 Gluon + W(--> l neu) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Random Polarization Selection;
C               'S' for Direct Sum over Polarizations.
C        R    ---  Input Random Number to Specify the Polarizations
C                (Relevant only when Mode = 'R')
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities   of 4  Quarks
C        GPOL ---  Physical Polarization of the Gluon
C        LPOL ---  Physical Helicities   of 2 Leptons
C        VPOL ---  Physical Polarization of the  real W
C      OUTPUT:
C        RWc00, RWc13, RWc24
C              ---  Spin Summed, Squared Matrix Elements.
C                   The meaning of the 3 is the same as in entries
C                   RWcQQQQ and RWc4Q1G of the subroutines RVQQQQ and
C                   RV4Q1G except that here all polarizations
C                   are summed over (not averaged!).
C
        CHARACTER*1 MODE
        real*8  PBAR(0:3,4), QBAR(0:4), KBAR(0:4), LBAR(0:3,2),
     1          R, RWc00(0:4), RWc13(0:4), RWc24(0:4)
        INTEGER FSIGN(4), GSIGN, KSIGN, LSIGN(2),
     1          LFLAVR(2), BOS, i3
C
        real*8  RN, PERXW, PERXV, WGT, Wc00(0:4), Wc13(0:4), Wc24(0:4)
      INTEGER FPOL(4), GPOL, VPOL, LPOL(2), NG, I, IQ, II, NC
C  Percentage Wgt for Quark Helicity configurations. For random helicity
C  selection a fraction PERXW of all events will be generated with
C  helicities which are possible also for nonidentical flavors. 1-PERXW
C  is reserved for helicity combinations particular to the identical
C  flavor case.
C
      DATA PERXW  /0.7d0/
      SAVE PERXW
C      COMMON /PERTG/ PERXW, PERXV
C
C---------------------------------------------------------------------
C ENTRY FOR A VIRTUAL W
      ENTRY ScLW4Q(MODE,R,PBAR,FSIGN,BOS,LBAR,LSIGN,LFLAVR,
     1            RWc00,RWc13,RWc24)
C
C---------------------------------------------------------------------
      NG  = 0
      GOTO  5
C---------------------------------------------------------------------
C ENTRY FOR A REAL W + GLUON
      ENTRY ScRW4QG(MODE,R,PBAR,FSIGN,
     1             QBAR,GSIGN,BOS,KBAR,KSIGN,
     2             RWc00,RWc13,RWc24)
C
C---------------------------------------------------------------------
C ENTRY FOR A VIRTUAL W + GLUON
      ENTRY  ScLW4QG(MODE,R,PBAR,FSIGN,
     1              QBAR,GSIGN,BOS,LBAR,LSIGN,LFLAVR,
     2              RWc00,RWc13,RWc24)
C
C---------------------------------------------------------------------
      NG   = 1
 5    CONTINUE
      if(NG.eq.0) then 
         NC = 2                 !max number of color structures
      else
         NC = 4                 !max number of color structures
      endif
c            
      do i3 = 0,NC
         RWc00(i3) = 0.0d0
         RWc13(i3) = 0.0d0
         RWc24(i3) = 0.0d0
      enddo
C
      IF (MODE.EQ.'R') THEN
C Monte Carlo Summation for Polarizations:
        IF (R .LT. 0.0d0 .OR. R .GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in SRW4QG:', R
           RETURN
        ENDIF
        IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
           RN   = 3.0d0 * R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
        ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
           DO 10 I = 1, 2
              LPOL(I)  = -LSIGN(I)
 10        CONTINUE
           WGT  = 1.0d0
           RN   = R
        ELSE
           PRINT *, ' Wrong BOS assignment in ScRW4Q', BOS
        END IF
C
C Gluon polarization:
        IF (NG.EQ.1) THEN
           RN  = 2D0 * RN
           IF (RN .GT. 1.0d0 ) THEN
              GPOL = 1
              RN   = RN - 1.0d0
           ELSE
              GPOL = 2
           END IF
           WGT = 2D0 * WGT
        END IF
C
        IF (RN .LE. 0.5d0 * PERXW) THEN
           FPOL(1)  = -FSIGN(1)
           FPOL(2)  = -FSIGN(2)
           FPOL(3)  = -FSIGN(3)
           FPOL(4)  = -FSIGN(4)
           WGT  = 2D0 * WGT/PERXW
        ELSE IF (RN .LE. PERXW) THEN
           FPOL(1)  = -FSIGN(1)
           FPOL(2)  = -FSIGN(2)
           FPOL(3)  =  FSIGN(3)
           FPOL(4)  =  FSIGN(4)
           WGT  = 2D0 *WGT/PERXW
        ELSE
           RN   = 2D0 *(RN - PERXW)/(1.0d0 -PERXW)
           IF ( RN .LE. 1.0d0 ) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           END IF
           WGT  = 2D0 *WGT/(1.0d0 - PERXW)
        END IF
C
        IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
           IF (NG.EQ.0) THEN
              CALL RWcQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,KBAR,KSIGN,VPOL,RWc00,RWc13,RWc24)
           ELSE IF (NG.EQ.1) THEN
              CALL RWc4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,RWc00,RWc13,RWc24)
           ELSE
              PRINT *, ' Wrong NG Assignment in ScRW4Q: ', NG
           END IF
        ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
           IF (NG.EQ.0) THEN
              CALL LWcQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,LBAR,LSIGN,LPOL,LFLAVR,RWc00,RWc13,RWc24)
           ELSE IF (NG.EQ.1) THEN
              CALL LWc4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,RWc00,RWc13,RWc24)
           ELSE
              PRINT *, ' Wrong NG assignment in ScLW4Q: ', NG
           END IF
        END IF
C
        do i3 = 0,NC
           RWc00(i3) = WGT*RWc00(i3)
           RWc13(i3) = WGT*RWc13(i3)
           RWc24(i3) = WGT*RWc24(i3)
        enddo
c
        RETURN
C
      ELSE IF (MODE.EQ.'S') THEN
C
C  Direct Summation over Polarizations:
        IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
           LPOL(1) = -LSIGN(1)
           LPOL(2) = -LSIGN(2)
        ENDIF
C
        FPOL(1) = -FSIGN(1)
        FPOL(2) = -FSIGN(2)
        DO 20 IQ   = -1, 1, 2
           FPOL(3) = IQ*FSIGN(3)
           FPOL(4) = IQ*FSIGN(4)
           IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
              DO 30 VPOL = 1, 3
                 IF (NG.EQ.0) THEN
                    CALL RWcQQQQ(PBAR,FSIGN,FPOL,
     1                      BOS,KBAR,KSIGN,VPOL,Wc00,Wc13,Wc24)
                    do i3 = 0,NC
                       RWc00(i3) = RWc00(i3) + Wc00(i3)
                       RWc13(i3) = RWc13(i3) + Wc13(i3)
                       RWc24(i3) = RWc24(i3) + Wc24(i3)
                    enddo
                 ELSE IF (NG.EQ.1) THEN
                    DO 31 GPOL = 1,2
                       CALL RWc4Q1G(PBAR,FSIGN,FPOL,
     1                             QBAR,GSIGN,GPOL,
     2                         BOS,KBAR,KSIGN,VPOL,Wc00,Wc13,Wc24)
                       do i3 = 0,NC
                          RWc00(i3) = RWc00(i3) + Wc00(i3)
                          RWc13(i3) = RWc13(i3) + Wc13(i3)
                          RWc24(i3) = RWc24(i3) + Wc24(i3)
                       enddo
C                       
31                  CONTINUE
                 END IF
30            CONTINUE
           ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
              IF (NG.EQ.0) THEN
                 CALL LWcQQQQ(PBAR,FSIGN,FPOL,
     1                   BOS,LBAR,LSIGN,LPOL,LFLAVR,Wc00,Wc13,Wc24)
                 do i3 = 0,NC
                    RWc00(i3) = RWc00(i3) + Wc00(i3)
                    RWc13(i3) = RWc13(i3) + Wc13(i3)
                    RWc24(i3) = RWc24(i3) + Wc24(i3)
                 enddo
C                
              ELSE IF (NG.EQ.1) THEN
                 DO 32 GPOL = 1,2
                    CALL LWc4Q1G(PBAR,FSIGN,FPOL,
     1                          QBAR,GSIGN,GPOL,
     2                 BOS,LBAR,LSIGN,LPOL,LFLAVR,Wc00,Wc13,Wc24)
                    do i3 = 0,NC
                       RWc00(i3) = RWc00(i3) + Wc00(i3)
                       RWc13(i3) = RWc13(i3) + Wc13(i3)
                       RWc24(i3) = RWc24(i3) + Wc24(i3)
                    enddo
C                    
32               CONTINUE
              END IF
           END IF
20      CONTINUE
C Identical Flavor Channels:
        DO 40 II = 1, 2
           IF (II.EQ.1) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ENDIF
           IF (BOS.EQ.3 .OR. BOS.EQ.4) THEN
              DO 50 VPOL = 1, 3
                 IF (NG.EQ.0) THEN
                    CALL RWcQQQQ(PBAR,FSIGN,FPOL,
     1                      BOS,KBAR,KSIGN,VPOL,Wc00,Wc13,Wc24)
                    do i3 = 0,NC
                       RWc13(i3) = RWc13(i3) + Wc13(i3)
                       RWc24(i3) = RWc24(i3) + Wc24(i3)
                    enddo
                 ELSE IF (NG.EQ.1) THEN
                    DO 51 GPOL = 1,2
                       CALL RWc4Q1G(PBAR,FSIGN,FPOL,
     1                             QBAR,GSIGN,GPOL,
     2                         BOS,KBAR,KSIGN,VPOL,Wc00,Wc13,Wc24)
                       do i3 = 0,NC
                          RWc13(i3) = RWc13(i3) + Wc13(i3)
                          RWc24(i3) = RWc24(i3) + Wc24(i3)
                       enddo
C                      
51                  CONTINUE
                 END IF
50            CONTINUE
           ELSE IF (BOS.EQ.30 .OR. BOS.EQ.40) THEN
              IF (NG.EQ.0) THEN
                 CALL LWcQQQQ(PBAR,FSIGN,FPOL,
     1                   BOS,LBAR,LSIGN,LPOL,LFLAVR,Wc00,Wc13,Wc24)
                 do i3 = 0,NC
                     RWc13(i3) = RWc13(i3) + Wc13(i3)
                     RWc24(i3) = RWc24(i3) + Wc24(i3)
                  enddo
C                
              ELSE IF (NG.EQ.1) THEN
                 DO 52 GPOL = 1,2
                    CALL LWc4Q1G(PBAR,FSIGN,FPOL,
     1                          QBAR,GSIGN,GPOL,
     2                 BOS,LBAR,LSIGN,LPOL,LFLAVR,Wc00,Wc13,Wc24)
                    do i3 = 0,NC
                       RWc13(i3) = RWc13(i3) + Wc13(i3)
                       RWc24(i3) = RWc24(i3) + Wc24(i3)
                    enddo
C                    
52               CONTINUE
              END IF
           END IF
40      CONTINUE
C
      ELSE
           PRINT *, ' Wrong MODE assignment in HScRW4Q: ', MODE
      END IF
C
      RETURN
      END
C
C********************  ScRV4Q,ScLV4Q, ScRV4QG,ScLV4QG **********************
C
      SUBROUTINE ScRV4Q(MODE,R,PBAR,FSIGN,BOS,KBAR,KSIGN,
     1                 RVcI,RVcNI)
      implicit none
C Entry for a real Z/photon
C
C---------------------------------------------------------------------
C      Polarization Summed, Squared Matrix Element Evaluations
C       for 4 Quarks + 0 or 1 Gluon + Z/photon(--> l lbar) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Random Polarization Selection;
C               'S' for Direct Sum over Polarizations.
C        R    ---  Input Random Number to Specify the Polarizations
C                (Relevant only when Mode = 'R')
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities   of 4  Quarks
C        GPOL ---  Physical Polarization of the Gluon
C        LPOL ---  Physical Helicities   of 2 Leptons
C        VPOL ---  Physical Polarization of the  real Z/photon
C      OUTPUT:
C        RVcI(ifl1,i3),RVcNI(ifl1,ifl3,i3)
C       
C              ---  Spin summed, Squared Matrix Elements.
C                   The meaning of the 2 is the same as in the
C                   subroutines RVQQQQ and RV4Q1G except that here
C                   all polarizations are summed over (not averaged!).
C
        CHARACTER*1 MODE
        real*8  PBAR(0:3,4), QBAR(0:4), KBAR(0:4), LBAR(0:3,2), R,
     1          RVcI(-2:2,0:4),RVcNI(-2:2,-2:2,0:4)
        INTEGER FSIGN(4), GSIGN, KSIGN, LSIGN(2),
     1          LFLAVR(2), BOS
C
        real*8  RN, PERXW, PERXV, WGT,
     1        VcI(-2:2,0:4), VcNI(-2:2,-2:2,0:4)
      INTEGER FPOL(4), GPOL, VPOL, LPOL(2), NG, I, IQ, II, IL
      integer ifl1,ifl3,i3,NC
c
C  Percentage Wgt for Quark Helicity configurations. For random helicity
C  selection a fraction PERXW of all events will be generated with helic
C  helicitieswhich are possible also for nonidentical flavors. 1-PERXW
C  is reserved for helicity combinations particular to the identical
C  flavor cases.
C
      DATA PERXV  /0.7d0/
      SAVE PERXV
C      COMMON /PERTG/ PERXW, PERXV
C
C---------------------------------------------------------------------
C Entry for a virtual Z/photon
      ENTRY ScLV4Q(MODE,R,PBAR,FSIGN,BOS,LBAR,LSIGN,LFLAVR,LPOL,
     1            RVcI,RVcNI)
C
C---------------------------------------------------------------------
      NG  = 0
      GOTO  5
C---------------------------------------------------------------------
C ENTRY FOR A REAL Z/PHOTON + GLUON
      ENTRY ScRV4QG(MODE,R,PBAR,FSIGN,
     1             QBAR,GSIGN,BOS,KBAR,KSIGN,
     2             RVcI,RVcNI)
C
C---------------------------------------------------------------------
C ENTRY FOR A VIRTUAL Z/PHOTON + GLUON
      ENTRY  ScLV4QG(MODE,R,PBAR,FSIGN,
     1              QBAR,GSIGN,BOS,LBAR,LSIGN,LFLAVR,LPOL,
     2              RVcI,RVcNI)
C
C---------------------------------------------------------------------
      NG   = 1
5     CONTINUE
      if(NG.eq.0) then
         NC = 2                 ! max number of color structures
      else
         NC = 4                 ! max number of color structures
      endif
c
      do i3 = 0,NC
         do ifl1 = -2,2
            RVcI(ifl1,i3) = 0.0d0
            do ifl3 = -2,2
               RVcNI(ifl1,ifl3,i3) = 0.0d0
            enddo
         enddo
      enddo
C
C
C Random (Monte Carlo) summation over Polarizations:
      IF (MODE.EQ.'R') THEN
        IF (R .LT. 0.0d0 .OR. R .GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in SRV4QG: ', R
           RETURN
        ENDIF
C Real Photon:
        IF (BOS.EQ.1) THEN
           RN   = 2D0 *R
           VPOL = MIN0((1 + INT(RN)), 2)
           WGT  = 2D0
           RN   = RN + 1 - VPOL
C Real Z:
        ELSE IF (BOS.EQ.2) THEN
           RN   = 3.0d0 *R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
C Virtual gamma & Z :
        ELSE IF (BOS .EQ. 10) THEN
c           IF (LFLAVR(1)/10 .EQ. 1) THEN
c              RN = R
c              WGT = 1.0d0
c              LPOL(1) = -LSIGN(1)
c              LPOL(2) = -LSIGN(2)
c           ELSE
              RN  = 2D0 *R
              IF (RN .GT. 1.0d0 ) THEN
                 LPOL(1) =  LSIGN(1)
                 LPOL(2) =  LSIGN(2)
                 RN = RN -  1.0d0
              ELSE IF(RN .LE. 1.0d0 ) THEN
                 LPOL(1) = -LSIGN(1)
                 LPOL(2) = -LSIGN(2)
              END IF
              WGT = 2D0
c           ENDIF
        ELSE
           PRINT *, ' Wrong BOS Assignment in ScRV4Q: ', BOS
           RETURN
        END IF
C
C Gluon polarization:
        IF (NG.EQ.1) THEN
           RN  = 2D0 *RN
           IF (RN .GT. 1.0d0) THEN
              GPOL = 1
              RN   = RN - 1.0d0
           ELSE
              GPOL = 2
           END IF
           WGT = 2D0 * WGT
        END IF
C
C  Quark polarizations:
        IF (RN .LE. PERXV) THEN
           RN = 4.0d0 * RN/PERXV
           IF (RN.LE.1) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (RN.LE.2) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE IF (RN.LE.3) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ENDIF
           WGT  = 4.0d0 *WGT/PERXV
        ELSE
           RN = 2D0 * (RN-PERXV)/(1.0d0 -PERXV)
           IF (RN.LE.1) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (RN.LE.2) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE
              PRINT*,' WRONG RN IN ScV4Q: ',RN
           ENDIF
           WGT  = 2D0 *WGT/(1.0d0 -PERXV)
        ENDIF
C
        IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
           IF (NG.EQ.0) THEN
              CALL RVcQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,KBAR,KSIGN,VPOL,
     2                RVcI,RVcNI)
           ELSE IF (NG.EQ.1) THEN
              CALL RVc4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,
     3                RVcI,RVcNI)
           END IF
        ELSE IF (BOS.EQ.10) THEN
           IF (NG.EQ.0) THEN
              CALL LVcQQQQ(PBAR,FSIGN,FPOL,
     1                BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2                RVcI,RVcNI)
           ELSE IF (NG.EQ.1) THEN
              CALL LVc4Q1G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3                RVcI,RVcNI)
           END IF
        END IF
        do i3 = 0,NC
           do ifl1 = -2,2
              RVcI(ifl1,i3) = RVcI(ifl1,i3)*WGT
           do ifl3 = -2,2
              RVcNI(ifl1,ifl3,i3) = RVcNI(ifl1,ifl3,i3)*WGT
           enddo
        enddo
       enddo
C     
      ELSE IF (MODE.EQ.'S') THEN
C
C  Direct Summation over Polarizations:
         DO 20 II = 1,6
           IF (II.EQ.1) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (II.EQ.2) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE IF (II.EQ.3) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (II.EQ.4) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           ELSE IF (II.EQ.5) THEN
              FPOL(1)  = -FSIGN(1)
              FPOL(2)  =  FSIGN(2)
              FPOL(3)  =  FSIGN(3)
              FPOL(4)  = -FSIGN(4)
           ELSE IF (II.EQ.6) THEN
              FPOL(1)  =  FSIGN(1)
              FPOL(2)  = -FSIGN(2)
              FPOL(3)  = -FSIGN(3)
              FPOL(4)  =  FSIGN(4)
           END IF
           IF (BOS.EQ.1 .OR. BOS.EQ.2) THEN
             DO 30 VPOL = 1, BOS+1
               IF (NG.EQ.0) THEN
                  CALL RVcQQQQ(PBAR,FSIGN,FPOL,
     1                    BOS,KBAR,KSIGN,VPOL,
     2                    VcI,VcNI)
C                
                  do i3 = 0,NC
                     do ifl1 = -2,2
                        RVcI(ifl1,i3) = RVcI(ifl1,i3) + VcI(ifl1,i3)
                        do ifl3 = -2,2
                           RVcNI(ifl1,ifl3,i3) = RVcNI(ifl1,ifl3,i3)
     1                         + VcNI(ifl1,ifl3,i3)
                        enddo
                     enddo
                  enddo
               ELSE IF (NG.EQ.1) THEN
                  DO 32 GPOL = 1,2
                     CALL RVc4Q1G(PBAR,FSIGN,FPOL,
     1                           QBAR,GSIGN,GPOL,
     2                       BOS,KBAR,KSIGN,VPOL,
     3                       VcI,VcNI)
                     do i3 = 0,NC
                        do ifl1 = -2,2
                           RVcI(ifl1,i3) = RVcI(ifl1,i3) + VcI(ifl1,i3)
                           do ifl3 = -2,2
                              RVcNI(ifl1,ifl3,i3) = RVcNI(ifl1,ifl3,i3)
     1                             + VcNI(ifl1,ifl3,i3)
                           enddo
                        enddo
                     enddo 
32                continue                
                  ENDIF
30           CONTINUE
          ELSE IF (BOS.EQ.10) THEN
             DO 40 IL  = -1, 1, 2
               IF (LFLAVR(1)/10.EQ.1 .AND. IL.EQ.1) GOTO 40
               LPOL(1) = IL*LSIGN(1)
               LPOL(2) = IL*LSIGN(2)
               IF (NG.EQ.0) THEN
                  CALL LVcQQQQ(PBAR,FSIGN,FPOL,
     1                    BOS,LBAR,LSIGN,LPOL,LFLAVR,
     2                    VcI,VcNI)
                  do i3 = 0,NC
                     do ifl1 = -2,2
                        RVcI(ifl1,i3) = RVcI(ifl1,i3) + VcI(ifl1,i3)
                        do ifl3 = -2,2
                           RVcNI(ifl1,ifl3,i3) = RVcNI(ifl1,ifl3,i3)
     1                         + VcNI(ifl1,ifl3,i3)
                        enddo
                     enddo
                  enddo 
        
               ELSE IF (NG.EQ.1) THEN
                  DO 42 GPOL = 1,2
                     CALL LVc4Q1G(PBAR,FSIGN,FPOL,
     1                           QBAR,GSIGN,GPOL,
     2                       BOS,LBAR,LSIGN,LPOL,LFLAVR,
     3                       VcI,VcNI)
                     do i3 = 0,NC
                        do ifl1 = -2,2
                           RVcI(ifl1,i3) = RVcI(ifl1,i3) + VcI(ifl1,i3)
                           do ifl3 = -2,2
                              RVcNI(ifl1,ifl3,i3) = RVcNI(ifl1,ifl3,i3)
     1                            + VcNI(ifl1,ifl3,i3)
                           enddo
                        enddo
                     enddo
 42                  continue
                  END IF
 40            CONTINUE
            END IF
 20    CONTINUE
      ELSE
        PRINT *, ' Wrong MODE assignment in HScRW4Q: ', MODE
      END IF
C
      RETURN
      END
C
C****************  HcSRW2Q  *********************************************
C
      SUBROUTINE HcSRW2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1                  BOS,KBAR,KSIGN,RW)
      implicit none
C
C---------------------------------------------------------------------
C      Helicity Summed, Squared Matrix Element Evaluations
C       for 2 Quarks + Ng Gluons + W(--> l neu) Processes
C
C      SPECIAL INPUT:
C        MODE --- 'R' for Random Helicity Selection;
C               'S' for Direct Sum over Helicities.
C        R    ---  Input Random Number to Specify the Helicities
C                (Relevant only when Mode = 'R')
C        NG   ---  Number of Gluons
C      INTERMEDIATE VARIABLES:
C        FPOL ---  Physical Helicities of 2  Quarks
C        GPOL ---  Physical Helicities of NG Gluons
C        LPOL ---  Physical Helicities of 2 Leptons
C        VPOL ---  Physical Helicity of the  real W
C      OUTPUT:
C        RW(j3) ---  Spin, Squared Matrix Element
C        j3 =0 is the color sum
c        j3 = 1,2,..,6 are color stuctures
c
C  For more information see the general comments on the package
C
      CHARACTER*1 MODE
      INTEGER NG, FSIGN(2), GSIGN(NG), KSIGN, LSIGN(2), LFLAVR(2), BOS
      real*8  PBAR(0:3,2), QBAR(0:4,NG), KBAR(0:4), LBAR(0:3,2),
     1      R, RW(0:6)
C
      real*8  RN, HW(0:6), WGT
      INTEGER FPOL(2), GPOL(3), VPOL, LPOL(2), I,I1,I2,I3,j3
C
      integer jn
C---------------------------------------------------------------------
C Entry for a Virtual W:
C
      ENTRY HcSLW2Q(MODE,R,NG,PBAR,FSIGN,QBAR,GSIGN,
     1               BOS,LBAR,LSIGN,LFLAVR,RW)
C
C---------------------------------------------------------------------
      if(ng.eq.1) then 
         jn = 1                 ! if one gluon
      else
         jn = ng*(ng - 1)       ! ng >1
      endif
         
      do j3 = 0,jn
         RW(j3) = 0.0d0
      enddo
C
C For a W Coupling, Fix the Fermion Helicities:
      DO 10 I = 1, 2
         IF (BOS .EQ. 30 .OR. BOS .EQ. 40) LPOL(I)  = -LSIGN(I)
         FPOL(I)  = -FSIGN(I)
10    CONTINUE
C
      IF (MODE .EQ. 'R') THEN
C Monte Carlo Summation over Polarizations:
        IF (R.LT. 0.0d0 .OR. R.GT. 1.0d0 ) THEN
           PRINT *, ' R out of Range in HcSRW2Q: ', R
           RETURN
        ENDIF
        IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
           RN   = 3.0d0 * R
           VPOL = MIN0((1 + INT(RN)), 3)
           WGT  = 3.0d0
           RN   = RN + 1 - VPOL
        ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
           WGT  = 1.0d0
           RN   = R
        ELSE
           PRINT *, ' Wrong BOS assignment in HcSRW2Q: ', BOS
           RETURN
        END IF
C
C  get random gluon polarizations
        DO 20 I = 1, NG
           RN  = 2D0 * RN
           IF (RN .GT. 1.0d0 ) THEN
              GPOL(I) = 1
              RN = RN - 1.0d0
           ELSE
              GPOL(I) = 2
           END IF
           WGT = 2D0 * WGT
 20      CONTINUE
C
         IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
           IF (NG .EQ. 1) THEN
              CALL RWcQQG(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,KBAR,KSIGN,VPOL,HW)
           ELSE IF (NG .EQ. 2) THEN
              CALL RWcQQGG(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,HW)
           ELSE IF (NG .EQ. 3) THEN
              CALL RWc2Q3G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,KBAR,KSIGN,VPOL,HW)
           ELSE
              PRINT *, ' Wrong NG assignment in HcSRW2Q: ', NG
              RETURN
           END IF
         ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
           IF (NG .EQ. 1) THEN
              CALL LWcQQG(PBAR,FSIGN,FPOL,
     1                   QBAR,GSIGN,GPOL,
     2               BOS,LBAR,LSIGN,LPOL,LFLAVR,HW)
           ELSE IF (NG .EQ. 2) THEN
              CALL LWcQQGG(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,HW)
           ELSE IF (NG .EQ. 3) THEN
              CALL LWc2Q3G(PBAR,FSIGN,FPOL,
     1                    QBAR,GSIGN,GPOL,
     2                BOS,LBAR,LSIGN,LPOL,LFLAVR,HW)
           ELSE
              PRINT *, ' Wrong NG assignment in HcSRW2Q: ', NG
           END IF
         END IF
         do j3 = 0,jn
            RW(j3) = WGT*HW(j3)
         enddo
C
      ELSE IF (MODE .EQ. 'S') THEN
C Direct Summation for Helicities:
         IF (NG .EQ. 1) THEN
            DO 30 I1  = 1, 2
               GPOL(1) = I1
               IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
                  DO 31 VPOL = 1, 3
                     CALL RWcQQG(PBAR,FSIGN,FPOL,
     1                          QBAR,GSIGN,GPOL,
     2                      BOS,KBAR,KSIGN,VPOL,HW)
                     do j3 = 0,jn
                        RW(j3) = RW(j3) + HW(j3)
                     enddo
31                CONTINUE
               ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
                  CALL LWcQQG(PBAR,FSIGN,FPOL,
     1                       QBAR,GSIGN,GPOL,
     2                   BOS,LBAR,LSIGN,LPOL,LFLAVR,HW)
                  do j3 = 0,jn
                     RW(j3) = RW(j3) + HW(j3)
                  enddo
               END IF
30          CONTINUE
         ELSE IF (NG .EQ. 2) THEN
            DO 40 I1  = 1, 2
               GPOL(1) = I1
               DO 40 I2  = 1, 2
                  GPOL(2) = I2
                  IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
                     DO 41 VPOL = 1, 3
                        CALL RWcQQGG(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,KBAR,KSIGN,VPOL,HW)
                        do j3 = 0,jn
                           RW(j3) = RW(j3) + HW(j3)
                        enddo
41                   CONTINUE
                  ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
                     CALL LWcQQGG(PBAR,FSIGN,FPOL,
     1                           QBAR,GSIGN,GPOL,
     2                       BOS,LBAR,LSIGN,LPOL,LFLAVR,HW)
                     do j3 = 0,jn
                        RW(j3) = RW(j3) + HW(j3)
                     enddo
C                                         
                  END IF
40          CONTINUE
         ELSE IF (NG .EQ. 3) THEN
            DO 50 I1  = 1, 2
               GPOL(1) = I1
               DO 50 I2  = 1, 2
                  GPOL(2) = I2
                  DO 50 I3  = 1, 2
                     GPOL(3) = I3
                     IF (BOS .EQ. 3 .OR. BOS .EQ. 4) THEN
                        DO 51 VPOL = 1, 3
                           CALL RWc2Q3G(PBAR,FSIGN,FPOL,
     1                                 QBAR,GSIGN,GPOL,
     2                             BOS,KBAR,KSIGN,VPOL,HW)
                           do j3 = 0,jn
                               RW(j3) = RW(j3) + HW(j3)
                            enddo    
                           
51                      CONTINUE
                     ELSE IF (BOS .EQ. 30 .OR. BOS .EQ. 40) THEN
                        CALL LWc2Q3G(PBAR,FSIGN,FPOL,
     1                              QBAR,GSIGN,GPOL,
     2                          BOS,LBAR,LSIGN,LPOL,LFLAVR,HW)
                        do j3 = 0,jn
                           RW(j3) = RW(j3) + HW(j3)
                        enddo                   
                     END IF
50          CONTINUE
         ELSE
              PRINT *, ' Wrong NG assignment in HcSRW2Q: ', NG
         END IF
C
      ELSE
         PRINT *, ' Wrong MODE assignment in HcSRW2Q: ', MODE
      END IF
C
      RETURN
      END
